<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The class used for enlight frontpage custom blocks.
 *
 * @package theme_enlight
 * @copyright 2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @author LMSACE Dev Team
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace theme_enlight;
use html_writer;


/**
 * The class for display the each frontpage custom blocks.
 */
class customblocks {

    /**
     * @var array Store the js data
     */
    public $themejsdata;

    /**
     * Define constructor.
     */
    public function __construct() {
        $this->themejsdata = array();
    }

    /**
     * Define the call slider for frontpage blocks.
     * @param string $blockid block name
     * @param array $data
     * @return void
     */
    public function include_frontslide_js($blockid, $data=[]) {
        global $PAGE;
        $PAGE->requires->js_call_amd('theme_enlight/front-sliders', $blockid, $data);
    }

    /**
     * Get title block animation info.
     */
    public static function animationclass() {
        $name = 'titleanimation';
        $animation = theme_enlight_get_setting($name);
        return ($animation) ? ' animated '. $animation : '';
    }

    /**
     * Get content block animation info.
     * @param string $config
     */
    public static function blockanimation($config) {
        $animation = theme_enlight_get_setting($config);
        return ($animation) ? ' animated '. $animation : '';
    }

    /**
     * Get block animation item info.
     * @param string $block string blockname
     * @return string block class
     */
    public static function animationitems($block) {
        $items = [
            'testimonials' => '.testimonial-block',
            'ourteam' => '.card-block',
            'sitefeatures' => '.featured-block',
            'popularcourses' => '.course-item',
            'sitenews' => 'article',
            'availablecourses' => '.course-slider-item',
            'enrolledcourses' => '.mycourse-slider-item',
            'categorynews' => '.category-block',
            'categorycombo' => '.category-combo-list',
            'counters' => '.content-block',
            'ourpartners' => '.img-block',
            'search' => '#coursesearch'
        ];

        return isset($items[$block]) ? $items[$block] : '';
    }

    /**
     * Site features block.
     * @return string template|HTML.
     */
    public function sitefeatures() {
        global $OUTPUT;
        $status = theme_enlight_get_setting('togglesitefeatures');
        $items = array();
        if ($status == 1) {
            $block = $this->get_block_title('sitefblocktitle', '');
            $block['subtitle'] = theme_enlight_get_setting('sitefblocksubtitle');
            $block['description'] = theme_enlight_get_setting('sitefblockdesc', 'format_html');
            $features = theme_enlight_get_setting('numberofsitef');
            for ($i = 1; $i <= $features; $i++) {
                $status = theme_enlight_get_setting('sitefblock'.$i.'status');
                $image = theme_enlight_get_setting('sitefblock'.$i.'image', 'file');
                $link = theme_enlight_get_setting('sitefblock'.$i.'link');
                $items[] = [
                'status' => $status,
                'image' => $image,
                'title' => theme_enlight_get_setting('sitefblock'.$i.'title'),
                'description' => theme_enlight_get_setting('sitefblock'.$i.'content'),
                'link' => ($link) ? $link : 'javascript:void(0)',
                ];
            }
            $block['features'] = array_chunk($items, 3);
            $block['sitefeatures'] = 1;
            $style = theme_enlight_get_setting("sitefblockstyle");
            $block[$style] = 1;
            return $this->render_template('sitefeatures', $block);
        }
    }

    /**
     * Testimonials block.
     * @return string template|HTML.
     */
    public function testimonials() {
        global $OUTPUT;
        $toggletmonial = theme_enlight_get_setting('toggletmonial');
        if ($toggletmonial == 1) {

            // TmonialTraversal status.
            $configlist = array(
            'count' => ['numberoftmonials'],
            'autoplay' => ['testimonialAutoplay'],
            'title' => ['testimonialtitle'],
            'description' => ['testimonial_desc'],
            'pagination' => ['tmonialTraversal_status'],
            'subtitle' => ['testimonialsubtitle']
            );

            $block = $this->get_config_array($configlist);
            $block['classes'] = theme_enlight_block_backgroundclass('testimonialsbg_type');
            $block['autoplayoption'] = (theme_enlight_get_setting('testimonialAutoplay') == 1 ) ? 'carousel' : 'false';
            $itemslist = array(
            'testimonial' => ['tmonial{i}text', 'format_text'],
            'username' => ['tmonial{i}uname'],
            'userimage' => ['tmonial{i}img', 'file'],
            'userposition' => ['tmonial{i}userposition'],
            );
            $items = $this->get_items_config($itemslist, $block['count']);
            $block['items'] = array_chunk($items, 2);
            $this->include_frontslide_js('testimonials', ['autoplay' => $block['autoplay'], 'pagination' => $block['pagination']]);
            return $this->render_template('testimonials', $block);
        }

        return '';
    }

    /**
     * Counter block.
     * @return string template|HTML.
     */
    public function counter() {
        global $OUTPUT;
        $status = theme_enlight_get_setting('counterBlock');
        if ($status == 1) {
            $block = $this->get_block_title('countBlockTitle', 'countBlockDesc');
            $block['subtitle'] = theme_enlight_get_setting('countBlockSubTitle');
            $block['bgimage'] = theme_enlight_get_setting('counterbg_image', 'file');
            for ($i = 1; $i <= 4; $i++) {
                $items[] = [
                    'value' => theme_enlight_get_setting('countBlockValue'.$i),
                    'content' => theme_enlight_get_setting('countBlockContent'.$i),
                    'imgstatus' => theme_enlight_get_setting('countblockimgstatus'.$i),
                    'counterimage' => theme_enlight_get_setting('countblockimg'.$i, 'file'),
                ];
            }
            $block['items'] = $items;
            $block['counter'] = 1;
            $this->include_frontslide_js('counters');
            return $this->render_template('counter', $block);
        }
    }

    /**
     * About us block (marketing spot)
     * @return string template|html
     */
    public function marketingspot1() {
        $status = theme_enlight_get_setting('marketingSpot1_status');
        if ($status == 1) {
            $ms['title'] = theme_enlight_get_setting('mspot1title');
            $ms['subtitle'] = theme_enlight_lang(theme_enlight_get_setting('mspot1subtitle'));
            $ms['description'] = theme_enlight_get_setting('mspot1desc', 'format_html');
            $ms['media'] = theme_enlight_get_setting('mspot1media', 'file');
            $ms['classes'] = theme_enlight_block_backgroundclass('mspot1bg_type');
            $ms['descanim'] = self::blockanimation('mspot1descanim');
            $ms['mediaanim'] = self::blockanimation('mspot1mediaanim');
            $ms['class'] = '';
            if (!empty($ms['media'])) {
                $ms['class'] .= ' media-available ';
            }

            if (!empty($ms['description'])) {
                $ms['class'] .= ' description_available ';
            }
            return $this->render_template('marketingspot1', $ms);
        }
    }

    /**
     * Jumbo view (marketing spot).
     */
    public function marketingspot2() {
        global $CFG;
        $status = theme_enlight_get_setting('marketingSpot2_status');
        if ($status == 1) {
            $msp2['title'] = theme_enlight_get_setting('mspot2title', 'format_text');
            $msp2['desc'] = theme_enlight_get_setting('mspot2desc', 'format_html');
            $url = theme_enlight_get_setting('mspot2url');
            $msp2['url'] = ($url) ? $url : 'javascript:void(0);';
            $msp2['urltxt'] = theme_enlight_get_setting('mspot2urltext', 'format_text');
            $target = theme_enlight_get_setting('mspot2urltarget');
            $msp2['urltarget'] = ($target) ? "_blank" : "_self";
            $msp2['wwwroot'] = $CFG->wwwroot;
            $msp2['urlanim'] = self::blockanimation('mspot2urlanim');
            $msp2['mobileappurl'] = theme_enlight_mobile_config();
            $msp2['classes'] = theme_enlight_block_backgroundclass('mspot2bg_type');
            return $this->render_template('marketingspot2', $msp2);
        }
    }

    /**
     * Ourpartners block
     * @return string block html.
     */
    public function ourpartner() {
        $status = theme_enlight_get_setting('ourPartner_status');
        if ($status == 1) {
            $js['autotime'] = theme_enlight_get_setting('ourPartner_autotime');
            $config = array(
            'autotime' => ['ourPartner_autotime'],
            'autoplay' => ['ourPartner_autoplay'],
            'count' => ['ourPartner_number'],
            'title' => ['ourPartner_title'],
            'description' => ['ourPartner_desc'],
            'subtitle' => ['ourPartner_subtitle']
            );
            $block = $this->get_config_array($config);
            $itemslist = array(
                'title' => ['partner{i}title'],
                'img' => ['partner{i}img', 'file'],
                'link' => ['partner{i}link']
            );

            $block['partners'] = $this->get_items_config($itemslist, $block['count']);
            $this->include_frontslide_js('ourpartners', ['autoplay' => $block['autoplay'], 'autotime' => $block['autotime']]);
            return $this->render_template('ourpartner', $block);
        }
    }

    /**
     * Get the ourteam block team status.
     * @param array $nums
     * @return array team info.
     */
    public function check_our_team_status($nums) {
        foreach ($nums as $key => $value) {
            $con = $this->get_team_user($value, 'check');
            if ($con == "") {
                unset($nums[$key]);
            }
        }
        return $nums;
    }

    /**
     * This code returns the our team block contents.
     * @return string display ourteam html.
     */
    public function our_team() {
        global $OUTPUT;
        $template = [];
        $numberofpersons = theme_enlight_get_setting('numberofpersons');
        $num = range(1, $numberofpersons);
        $rows = $this->check_our_team_status($num);
        if (empty($rows)) {
            return "";
        }
        $title = theme_enlight_get_setting('ourteamTitle');
        $subtitle = theme_enlight_get_setting('ourteamSubTitle');
        $template['status'] = theme_enlight_get_setting('ourteamStatus');
        $template['title'] = theme_enlight_lang($title);
        $template['subtitle'] = theme_enlight_lang($subtitle);
        $template['description'] = theme_enlight_lang(theme_enlight_get_setting('ourteam_desc'));
        $template['ourteamAutoplay'] = theme_enlight_get_setting('ourteamAutoplay');
        $ourteamslide = [];
        foreach ($rows as $row) {
            $ourteamslide[] = $this->get_team_user($row);
        }
        $template['teams'] = $ourteamslide;
        $this->include_frontslide_js('ourteam', ['ourteamAutoplay' => $template['ourteamAutoplay']]);
        return $this->render_template('ourteam', $template);
    }

    /**
     * Get the team user info.
     * @param int $i Team number.
     * @param string $type
     * @return array user info.
     */
    public function get_team_user($i, $type = "check") {

        global $CFG, $PAGE, $OUTPUT;
        $username = theme_enlight_get_setting('team'.$i.'pname');
        $userimg = $PAGE->theme->setting_file_url('team'.$i.'img', 'team'.$i.'img');
        $userposition = theme_enlight_get_setting('team'.$i.'position');
        $userprofileurl = theme_enlight_get_setting('team'.$i.'profileUrl');

        $usersocialicon1 = theme_enlight_get_setting('team'.$i.'socialIcon1');
        $usersocialicon2 = theme_enlight_get_setting('team'.$i.'socialIcon2');
        $usersocialicon3 = theme_enlight_get_setting('team'.$i.'socialIcon3');
        $usersocialicon4 = theme_enlight_get_setting('team'.$i.'socialIcon4');

        $usersocialurl1 = theme_enlight_get_setting('team'.$i.'sIconurl1');
        $usersocialurl2 = theme_enlight_get_setting('team'.$i.'sIconurl2');
        $usersocialurl3 = theme_enlight_get_setting('team'.$i.'sIconurl3');
        $usersocialurl4 = theme_enlight_get_setting('team'.$i.'sIconurl4');

        $description = theme_enlight_get_setting('team'.$i.'description');
        $nouserimg = $OUTPUT->image_url('no-user', 'theme');

        $content = array();
        $content['username'] = (isset($username)) ? $username : "";
        $content['userPosition'] = (isset($userposition)) ? $userposition : "";
        $content['userProfileurl'] = (isset($userprofileurl)) ? $userprofileurl : "";
        $content['userSocial_icon1'] = (isset($usersocialicon1)) ? trim($usersocialicon1) : "";
        $content['userSocial_icon2'] = (isset($usersocialicon2)) ? trim($usersocialicon2) : "";
        $content['userSocial_icon3'] = (isset($usersocialicon3)) ? trim($usersocialicon3) : "";
        $content['userSocial_icon4'] = (isset($usersocialicon4)) ? trim($usersocialicon4) : "";

        $content['userSocial_url1'] = (!empty($usersocialurl1)) ? $usersocialurl1 : "#";
        $content['userSocial_url2'] = (!empty($usersocialurl2)) ? $usersocialurl2 : "#";
        $content['userSocial_url3'] = (!empty($usersocialurl3)) ? $usersocialurl3 : "#";
        $content['userSocial_url4'] = (!empty($usersocialurl4)) ? $usersocialurl4 : "#";

        $content['description'] = (isset($description)) ? $description : "";
        $content['userImg'] = (!empty($userimg)) ? $userimg : "";
        if (empty($content['username']) && empty($content['userPosition']) &&
            empty($content['description']) && empty($content['userImg'])) {
            return false;
        }
        $content['userImg'] = (!empty($userimg)) ? $userimg : $nouserimg;
        return $content;
    }

    /**
     * Add the theme js data.
     * @param string $variable
     * @param string|object|array $data
     * @return void
     */
    public function add_theme_jsdata($variable, $data) {
        $this->themejsdata[$variable] = $data;
    }

    /**
     * Get block items config.
     * @param array $configlist
     * @param int $count
     * @return array items info.
     */
    public function get_items_config($configlist=[], $count=null) {
        global $OUTPUT;
        $items = [];
        if (!empty($count)) {
            for ($i = 1; $i <= $count; $i++) {
                $block = [];
                foreach ($configlist as $key => $config) {
                    $setting = $config[0];
                    $newconfig = preg_replace("/{i}/i", $i , $setting); // Convert the config with count variable to variable.
                    $type = (isset($config[1])) ? $config[1] : '';
                    $block[$key] = theme_enlight_get_setting($newconfig, $type);
                }
                if (!empty($block['testimonial']) || !empty($block['username']) || !empty($block['userposition']) ) {
                    $enablecontent = true;
                } else {
                    $enablecontent = false;
                }
                if (isset($block['testimonial'])) {
                    $block['testimonial'] = format_text($block['testimonial'], FORMAT_HTML);
                    if (empty($block['userimage'])) {
                        $block['userimage'] = $OUTPUT->image_url('no-user', 'theme');
                    }
                }
                $block['enable_content'] = $enablecontent;
                $items[] = $block;
            }
        }
        return isset($items) ? $items : [];
    }

    /**
     * Get configs setting.
     * @param array $configlist
     * @return array configs info.
     */
    public function get_config_array($configlist=[]) {
        if (!empty($configlist)) {
            foreach ($configlist as $key => $config) {
                $type = (isset($config[1])) ? $config[1] : '';
                $block[$key] = theme_enlight_get_setting($config[0], $type);
            }
            return $block;
        }
    }

    /**
     * Get block title & desc info.
     * @param string $title
     * @param string $desc
     * @return array title & desc info.
     */
    public function get_block_title($title, $desc) {
        $return['title'] = ($title) ? theme_enlight_get_setting($title) : '';
        $return['description'] = ($desc) ? theme_enlight_get_setting($desc) : '';
        return $return;
    }

    /**
     * Customblocks render template.
     * @param string $template
     * @param array $data
     * @return string customblocks html.
     */
    public function render_template($template, $data) {
        global $OUTPUT;
        static $titleanimation;
        if (empty($titleanimation)) {
            $animation = theme_enlight_get_setting('titleanimation');
            $titleanimation = ($animation) ? ' animated '.$animation : '';
        }
        $data['titleanimation'] = ($titleanimation == 'NONE') ? '' : $titleanimation;
        $data[$template] = 1;
        $data['ouput'] = $OUTPUT;
        return $OUTPUT->render_from_template('theme_enlight/custom_blocks', $data);
    }
}
