<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The class define the homepage slider.
 *
 * @package    theme_enlight
 * @copyright  2017 Willian Mano - http://conecti.me
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace theme_enlight;

use stdclass;
use moodle_url;
use html_writer;

/**
 * Homepage slider
 */
class homeslider extends customblocks {
    /**
     * @var object
     */
    public $options;

    /**
     * @var array
     */
    public $jsdata;

    /**
     * define for Constractor.
     */
    public function __construct() {
        $this->generate_settings();
        return '';
    }

    /**
     * Get the slider html.
     * @return string homeslider html.
     */
    public function get_slider() {
        global $OUTPUT, $PAGE;
        $PAGE->requires->data_for_js('homecarouselconfig', $this->jsdata);
        $PAGE->requires->js_call_amd('theme_enlight/homecarousel', 'init', ['selector' => '#homepage-carousel']);
        $PAGE->requires->css("/theme/enlight/style/animate.css");
        return $OUTPUT->render_from_template('theme_enlight/homeslider', (array) $this->options);
    }

    /**
     * Generate the slider settings.
     */
    public function generate_settings() {
        $this->slideshow_general_config();
        $this->prepare_slide_options();
        $this->jsdata = $this->options;
    }

    /**
     * Set the slider content.
     */
    public function slideshow_general_config() {
        $generalitems = array(
            'slideshowstatus' => ['slideshowStatus'],
            'numberofslides' => ['numberofslides'],
            'autoslideshow' => ['autoslideshow'],
            'slideinterval' => ['slideinterval'],
            'slideheight' => ['slideHeight'],
            'slideshowpagination' => ['slideshowpagination'],
        );

        $general = (object) $this->get_config_array($generalitems);
        $general->slideheight = intval($general->slideheight);
        $general->slideheight = (int) !empty($general->slideheight) ? $general->slideheight : "550";
        $slideinterval = intval($general->slideinterval);
        $general->slideinterval = empty($slideinterval) ? 3000 : $slideinterval;
        if ($general->autoslideshow == 1) {
            $general->autoslideshow = 'true';
            $general->resume = $general->slideinterval;
        } else {
            $general->autoslideshow = 'false';
            $general->resume = false;
        }

        switch($general->slideshowpagination) {
            case "NONE":
                $thumbnails = "false";
                $pagination = "false";
            break;
            case "DOTTHUMB":
                $thumbnails = "true";
                $pagination = "true";
                break;
            case "THUMB" :
                $thumbnails = "true";
                $pagination = "false";
                break;
            case "DOT":
                $thumbnails = "false";
                $pagination = "true";
                break;
            default:
                $thumbnails = "true";
                $pagination = "true";
        }
        $general->thumbnails = $thumbnails;
        $general->pagination = $pagination;
        $general->hasyoutube = false;
        $this->options = $general;
    }

    /**
     * Set the slider controls.
     */
    public function prepare_slide_options() {
        global $OUTPUT;
        $sliderlevel = 0;
        $imgarr = [];
        $general = $this->options;
        if (!empty($general->numberofslides)) {
            for ($s1 = 1; $s1 <= $general->numberofslides; $s1++) {
                $status = theme_enlight_get_setting('slide'.$s1.'status');
                if ($status) {
                    $slide = $this->slideshow_slide_settings($s1);
                    $imgarr[$s1] = $slide['image'];
                    $this->options->slides[] = $slide;
                }
            }
        }
        $this->options->slidecontrol = $this->get_slide_controls($imgarr);
    }

    /**
     * Get Slider settings.
     * @param int $s
     * @return array slider content.
     */
    public function slideshow_slide_settings($s) {
        $configs = array(
            'status', 'url1', 'url2', 'type', 'videourl', 'contentPosition', 'contentAnimation', 'urltarget1',
            'urltarget2', 'contentStyle', 'effect', 'contFullwidth', 'urltext1', 'urltext2', 'caption'
        );

        foreach ($configs as $config) {
            $slide[$config] = theme_enlight_get_setting('slide'.$s.$config);
        }

        $slide['num'] = $s;
        $slide['image'] = theme_enlight_render_slideimg($s, 'slide' . $s . 'image');
        $slide['desc'] = theme_enlight_get_setting('slide'.$s.'desc', 'format_html');
        $slide['caption'] = theme_enlight_lang($slide['caption']);
        $slide['urltext1'] = theme_enlight_lang($slide['urltext1']);
        $slide['urltext2'] = theme_enlight_lang($slide['urltext2']);
        $slide['desc'] = theme_enlight_lang($slide['desc']);
        $slide['slidecontent'] = false;
        if ($slide['desc'] || $slide['caption'] || $slide['urltext1'] || $slide['urltext2']) {
            $slide['slidecontent'] = true;
        }

        $slide['imageslide'] = ($slide['type'] == 3) ? true : false;
        $slide['videoslide'] = ($slide['type'] != 3) ? true : false;
        $slide['youtube'] = ($slide['type'] == 1 && !empty($slide['videourl'])) ? true : false;
        if ($slide['youtube']) {
            $this->options->hasyoutube = true;
        }
        $slide['hostedvideo'] = ($slide['type'] == 2 && !empty($slide['videourl'])) ? true : false;
        $slide['baseclass'] = ($slide['type'] == 3) ? 'img-slider' : 'video-slide';
        $slide['maskimage'] = ( isset($slide['image']) && !empty($slide['image']) ) ?
            "background-image: url(".$slide['image'].");" : '';

        if (!empty($s)) {
            // Calculate the width of the slide content.
            $contwidth = $slide['contFullwidth'];
            if ($contwidth == "auto" || $contwidth == '') {
                $contwidth = "auto";
            } else {
                $contwidth = intval($contwidth);
                if ($contwidth > '100' ) {
                    $contwidth = '100%';
                } else if ($contwidth <= 0) {
                    $contwidth = "auto";
                } else {
                    $contwidth = $contwidth.'%';
                }
            }
            $slide['contwidth'] = $contwidth;

            // Content animation level.
            $slide['contentanimation'] = htmlspecialchars($slide['contentAnimation']);
            if ($slide['contentanimation'] != "NONE") {
                $slide['contentAclass'] = "animated ". $slide['contentanimation'];
            } else {
                $slide['contentAclass'] = "none";
            }

            // URL target method.
            if ($slide['urltarget1'] == 1) {
                $slide['btntarget1'] = "_blank";
            } else {
                $slide['btntarget1'] = "_self";
            }
            if ($slide['urltarget2'] == 1) {
                $slide['btntarget2'] = "_blank";
            } else {
                $slide['btntarget2'] = "_self";
            }
            if (!empty($slide['contentPosition'])) {
                $slide['contentClass'] = 'content-'.$slide['contentPosition'];
            } else {
                $slide['contentClass'] = "content-centerRight";
            }

            // Slide content style.
            if ($slide['contentStyle'] == '2' ) {
                $slide['overlayClass'] = "content-overlay";
            } else if ($slide['contentStyle'] == '4') {
                $slide['slideoverlay'] = "true";
                $slide['overlayClass'] = "slide-overlay";

            } else {
                $slide['overlayClass'] = "";
            }

            $slide['dataUrl'] = "";
            if (!empty($slide['url1']) && empty($slide['urltext1'])) {
                $slide['dataUrl'] = $slide['slideurl1'];
            }
            if (empty($slide['dataUrl'])) {
                if (!empty($slide['url2']) && empty($slide['urltext2'])) {
                    $slide['dataUrl'] = $slide['slideurl2'];
                }
            }

            $baseid = 'base-'.$slide['num'];
            $slide['videoproperty'] = json_encode([
                'videoURL' => $slide['videourl'],
                'containment' => '#'.$baseid,
                'autoPlay' => true,
                'mute' => true,
                'startAt' => 0,
                'opacity' => 1,
                'showControls' => false,
                'loop' => true,
                'abundance' => 0.9,
                'showYTLogo' => false,
                'coverImage' => $slide['image']
            ]);

            $this->slide_content_styleclass($slide);
        }

        return $slide;
    }

    /**
     * Style for style content.
     * @param array $slide
     * @return array slide content.
     */
    public function slide_content_styleclass(&$slide) {
        switch ($slide['contentStyle']) {
            case 1:
                $contentstyleclass = "text-item-content"; // Theme color title contents with dbl btn.
                break;
            case 2:
                $contentstyleclass = "content_overlayer";
                break;
            case 3:
                $contentstyleclass = "double-heading-content"; // Double heading.
                break;
            case 4:
                $contentstyleclass = "slide_overlay";
                break;
            default:
                $contentstyleclass = "content_overlayer";
        }
        $contentclass = "slide-text animated fadeIn ";
        $contentclass .= $slide['contentClass'] ." ";
        $contentclass .= $contentstyleclass;
        $slide['contentclass'] = $contentclass;

        if ($slide['contentStyle'] == 3) {
            $title = explode('||', $slide['caption']);
            $slide['caption'] = isset($title[0]) ? $title[0] : $slide['caption'];
            $slide['caption2'] = isset($title[1]) ? $title[1] : "";
        }
    }

    /**
     * Get the slide controls.
     * @param array $imgarrs
     * @return string slide control html.
     */
    public function get_slide_controls($imgarrs = array()) {

        $ctype = theme_enlight_get_setting('slideshowpagination');
        if ($ctype != "NONE") :
            $slidecontrol = html_writer::start_tag('span', array('class' => 'controls'));
            $slidecontrol .= html_writer::start_tag('div', array('class' => 'centeredDotsControls'));
            $s = 1;
            if (!empty($imgarrs)) {
                foreach ($imgarrs as $key => $imgarr) {
                    $class = ($ctype == "DOT" || $ctype == "DOTTHUMB") ? 'doted' : "thumb";
                    $slidecontrolclass = "homepage-carouselLink ". $class;
                    $slidecontrol .= html_writer::start_tag('a', array('class' => $slidecontrolclass, 'data-target' => $s,
                        'href' => '#'));
                    $slidecontrol .= html_writer::tag('span', null);
                    if ($ctype == "THUMB" || $ctype == 'DOTTHUMB') {
                        if (empty($imgarr)) {
                            $slidecontrol .= html_writer::tag('span', '', ['class' => 'empty-image fill-color']);
                        } else {
                            $slidecontrol .= html_writer::empty_tag('img', array('src' => $imgarr, 'height' => '50',
                                'width' => '50' ));
                        }
                    }
                    $slidecontrol .= html_writer::end_tag('a');
                    $s++;
                }
            }
            $slidecontrol .= html_writer::end_tag('div');
            $slidecontrol .= html_writer::end_tag('span');
            return $slidecontrol;
        endif;
    }
}
