<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Renderers to align Moodle's HTML with that expected by Bootstrap
 *
 * @package theme_enlight
 * @copyright 2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @author LMSACE Dev Team
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace theme_enlight\output;

use moodle_url;
use lang_string;
use html_writer;
use stdClass;
use core_course_category;


/**
 * This class has function for renderer primary menu and top course menus
 */
class core_renderer extends \theme_boost\output\core_renderer {

    /**
     * Returns the moodle_url for the favicon.
     *
     * This renderer function is copied and modified from /lib/outputrenderers.php
     *
     * @since Moodle 2.5.1 2.6
     * @return moodle_url The moodle_url for the favicon
     */
    public function favicon() {
        if (!empty($this->page->theme->settings->favicon)) {
            return $this->page->theme->setting_file_url('favicon', 'favicon');
        } else {
            return $this->image_url('favicon', 'theme');
        }
    }

    /**
     * This function have the code to create the primary menu from the settings.
     * @return string
     */
    public function primarymenu() {
        global $CFG;
        require_once($CFG->dirroot . '/lib/outputrenderers.php');
        $custommenuitems = isset($this->page->theme->settings->primarymenu) ? $this->page->theme->settings->primarymenu : "";
        $custommenu = new \custom_menu($custommenuitems, current_language());
        return $this->custom_menu_render($custommenu);
    }

    /**
     * This renders the bootstrap top menu.
     * @param custom_menu $menu
     * @return string
     */
    protected function custom_menu_render(\custom_menu $menu) {

        global $CFG;
        if (isset($this->page->theme->settings->cmenuPosition)) {
            $cmenuposition = $this->page->theme->settings->cmenuPosition;
        } else {
            $cmenuposition = "";
        }
        $cmenushow = theme_enlight_get_setting('cmenushow');
        $langs = get_string_manager()->get_list_of_translations();
        $haslangmenu = $this->lang_menu() != '';
        if (!$menu->has_children() ) {
            if ($cmenushow ) {
                return $this->course_menu();
            }
            return '';
        }
        $content = [];
        $count = count($menu->get_children());
        foreach ($menu->get_children() as $key => $item) {
            $context = $item->export_for_template($this);
            $context = $item->export_for_template($this);
            $content[] = $this->render_from_template('core/custom_menu_item', $context);
        }
        $cmenudisplay = theme_enlight_get_setting('cmenudisplay');
        $enableheader = theme_enlight_get_setting('enableheader');
        if ($cmenushow && $cmenudisplay == 'header' && $enableheader) {
            $coursemenu = $this->course_menu();
            $cmenuposition = $cmenuposition - 1;
            if ($cmenuposition) {
                if ($cmenuposition >= count($content)) {
                    array_push($content, $coursemenu);
                } else {
                    $start = array_slice($content, 0, $cmenuposition);
                    $end = array_slice($content, $cmenuposition, count($content) - 1);
                    $content = $start;
                    array_push($content, $coursemenu);
                    $content = array_merge($content, array_values($end));
                }
            } else {
                array_unshift($content, $coursemenu);
            }
        }

        if ($this->is_headercentersplit() && $enableheader) {
            return $this->split_header($content);
        } else {
            if (count($content) > 6) {
                $displaymenu = array_slice($content, 0, 5);
                $displaymenu = implode('', $displaymenu);
                return $displaymenu . $this->moremenu_context(array_slice($content, 5));
            } else {
                return implode('', $content);
            }
        }
    }

    /**
     * This function contains the code for show the drop down more menu option in the top header.
     * @param display_menu $displaymenu
     * @return string html
     */
    public function moremenu_context($displaymenu) {
        $menu = implode('', $displaymenu);
        $content = "";
        $content .= html_writer::start_tag('li', array('class' => 'dropdown show'));
            $content .= html_writer::link('#', get_string('more', 'theme_enlight'),
                array('class' => 'dropdown-toggle nav-link', 'role' => 'button', 'id' => 'enlightmoremenulink',
                'data-toggle' => 'dropdown', 'aria-haspopup' => 'true', 'aria-expanded' => 'false'));
            $content .= html_writer::start_tag('ul', array('class' => 'dropdown-menu', 'aria-labelledby' => 'enlightmoremenulink'));
            $content .= $menu;
            $content .= html_writer::end_tag('ul');
        $content .= html_writer::end_tag('li');
        return $content;
    }

    /**
     * Check header split.
     * @return bool
     */
    public function is_headercentersplit() {
        $headerstyle = theme_enlight_get_setting('headerStyle');
        if ( (int) $headerstyle == 5) {
            return true;
        }
        return false;
    }

    /**
     * Spilt the header
     * @param array $menuslist
     * @return array|string
     */
    public function split_header($menuslist=array()) {
        if (!empty($menuslist)) {
            $count = count($menuslist);
            if ($count > 8) {
                $displaymenu = array_slice($menuslist, 0, 5);
                $displaymenucount = count($displaymenu);
                $menus = array_chunk($displaymenu, ceil( $displaymenucount / 2 ));
                $splitone = (isset($menus[0])) ? $menus[0] : $menus;
                $splittwo = (isset($menus[1])) ? $menus[1] : [];
                return ['leftside' => implode('', $splitone),
                'rightside' => implode('', $splittwo), 'moremenu' => $this->moremenu_context(array_slice($menuslist, 8))];
            } else {
                $menus = array_chunk($menuslist, ceil( $count / 2 ));
                $splitone = (isset($menus[0])) ? $menus[0] : $menus;
                $splittwo = (isset($menus[1])) ? $menus[1] : [];
                return ['leftside' => implode('', $splitone), 'rightside' => implode('', $splittwo) ];
            }
        }
        return '';
    }

    /**
     * This function contains the code for create and add the course menu into primary menu
     * items.
     * @return string
     */
    public function course_menu() {
        $data = [];
        $tcmenu = $this->top_course_menu();
        $cmenushow = theme_enlight_get_setting('cmenushow');
        if ($cmenushow) {
            $data['topmmenu'] = $tcmenu['topmmenu'];
            $data['topcmenu'] = $tcmenu['topcmenu'];
            $data['courseindexurl'] = new moodle_url('/course/index.php');
            return $this->render_template('course_menu', $data);
        }
        return '';

    }


    /* top course menu custom */
    /**
     * Course list for course menu on header
     * @return array
     */
    public function top_course_menu() {
        global $CFG, $DB;

        $list = \core_course_category::make_categories_list();
        $mclist = array();
        if ($CFG->dbtype == "postgresql") {
            $sql = "SELECT a.category , a.cnt from (
            SELECT category , count(category) as cnt FROM {course}
            WHERE category != '0' and visible = ?
            group by category
            order by randam()) as a
            order by a.cnt desc ";
        } else {
            $sql = "SELECT a.category , a.cnt from (
            SELECT category , count(category) as cnt FROM {course}
            WHERE category != '0' and visible = ?
            group by category
            order by rand()) as a
            order by a.cnt desc ";
        }
        $params = array('1');
        $result = $DB->get_records_sql($sql, $params, 0, 4);
        if ($result) {
            foreach ($result as $rowcat) {
                $mclist[] = $rowcat->category;
            }
        }
        $rcourseids = array();
        foreach ($mclist as $catid) {
            $cname = core_course_category::get($catid, MUST_EXIST, true)->get_formatted_name();
            $menuheader = html_writer::start_tag('div', array('class' => 'cols'));
            $menuheader .= html_writer::tag('h6', $cname);
            $menuheader .= html_writer::start_tag('ul');
            $menufooter = html_writer::end_tag('ul');
            $menufooter .= html_writer::end_tag('div');
            $mmenuheader = html_writer::start_tag('li', array('class' => "dropdown-submenu"));
            $mmenuheader .= html_writer::link("#", $cname);
            $mmenuheader .= html_writer::start_tag('ul', array('class' => "dropdown-menu"));
            $mmenufooter = html_writer::end_tag('ul');
            $mmenufooter .= html_writer::end_tag('li');
            $menuitems = '';
            if ($CFG->dbtype == "postgresql") {
                $condition = "c.id DESC,c.sortorder ASC LIMIT 6 OFFSET 0";
            } else {
                $condition = "c.id DESC,c.sortorder ASC LIMIT 0, 6";
            }

            if ($ccc = get_courses($catid, $condition, 'c.id,c.shortname,c.fullname,c.visible')) {
                foreach ($ccc as $cc) {
                    if ($cc->visible == "0" || $cc->id == "1") {
                        continue;
                    }
                    $courseurl = new moodle_url("/course/view.php", array("id" => $cc->id));
                    $menuitems .= html_writer::start_tag('li');
                    $menuitems .= html_writer::link($courseurl, format_string($cc->fullname));
                    $menuitems .= html_writer::end_tag('li');
                }
                if (!empty($menuitems)) {
                    $rcourseids[$catid] = array("desk" => $menuheader.$menuitems.$menufooter,
                         "mobile" => $mmenuheader.$menuitems.$mmenufooter
                    );
                }
            }
        }
        $mcourseids = array_slice($rcourseids, 0, 4);
        $strcourse = $mstrcourse = '';
        foreach ($mcourseids as $ctid => $marr) {
            $strcourse .= $marr["desk"]."\n";
            $mstrcourse .= $marr["mobile"]."\n";
        }
        $courseaurl = new moodle_url('/course/index.php');
        if (!empty($strcourse)) {
            $topcmenu = html_writer::start_tag('div', array("class" => "custom-dropdown-menu", "id" => "cr_menu",
                "style" => "display:none;"));
            $topcmenu .= html_writer::start_tag('div', array("class" => 'cols-wrap'));
            $topcmenu .= $strcourse;
            $topcmenu .= html_writer::tag('div', null, array('class' => "clearfix"));
            $topcmenu .= html_writer::end_tag('div');
            $topcmenu .= html_writer::end_tag('div');
        } else {
            $topcmenu = "";
        }

        $topmmenu = html_writer::start_tag("ul", array("class" => "dropdown-menu"));
        $topmmenu .= $mstrcourse;
        $topmmenu .= html_writer::start_tag('li');
        $topmmenu .= html_writer::link($courseaurl, get_string('viewall', 'theme_enlight'));
        $topmmenu .= html_writer::end_tag('li');
        $topmmenu .= html_writer::end_tag("ul");
        return ['topcmenu' => $topcmenu, 'topmmenu' => $topmmenu];
    }

    /**
     * This code renders the custom menu items for the
     * bootstrap dropdown menu.
     * @param custom_menu_item $menunode
     * @param int $level
     *
     */
    protected function render_custom_menu_item(\custom_menu_item $menunode, $level = 0 ) {
        static $submenucount = 0;
        $content = '';
        if ($menunode->has_children()) {
            if ($level == 1) {
                $class = 'dropdown';
            } else {
                $class = 'dropdown-submenu';
            }
            if ($menunode === $this->language) {
                $class .= ' langmenu';
            }
            $content = html_writer::start_tag('li', array('class' => $class));
            // If the child has menus render it as a sub menu.
            $submenucount++;
            if ($menunode->get_url() !== null) {
                $url = $menunode->get_url();
            } else {
                $url = '#cm_submenu_'.$submenucount;
            }
            $content .= html_writer::start_tag('a', array('href' => $url, 'class' => 'dropdown-toggle',
                'data-toggle' => 'dropdown', 'title' => $menunode->get_title()));
            $content .= $menunode->get_text();
            if ($level == 1) {
                $content .= html_writer::tag("b", null, array("class" => "caret"));
            }
            $content .= html_writer::end_tag("a");
            $content .= html_writer::start_tag("ul", array("dropdown-menu"));
            foreach ($menunode->get_children() as $menunode) {
                $content .= $this->render_custom_menu_item($menunode, 0);
            }
            $content .= html_writer::end_tag("ul");
        } else {
            // The node doesn't have children so produce a final menuitem.
            // Also, if the node's text matches '####', add a class so we can treat it as a divider.
            if (preg_match("/^#+$/", $menunode->get_text())) {
                // This is a divider.
                $content = html_writer::tag('li', "&nbsp;", array('class' => "divider"));
            } else {
                $content = html_writer::start_tag("li");
                if ($menunode->get_url() !== null) {
                    $url = $menunode->get_url();
                } else {
                    $url = '#';
                }
                $content .= html_writer::link($url, $menunode->get_text(), array('title' => $menunode->get_title()));
                $content .= html_writer::end_tag("li");
            }
        }
        return $content;
    }

    /**
     * Construct a user menu, returning HTML that can be echoed out by a
     * layout file.
     *
     * @param stdClass $user A user object, usually $USER.
     * @param bool $withlinks true if a dropdown should be built.
     * @return string HTML fragment.
     */
    public function user_menu($user = null, $withlinks = null) {
        global $USER, $CFG;
        require_once($CFG->dirroot . '/user/lib.php');

        if (is_null($user)) {
            $user = $USER;
        }

        // Note: this behaviour is intended to match that of core_renderer::login_info,
        // but should not be considered to be good practice; layout options are
        // intended to be theme-specific. Please don't copy this snippet anywhere else.
        if (is_null($withlinks)) {
            $withlinks = empty($this->page->layout_options['nologinlinks']);
        }

        // Add a class for when $withlinks is false.
        $usermenuclasses = 'usermenu';
        if (!$withlinks) {
            $usermenuclasses .= ' withoutlinks';
        }

        $returnstr = "";

        // If during initial install, return the empty return string.
        if (during_initial_install()) {
            return $returnstr;
        }

        $loginpage = $this->is_login_page();
        $loginurl = get_login_url();
        // If not logged in, show the typical not-logged-in string.
        if (!isloggedin()) {
            $returnstr = "";
            if (isset($CFG->registerauth) && $CFG->registerauth == 'email') {
                $returnstr .= " <a href='".new moodle_url('/login/signup.php')."'>". get_string('startsignup')."</a>";
            }
            if (!$loginpage) {
                if (isset($CFG->registerauth) && $CFG->registerauth == 'email') {
                    $returnstr .= " | ";
                }
                $returnstr .= " <a href=\"$loginurl\">" . get_string('login') . '</a>';
            }
            return html_writer::div(
                html_writer::span(
                    $returnstr,
                    'login'
                ),
                $usermenuclasses
            );

        }

        // If logged in as a guest user, show a string to that effect.
        if (isguestuser()) {
            $returnstr = get_string('loggedinasguest');
            if (!$loginpage && $withlinks) {
                $returnstr .= " (<a href=\"$loginurl\">".get_string('login').'</a>)';
            }

            return html_writer::div(
                html_writer::span(
                    $returnstr,
                    'login'
                ),
                $usermenuclasses
            );
        }

        // Get some navigation opts.
        $opts = user_get_user_navigation_info($user, $this->page);

        $avatarclasses = "avatars";
        $avatarcontents = html_writer::span($opts->metadata['useravatar'], 'avatar current');
        $usertextcontents = $opts->metadata['userfullname'];

        // Other user.
        if (!empty($opts->metadata['asotheruser'])) {
            $avatarcontents .= html_writer::span(
                $opts->metadata['realuseravatar'],
                'avatar realuser'
            );
            $usertextcontents = $opts->metadata['realuserfullname'];
            $usertextcontents .= html_writer::tag(
                'span',
                get_string(
                    'loggedinas',
                    'moodle',
                    html_writer::span(
                        $opts->metadata['userfullname'],
                        'value'
                    )
                ),
                array('class' => 'meta viewingas')
            );
        }

        // Role.
        if (!empty($opts->metadata['asotherrole'])) {
            $role = \core_text::strtolower(preg_replace('#[ ]+#', '-', trim($opts->metadata['rolename'])));
            $usertextcontents .= html_writer::span(
                $opts->metadata['rolename'],
                'meta role role-' . $role
            );
        }

        // User login failures.
        if (!empty($opts->metadata['userloginfail'])) {
            $usertextcontents .= html_writer::span(
                $opts->metadata['userloginfail'],
                'meta loginfailures'
            );
        }

        // MNet.
        if (!empty($opts->metadata['asmnetuser'])) {
            $mnet = strtolower(preg_replace('#[ ]+#', '-', trim($opts->metadata['mnetidprovidername'])));
            $usertextcontents .= html_writer::span(
                $opts->metadata['mnetidprovidername'],
                'meta mnet mnet-' . $mnet
            );
        }

        $returnstr .= html_writer::span(
            html_writer::span($usertextcontents, 'usertext mr-2') .
            html_writer::span($avatarcontents, $avatarclasses),
            'userbutton'
        );

        // Create a divider (well, a filler).
        $divider = new \action_menu_filler();
        $divider->primary = false;

        $am = new \action_menu();
        $am->set_menu_trigger(
            $returnstr
        );
        $am->set_alignment(\action_menu::TR, \action_menu::BR);
        $am->set_nowrap_on_items();
        if ($withlinks) {
            $navitemcount = count($opts->navitems);
            $idx = 0;
            foreach ($opts->navitems as $key => $value) {

                switch ($value->itemtype) {
                    case 'divider':
                        // If the nav item is a divider, add one and skip link processing.
                        $am->add($divider);
                        break;

                    case 'invalid':
                        // Silently skip invalid entries (should we post a notification?).
                        break;

                    case 'link':
                        // Process this as a link item.
                        $pix = null;
                        if (isset($value->pix) && !empty($value->pix)) {
                            $pix = new \pix_icon($value->pix, $value->title, null, array('class' => 'iconsmall'));
                        } else if (isset($value->imgsrc) && !empty($value->imgsrc)) {
                            $value->title = html_writer::img(
                                $value->imgsrc,
                                $value->title,
                                array('class' => 'iconsmall')
                            ) . $value->title;
                        }

                        $al = new \action_menu_link_secondary(
                            $value->url,
                            $pix,
                            $value->title,
                            array('class' => 'icon')
                        );
                        if (!empty($value->titleidentifier)) {
                            $al->attributes['data-title'] = $value->titleidentifier;
                        }
                        $am->add($al);
                        break;
                }

                $idx++;

                // Add dividers after the first item and before the last item.
                if ($idx == 1 || $idx == $navitemcount - 1) {
                    $am->add($divider);
                }
            }
        }

        return html_writer::div(
            $this->render($am),
            $usermenuclasses
        );
    }

    /**
     * Renders the login form.
     *
     * @param \core_auth\output\login $form The renderable.
     * @return string
     */
    public function render_login(\core_auth\output\login $form) {
        global $SITE, $CFG;

        $context = $form->export_for_template($this);

        // Override because rendering is not supported in template yet.
        $context->cookieshelpiconformatted = $this->help_icon('cookiesenabled');
        $context->errorformatted = $this->error_text($context->error);
        $url = $this->get_logo_url();
        if ($url) {
            $url = $url->out(false);
        }
        $context->logourl = ($url) ? $url : theme_enlight_get_logo_url();
        $context->sitename = format_string($SITE->fullname, true,
            ['context' => \context_course::instance(SITEID), "escape" => false]);

        $loginpagetype = $this->page->theme->settings->loginpageType;
        $context->loginheader = get_string('loginheader', 'theme_enlight');
        $context->siteurl = new \moodle_url('/?redirect=0');
        if ($loginpagetype == "1") {
            $context->slideimages = theme_enlight_get_login_slideimages();
            $context->show_slideimages = (!empty($context->slideimages)) ? true : false;
            return $this->render_from_template('theme_enlight/loginform', $context);
        } else {
            return $this->render_from_template('core/loginform', $context);
        }

    }

    /**
     * Header logo
     * @return string
     */
    public function header_logo() {
        global $CFG, $SITE;
        $logourl = theme_enlight_get_logo_url();
        $classes = 'navbar-brand';
        $classes .= (!empty($logourl)) ? ' has-logo ' : ' d-none d-sm-block ';
        $output = html_writer::start_tag('a', ['href' => $CFG->wwwroot, 'class' => $classes ]);
        if (!empty($logourl)) {
            $output .= html_writer::start_tag('span', ['class' => 'logo']);
            $output .= html_writer::empty_tag('img', ['src' => $logourl]);
            $output .= html_writer::end_tag('span');
        } else {
            $output .= html_writer::tag('span', '', ['class' => 'site-name d-none d-sm-block']);
        }
        $output .= html_writer::end_tag('a');
        return $output;
    }

    /**
     * Wrapper for header elements.
     *
     * @return string HTML to display the main header.
     */
    public function full_header() {
        global $USER, $DB;
        if ($this->page->include_region_main_settings_in_header_actions() &&
                !$this->page->blocks->is_block_present('settings')) {

            // Only include the region main settings if the page has requested it and it doesn't already have
            // the settings block on it. The region main settings are included in the settings block and
            // duplicating the content causes behat failures.
            $this->page->add_header_action(html_writer::div(
                $this->region_main_settings_menu(),
                'd-print-none',
                ['id' => 'region-main-settings-menu']
            ));
        }
        // Add ediweser course format plugin.
        $course = $this->page->course;
        $courseformat = course_get_format($course);
        $imgurl = '';
        $remuiformat = false;
        if ($this->page->pagelayout == 'course') {
            if ($courseformat->get_format() == 'remuiformat') {
                $remuiformat = true;
                // Add image url.
                $settings = $courseformat->get_settings();
                $config = \format_remuiformat\course_format_data_common_trait::getinstance();;
                $imgurl = $config->display_file($settings['remuicourseimage_filemanager']);
                // Add blocks.
                $enrollusers = enrol_get_course_users($course->id);
                $sql = "SELECT * FROM {course_completions} WHERE course=:courseid AND timecompleted IS NOT NULL ";
                $completedusers = $DB->get_records_sql($sql, ['courseid' => $course->id]);
                $activites = course_modinfo::get_array_of_activities($course);
                $context = \context_course::instance($course->id);
                $teachers = get_role_users(3 , $context);
            }
        }

        $header = new stdClass();
        $header->settingsmenu = $this->context_header_settings_menu();
        $header->contextheader = $this->context_header();
        $header->hasnavbar = empty($this->page->layout_options['nonavbar']);
        $header->navbar = $this->navbar();
        $header->pageheadingbutton = $this->page_heading_button();
        $header->courseheader = $this->course_header();
        $header->headeractions = $this->page->get_header_actions();
        // Image url added.
        $header->remuiformatimgurl = $imgurl;
        $header->enrollusers = isset($enrollusers) ? count($enrollusers) : '';
        $header->completedusers = isset($completedusers) ? count($completedusers) : '';
        $header->activites = isset($activites) ? count($activites) : '';
        $header->teachers = isset($teachers) ? count($teachers) : '';
        $header->remuiformat = $remuiformat;
        return $this->render_from_template('theme_enlight/core/full_header', $header);
    }

    /**
     * Render the core blocks template
     * @param string $template
     * @param array $data
     */
    public function render_template($template, $data) {
        $data[$template] = 1;
        return $this->render_from_template('theme_enlight/core_blocks', $data);
    }
} // Here the course renderer fucntion closed.

