<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Frontpage editing on settings.
 * @package   theme_enlight
 * @copyright 2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


if (!class_exists('admin_setting_frontpageblocks')) {
    /**
     * Special select - lists on the frontpage - hacky
     *
     * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
     */
    class admin_setting_frontpageblocks extends admin_setting {
        /** @var array Array of choices value=>label */
        public $choices;

        /**
         * Construct override, requires one param
         *
         * @param bool $loggedin Is the user logged in
         */
        public function __construct($loggedin) {
            global $CFG;
            require_once($CFG->dirroot.'/course/lib.php');
            require_once($CFG->dirroot.'/theme/enlight/lib.php');
            $name        = 'theme_enlight/frontpage'.($loggedin ? 'loggedin' : '');
            $visiblename = get_string('frontpage'.($loggedin ? 'loggedin' : ''), 'admin');
            $description = get_string('configfrontpage'.($loggedin ? 'loggedin' : ''), 'admin');
            $defaults = array(
                FRONTPAGEFEATURES => 0,
                FRONTPAGEALLCOURSELIST => 1,
                FRONTPAGECATEGORYNAMES => 1,
                FRONTPAGEENROLLEDCOURSELIST => 1,
                FRONTPAGENEWS => 1,
                FRONTPAGECATEGORYCOMBO => 1,
                FRONTPAGECOUNTERS => 0,
                FRONTPAGECOURSESEARCH => 1,
                FRONTPAGEMARKETING1 => 0,
                FRONTPAGEMARKETING2 => 0
            );
            if ($loggedin) {
                $frontpageloggedin = explode(",", $CFG->frontpageloggedin);
                foreach ($defaults as $key => $courseblock) {
                    if ($courseblock) {
                        if (!in_array($key, $frontpageloggedin)) {
                            unset($defaults[$key]);
                        }
                    }
                }
            } else {
                $frontpage = explode(",", $CFG->frontpage);
                foreach ($defaults as $key => $courseblock) {
                    if ($courseblock) {
                        if (!in_array($key, $frontpage)) {
                            unset($defaults[$key]);
                        }
                    }
                }
            }
            $defaults = array_keys($defaults);
            parent::__construct($name, $visiblename, $description, $defaults);
        }

        /**
         * Loads the choices available.
         *
         * @return bool always returns true
         */
        public function load_choices() {
            if (is_array($this->choices)) {
                return true;
            }
            $this->choices = array(
                FRONTPAGENEWS          => get_string('frontpagenews'),
                FRONTPAGEALLCOURSELIST => get_string('frontpagecourselist'),
                FRONTPAGEENROLLEDCOURSELIST => get_string('frontpageenrolledcourselist'),
                FRONTPAGECATEGORYNAMES => get_string('frontpagecategorynames'),
                FRONTPAGECATEGORYCOMBO => get_string('frontpagecategorycombo'),
                FRONTPAGECOURSESEARCH  => get_string('frontpagecoursesearch'),
                FRONTPAGEPOPULARCOURSES => get_string('popularcourses', 'theme_enlight'),
                FRONTPAGEMARKETING1 => get_string('marketingspot', 'theme_enlight') .
                    ' 1 (' . get_string('aboutustxt', 'theme_enlight').')',
                FRONTPAGEMARKETING2 => get_string('marketingspot', 'theme_enlight') .
                    ' 2 ( ' . get_string('learntitle', 'theme_enlight')." )",
                FRONTPAGEFEATURES => get_string('sitefeature_tabhead', 'theme_enlight'),
                FRONTPAGEOURTEAM => get_string('ourTeam', 'theme_enlight'),
                FRONTPAGETESTIMONIALS => get_string('tmonialheading', 'theme_enlight'),
                FRONTPAGECOUNTERS => get_string('countertabheading', 'theme_enlight'),
                FRONTPAGEPARTNERS => get_string('ourpartnerheading', 'theme_enlight'),
                'none'                 => get_string('none'));
            if ($this->name === 'frontpage') {
                unset($this->choices[FRONTPAGEENROLLEDCOURSELIST]);
            }
            return true;
        }

        /**
         * Returns the selected settings
         *
         * @return mixed array or setting or null
         */
        public function get_setting() {
            $result = $this->config_read($this->name);
            if (is_null($result)) {
                return null;
            }
            if ($result === '') {
                return array();
            }
            return explode(',', $result);
        }

        /**
         * Save the selected options
         *
         * @param array $data
         * @return mixed empty string (data is not an array) or bool true=success false=failure
         */
        public function write_setting($data) {
            if (!is_array($data)) {
                return '';
            }
            $this->load_choices();
            $save = array();
            foreach ($data as $datum) {
                if (!array_key_exists($datum, $this->choices)) {
                    continue;
                }
                $save[$datum] = $datum; // No duplicates.
            }
            return ($this->config_write($this->name, implode(',', $save)) ? '' : get_string('errorsetting', 'admin'));
        }

        /**
         * Return XHTML select field and wrapping div
         *
         * @todo Add vartype handling to make sure $data is an array
         * @param array $data Array of elements to select by default
         * @param string $query
         * @return string XHTML select field and wrapping div
         */
        public function output_html($data, $query='') {
            global $OUTPUT;

            $this->load_choices();
            $currentsetting = array();
            foreach ($data as $key) {
                if (array_key_exists($key, $this->choices)) {
                    $currentsetting[] = $key; // Already selected first.
                }
            }

            $context = (object) [
                'id' => $this->get_id(),
                'name' => $this->get_full_name(),
            ];

            $options = $this->choices;
            $selects = [];
            for ($i = 0; $i < count($this->choices) - 1; $i++) {
                if (!array_key_exists($i, $currentsetting)) {
                    $currentsetting[$i] = 'none';
                }
                $selects[] = [
                    'key' => $i,
                    'options' => array_map(function($option) use ($options, $currentsetting, $i) {
                        return [
                            'name' => $options[$option],
                            'value' => $option,
                            'selected' => $currentsetting[$i] == $option
                        ];
                    }, array_keys($options))
                ];
            }
            $context->selects = $selects;

            $element = $OUTPUT->render_from_template('core_admin/setting_courselist_frontpage', $context);

            return format_admin_setting($this, $this->visiblename, $element, $this->description, false, '', null, $query);
        }
    }

}
