<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Contains the default section course format output class.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace format_levels\output\courseformat\content;

use context_course;
use core\output\named_templatable;
use core_courseformat\base as course_format;
use core_courseformat\output\local\courseformat_named_templatable;
use renderable;
use renderer_base;
use section_info;
use stdClass;
use core_courseformat\output\local\content\section as section_base;

/**
 * Base class to render a course section.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class section extends section_base {

    /**
     * Add the section cm list to the data structure.
     *
     * @param stdClass $data the current cm data reference
     * @param renderer_base $output typically, the renderer that's calling this function
     * @return bool if the cm has name data
     */
    protected function add_cm_data(stdClass &$data, renderer_base $output): bool {
        $result = false;

        $section = $this->section;
        $format = $this->format;

        $showsummary = ($section->section != 0 &&
            $section->section != $format->get_section_number() &&
            $format->get_course_display() == COURSE_DISPLAY_MULTIPAGE &&
            !$format->show_editor()
        );

        $showcmlist = $section->uservisible;

        // Add activities summary if necessary.
        if ($showsummary) {
            $cmsummary = new $this->cmsummaryclass($format, $section);
            $data->cmsummary = $cmsummary->export_for_template($output);
            $data->onlysummary = true;
            $result = true;

            if (!$format->is_section_current($section)) {
                // In multipage, only the current section (and the section zero) has elements.
                $showcmlist = false;
            }
        }
        // Add the cm list.
        if ($showcmlist) {
            $cmlist = new $this->cmlistclass($format, $section);
            $cms = $cmlist->export_for_template($output);

            // Next available module.
            // Format levels data.
            $formatoptions = new \format_levels\options($this->format->get_course());
            foreach ($cms->cms as $cmid => $cm) {
                $cm = $formatoptions->format->get_modinfo()->get_cm($cm->cmitem->id);
                $cmitem = $cms->cms[$cmid]->cmitem;
                $iscompleted = $formatoptions->get_module_completion($cm);
                $cmitem->completion = $iscompleted;
                $completion = $formatoptions->has_completion_enabled($cm);
                if (!isset($nextmodule) &&
                    $cm->uservisible && ($completion) && !$formatoptions->get_module_completion($cm)) {
                    $nextmodule = true;
                    $cmitem->isnext = true;
                }
                $cmitem->levelimage = $formatoptions->get_activity_image($cmitem, $cm);

                $moduleimages = $formatoptions->get_course_images('activestate', $cm->id, \context_module::instance($cm->id));
                if ($cm->uservisible && !$iscompleted) {
                    $cmitem->activeimage = $moduleimages ?: $formatoptions->get_global_fallback('modactivebg');
                }
                $cmitem->completionimage = $formatoptions->is_completion_stateimage($cmitem);
                $cmitem->unavailableimage = $formatoptions->is_unavailable_stateimage($cmitem);
                $cmitem->imgheight = $formatoptions->get_option($cm->id, 'imgheight') ?: false;
                $cmitem->imgwidth = $formatoptions->get_option($cm->id, 'imgwidth') ?: false;
                $objectposition = $formatoptions->get_option($cm->id, 'objectposition');
                $cmitem->objectposition = "object-pos-".($objectposition == \format_levels::OBJECTTOP ? 'top'
                    : ( $objectposition == \format_levels::OBJECTBOTTOM ? 'bottom'
                    : ($objectposition == \format_levels::OBJECTLEFT ? 'left' : 'right'))
                );

                $cms->cms[$cmid]->cmitem = $cmitem;
            }

            if (!isset($nextmodule)) {
                $nextcm = current(array_keys($cms->cms));
                if (isset($cms->cms[$nextcm]->cmitem)) {
                    $cms->cms[$nextcm]->cmitem->isnext = true;
                }
            }

            $data->cmlist = $cms;
            if ($this->format->get_section_option($section->id, 'useactivityimages')) {
                $data->useactivityimage = true;
            }
            $result = true;
        }

        return $result;
    }

    /**
     * Add the section format attributes to the data structure.
     *
     * @param stdClass $data the current cm data reference
     * @param bool[] $haspartials the result of loading partial data elements
     * @param renderer_base $output typically, the renderer that's calling this function
     * @return bool if the cm has name data
     */
    protected function add_format_data(stdClass &$data, array $haspartials, renderer_base $output): bool {
        $section = $this->section;
        $format = $this->format;
        $singlesection = $this->format->get_section_number();
        if ($singlesection) {
            $data->showcmlistonly = true;
        }
        $data->iscoursedisplaymultipage = ($format->get_course_display() == COURSE_DISPLAY_MULTIPAGE);

        if ($data->num === 0 && !$data->iscoursedisplaymultipage) {
            $data->collapsemenu = true;
        }

        $data->contentcollapsed = false;
        $preferences = $format->get_sections_preferences();
        if (isset($preferences[$section->id])) {
            $sectionpreferences = $preferences[$section->id];
            if (!empty($sectionpreferences->contentcollapsed)) {
                $data->contentcollapsed = true;
            }
        }

        if ($format->is_section_current($section)) {
            $data->iscurrent = true;
            $data->currentlink = get_accesshide(
                get_string('currentsection', 'format_' . $format->get_format())
            );
        }
        // Format levels data.
        $formatoptions = new \format_levels\options($this->format->get_course());
        $sectionimages = $formatoptions->get_section_images($this->section->id);

        $sectionoptions = $this->format->get_format_options($section->section);
        $image = $sectionimages['coursepreview'];
        if (empty($image)) {
            $image = $sectionimages['availablelevel'] ?: $formatoptions->get_global_fallback('sectionpreviewlevelbg');
        }
        $data->sectionimage = $image;
        $data->sectionbgimage = $formatoptions->get_section_bgimage($section);
        $data->activityimagemode = $this->format->get_section_option($section->id, 'useactivityimages');
        $data->levelmapsizeclass = $formatoptions->get_levelmap_sizeclass($data->num);
        $data->course = $this->format->get_course();
        $data->courseurl = new \moodle_url('/course/view.php', ['id' => $data->course->id]);
        $data->sectionurl = course_get_url($section->course, $section->section, array('navigation' => true));
        $data->completed = $formatoptions->get_section_completion($this->section->id);
        $data->initialposition = $sectionoptions['initialposition'] ?? 'lefttop';
        $data->imgheight = $sectionoptions['imgheight'] ?? '';
        $data->imgwidth = $sectionoptions['imgwidth'] ?? '';
        $data->backgroundheight = $sectionoptions['backgroundheight'] ?? '';
        $data->backgroundwidth = $sectionoptions['backgroundwidth'] ?? '';
        $data->showloader = (AJAX_SCRIPT && $sectionoptions['levelmapsize'] == 'original') ? false : true;
        return true;
    }
}
