<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * This file contains main class for LEVELS course format.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

define('LEVELS_ENABLE_POPUPACTIVITIES', 1);

define('LEVELS_DISABLE_POPUPACTIVITIES', 0);

define('LEVELS_MAPSIZE_ORIGINAL', 'original');

define('LEVELS_MAPSIZE_FITHEIGHT', 'fitheight');

define('LEVELS_MAPSIZE_FITWIDTH', 'fitwidth');

define('LEVELS_DEFAULT_XPOSITION', 200);

define('LEVELS_DEFAULT_YPOSITION', 200);

require_once($CFG->dirroot. '/course/format/lib.php');

use core\output\inplace_editable;
use format_levels\options;

/**
 * Main class for the LEVELS course format.
 *
 * @package   format_levels
 * @copyright 2022 bdecent gmbh <https://bdecent.de>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class format_levels extends core_courseformat\base {

    /**
     * Hide the course index bar in course pages only.
     */
    const HIDE_ON_COURSEPAGE = 1;

    /**
     * Hide the course index bar everywhere.
     */
    const HIDE_EVERYWHERE = 2;

    /**
     * Object position to top.
     */
    const OBJECTTOP = 0;

    /**
     * Object position to right.
     */
    const OBJECTRIGHT = 1;

    /**
     * Object position to bottom.
     */
    const OBJECTBOTTOM = 2;

    /**
     * Object minimize to left.
     */
    const OBJECTLEFT = 3;

    /**
     * Returns true if this course format uses sections.
     *
     * @return bool
     */
    public function uses_sections() {
        return true;
    }

    /**
     * Returns true if this course format uses course index
     *
     * This function may be called without specifying the course id
     * i.e. in course_index_drawer()
     *
     * @return bool
     */
    public function uses_course_index() {
        global $PAGE;
        $course = $this->get_course();
        $index = isset($course->courseindex) ?
            ($course->courseindex == self::HIDE_EVERYWHERE ? false :
            ($course->courseindex == 1 && $PAGE->cm == null ? false : true) ) : true;
        return $index;
    }

    /**
     * Uses the identation for the moduels.
     *
     * @return bool
     */
    public function uses_indentation(): bool {
        return false;
    }

    /**
     * Returns the display name of the given section that the course prefers.
     *
     * Use section name is specified by user. Otherwise use default ("Topic #").
     *
     * @param int|stdClass $section Section object from database or just field section.section
     * @return string Display name that the course format prefers, e.g. "Topic 2"
     */
    public function get_section_name($section) {
        $section = $this->get_section($section);
        if ((string)$section->name !== '') {
            return format_string($section->name, true,
                ['context' => context_course::instance($this->courseid)]);
        } else {
            return $this->get_default_section_name($section);
        }
    }

    /**
     * Returns the default section name for the levels course format.
     *
     * If the section number is 0, it will use the string with key = section0name from the course format's lang file.
     * If the section number is not 0, the base implementation of course_format::get_default_section_name which uses
     * the string with the key = 'sectionname' from the course format's lang file + the section number will be used.
     *
     * @param stdClass $section Section object from database or just field course_sections section
     * @return string The default value for the section name.
     */
    public function get_default_section_name($section) {
        if ($section->section == 0) {
            // Return the general section.
            return get_string('section0name', 'format_levels');
        } else {
            // Use course_format::get_default_section_name implementation which
            // will display the section name in "Topic n" format.
            return parent::get_default_section_name($section);
        }
    }

    /**
     * The URL to use for the specified course (with section).
     *
     * @param int|stdClass $section Section object from database or just field course_sections.section
     *     if omitted the course view page is returned
     * @param array $options options for view URL. At the moment core uses:
     *     'navigation' (bool) if true and section has no separate page, the function returns null
     *     'sr' (int) used by multipage formats to specify to which section to return
     * @return null|moodle_url
     */
    public function get_view_url($section, $options = []) {
        global $CFG;
        $course = $this->get_course();
        $url = new moodle_url('/course/view.php', ['id' => $course->id]);

        $sr = null;
        if (array_key_exists('sr', $options)) {
            $sr = $options['sr'];
        }
        if (is_object($section)) {
            $sectionno = $section->section;
        } else {
            $sectionno = $section;
        }
        if ($sectionno !== null) {
            if ($sr !== null) {
                if ($sr) {
                    $sectionno = $sr;
                }
            }
            if ($sectionno != 0) {
                $url->param('section', $sectionno);
            }
        }
        return $url;
    }

    /**
     * Returns the information about the ajax support in the given source format.
     *
     * The returned object's property (boolean)capable indicates that
     * the course format supports Moodle course ajax features.
     *
     * @return stdClass
     */
    public function supports_ajax() {
        $ajaxsupport = new stdClass();
        $ajaxsupport->capable = true;
        return $ajaxsupport;
    }

    /**
     * Returnd the information about the component support in the given source format.
     *
     * @return bool
     */
    public function supports_components() {
        return true;
    }

    /**
     * Custom action after section has been moved in AJAX mode.
     *
     * Used in course/rest.php
     *
     * @return array This will be passed in ajax respose
     */
    public function ajax_section_move() {
        global $PAGE;
        $titles = [];
        $course = $this->get_course();
        $modinfo = get_fast_modinfo($course);
        $renderer = $this->get_renderer($PAGE);
        if ($renderer && ($sections = $modinfo->get_section_info_all())) {
            foreach ($sections as $number => $section) {
                $titles[$number] = $renderer->section_title($section, $course);
            }
        }
        return ['sectiontitles' => $titles, 'action' => 'move'];
    }

    /**
     * Returns the list of blocks to be automatically added for the newly created course.
     *
     * @return array of default blocks, must contain two keys BLOCK_POS_LEFT and BLOCK_POS_RIGHT
     *     each of values is an array of block names (for left and right side columns)
     */
    public function get_default_blocks() {
        return [
            BLOCK_POS_LEFT => [],
            BLOCK_POS_RIGHT => [],
        ];
    }

    /**
     * Definitions of the additional options that this course format uses for course.
     *
     *
     * @param bool $foreditform
     * @return array of options
     */
    public function course_format_options($foreditform = false) {
        static $courseformatoptions = false;
        if ($courseformatoptions === false) {
            $courseconfig = get_config('moodlecourse');
            $courseformatoptions = [
                'coursebgimage_filemanager' => [
                    'default' => 0,
                    'type' => PARAM_FILE
                ],
                'courseindex' => [
                    'default' => get_config('format_levels', 'courseindex'),
                    'type' => PARAM_INT
                ],
                'popupactivities' => [
                    'default' => get_config('format_levels', 'popupactivities'),
                    'type' => PARAM_INT
                ],
                'popupactivitiesinfo' => [
                    'default' => get_string('popupactivitiesnotinstalled', 'format_levels'),
                    'type' => PARAM_RAW,
                    'label' => get_string('popupactivities', 'format_levels'),
                ]
            ];
        }
        if ($foreditform && !isset($courseformatoptions['coursedisplay']['label'])) {
            $courseformatoptionsedit = [
                'coursebgimage_filemanager' => [
                    'label' => new lang_string('coursebgimage', 'format_levels'),
                    'element_type' => 'filemanager',
                    'element_attributes' => array([], format_levels_course_bgimage_options()),
                ],
                'courseindex' => [
                    'label' => new lang_string('courseindex', 'format_levels'),
                    'element_type' => 'select',
                    'element_attributes' => [
                        [
                            0 => new lang_string('show'),
                            self::HIDE_ON_COURSEPAGE => new lang_string('hideoncourses', 'format_levels'),
                            self::HIDE_EVERYWHERE => new lang_string('hideeverywhere', 'format_levels'),
                        ],
                    ],
                    'help' => 'courseindex',
                    'help_component' => 'format_levels',
                ]
            ];

            if (format_levels_popup_installed()) {
                $courseformatoptionsedit['popupactivities'] = [
                    'label' => new lang_string('popupactivities', 'format_levels'),
                    'element_type' => 'select',
                    'element_attributes' => [
                        [
                            LEVELS_DISABLE_POPUPACTIVITIES => new lang_string('disable'),
                            LEVELS_ENABLE_POPUPACTIVITIES => new lang_string('enable'),
                        ],
                    ],
                    'help' => 'popupactivities',
                    'help_component' => 'format_levels',
                ];
                $courseformatoptionsedit['popupactivitiesinfo'] = [
                    'element_type' => 'hidden',
                ];
            } else {
                $courseformatoptionsedit['popupactivitiesinfo'] = [
                    'element_type' => 'static',
                ];
                $courseformatoptionsedit['popupactivities'] = [
                    'element_type' => 'hidden',
                    'label' => get_string('popupactivities', 'format_levels'),
                ];
            }
            $courseformatoptions = array_merge_recursive($courseformatoptions, $courseformatoptionsedit);
        }
        return $courseformatoptions;
    }

    /**
     * Adds format options elements to the course/section edit form.
     *
     * This function is called from {@see course_edit_form::definition_after_data()}.
     *
     * @param MoodleQuickForm $mform form the elements are added to.
     * @param bool $forsection 'true' if this is a section edit form, 'false' if this is course edit form.
     * @return array array of references to the added form elements.
     */
    public function create_edit_form_elements(&$mform, $forsection = false) {
        global $COURSE;

        $elements = parent::create_edit_form_elements($mform, $forsection);
        // Prepare draft area for courseimage.
        if ($bgfileoptions = format_levels_course_bgimage_options()) {
            $draftid = 0;
            file_prepare_draft_area($draftid, $this->get_course_context()->id, 'format_levels', 'coursebgimage', 0, $bgfileoptions);
            if ($mform->elementExists('coursebgimage_filemanager')) {
                $mform->setDefault('coursebgimage_filemanager', $draftid);
            }
        }

        if (!$forsection && (empty($COURSE->id) || $COURSE->id == SITEID)) {
            // Add "numsections" element to the create course form - it will force new course to be prepopulated
            // with empty sections.
            // The "Number of sections" option is no longer available when editing course, instead teachers should
            // delete and add sections when needed.
            $courseconfig = get_config('moodlecourse');
            $max = (int)$courseconfig->maxsections;
            $element = $mform->addElement('select', 'numsections', get_string('numberweeks'), range(0, $max ?: 52));
            $mform->setType('numsections', PARAM_INT);
            if (is_null($mform->getElementValue('numsections'))) {
                $mform->setDefault('numsections', $courseconfig->numsections);
            }
            array_unshift($elements, $element);
        }

        return $elements;
    }

    /**
     * Updates format options for a course.
     *
     * In case if course format was changed to 'levels', we try to copy options
     * 'coursedisplay' and 'hiddensections' from the previous format.
     *
     * @param stdClass|array $data return value from {@see moodleform::get_data()} or array with data
     * @param stdClass $oldcourse if this function is called from {@see update_course()}
     *     this object contains information about the course before update
     * @return bool whether there were any changes to the options values
     */
    public function update_course_format_options($data, $oldcourse = null) {
        $data = (array)$data;
        if ($oldcourse !== null) {
            $oldcourse = (array)$oldcourse;
            $options = $this->course_format_options();
            foreach ($options as $key => $unused) {
                if (!array_key_exists($key, $data)) {
                    if (array_key_exists($key, $oldcourse)) {
                        $data[$key] = $oldcourse[$key];
                    }
                }
                if ($key == 'popupactivities' && !format_levels_popup_installed()) {
                    $data[$key] = false;
                }
            }

        }
        if ($bgfileoptions = format_levels_course_bgimage_options()) {
            // Save the course overviewfiles/.
            $data = (array) file_postupdate_standard_filemanager((object) $data, 'coursebgimage',
                $bgfileoptions, $this->get_course_context(), 'format_levels', 'coursebgimage', 0);
        }
        unset($data['popupactivitiesinfo']);
        return $this->update_format_options($data);
    }

    /**
     * Fetch the context of the current course.
     *
     * @return \context_course
     */
    public function get_course_context() {
        return $this->get_course() ? \context_course::instance($this->get_course()->id) : context_system::instance();
    }

    /**
     * Whether this format allows to delete sections.
     *
     * Do not call this function directly, instead use {@see course_can_delete_section()}
     *
     * @param int|stdClass|section_info $section
     * @return bool
     */
    public function can_delete_section($section) {
        return true;
    }

    /**
     * Definitions of the additional options that this course format uses for section
     *
     * See course_format::course_format_options() for return array definition.
     *
     * Additionally section format options may have property 'cache' set to true
     * if this option needs to be cached in get_fast_modinfo(). The 'cache' property
     * is recommended to be set only for fields used in course_format::get_section_name(),
     * course_format::extend_course_navigation() and course_format::get_view_url()
     *
     * For better performance cached options are recommended to have 'cachedefault' property
     * Unlike 'default', 'cachedefault' should be static and not access get_config().
     *
     * Regardless of value of 'cache' all options are accessed in the code as
     * $sectioninfo->OPTIONNAME
     * where $sectioninfo is instance of section_info, returned by
     * get_fast_modinfo($course)->get_section_info($sectionnum)
     * or get_fast_modinfo($course)->get_section_info_all()
     *
     * All format options for particular section are returned by calling:
     * $this->get_format_options($section);
     *
     * @param bool $foreditform
     * @return array
     */
    public function section_format_options($foreditform = false) {
        global $PAGE, $DB;
        $sectionid = optional_param('id', 0, PARAM_INT);
        if ($sectionid && $PAGE->pagetype == 'course-editsection') {
            $sectionimages = self::section_levels_background_images();
            $bgfileoptions = format_levels_course_bgimage_options();
            foreach ($sectionimages as $filearea) {
                $draftid = 0;
                file_prepare_draft_area($draftid, $PAGE->context->id, 'format_levels',
                    $filearea, $sectionid, $bgfileoptions);
                if ($DB->record_exists('course_format_options', array('courseid' => $PAGE->course->id,
                    'sectionid' => $sectionid, 'name' => $filearea."_filemanager"))) {
                    $DB->set_field('course_format_options', 'value', $draftid, array('courseid' =>
                    $PAGE->course->id, 'sectionid' => $sectionid, 'name' => $filearea."_filemanager"));
                }
            }
        }
        return self::section_format_options_list($foreditform);
    }

    /**
     * List of fileareas background images and thumbnail image for sections.
     *
     * @return array
     */
    public static function section_levels_background_images() {
        return ['availablelevel', 'unavailablelevel', 'completedlevel', 'coursepreview'];
    }

    /**
     * List of fileareas background images and thumbnail image for modules.
     *
     * @return array
     */
    public static function module_levels_bgimages() {
        return ['available', 'nextavailable', 'unavailable', 'completed', 'activestate'];
    }

    /**
     * Config list for sections. Used in global settings and section edit.
     *
     * @param bool $foreditform
     * @return array List of section settings.
     */
    public static function section_format_options_list($foreditform) {
        global $CFG, $PAGE;
        $sectionoptions = array(
            'sectiontype' => array(
                'type' => PARAM_ALPHANUMEXT,
                'label' => '',
                'element_type' => 'hidden',
                'default' => 'default',
            ),
        );

        $sectionoptions['headsectionappearence'] = array(
            'type' => PARAM_TEXT,
            'element_type' => 'header',
            'default' => get_string('appearance'),
            'label' => get_string('appearance'),
        );

        $sectionimages = self::section_levels_background_images();

        foreach ($sectionimages as $image) {
            $sectionoptions[$image.'_filemanager'] = [
                'label' => new lang_string($image.':title', 'format_levels'),
                'element_type' => 'filemanager',
                'element_attributes' => [[], format_levels_section_bgimage_options()],
            ];
        }

        $sectionoptions += [
            'levelmapsize' => [
                'label' => new lang_string('levelmapsize', 'format_levels'),
                'element_type' => 'select',
                'element_attributes' => [
                    [
                        LEVELS_MAPSIZE_ORIGINAL => new lang_string('originalsize', 'format_levels'),
                        LEVELS_MAPSIZE_FITHEIGHT => new lang_string('fitheight', 'format_levels'),
                        LEVELS_MAPSIZE_FITWIDTH => new lang_string('fitwidth', 'format_levels'),
                    ]
                ],
                'default' => get_config('format_levels', 'levelmapsize')
            ],

            'initialposition' => [
                'label' => new lang_string('initialposition', 'format_levels'),
                'element_type' => 'select',
                'element_attributes' => [
                    [
                        'lefttop' => new lang_string('lefttop', 'format_levels'),
                        'leftcenter' => new lang_string('leftcenter', 'format_levels'),
                        'leftbottom' => new lang_string('leftbottom', 'format_levels'),
                        'righttop' => new lang_string('righttop', 'format_levels'),
                        'rightbottom' => new lang_string('rightbottom', 'format_levels'),
                        'rightcenter' => new lang_string('rightcenter', 'format_levels'),
                        'centerbottom' => new lang_string('centerbottom', 'format_levels'),
                        'centertop' => new lang_string('centertop', 'format_levels'),
                        'centercenter' => new lang_string('centercenter', 'format_levels'),
                    ]
                ],
                'default' => get_config('format_levels', 'initialposition')
            ],
            'useactivityimages' => [
                'label' => new lang_string('useactivityimages', 'format_levels'),
                'element_type' => 'select',
                'element_attributes' => [
                    [
                        1 => get_string('enable'),
                        0 => get_string('disable')
                    ]
                ],
                'default' => get_config('format_levels', 'useactivityimages') ? 1 : 0

            ],
            'backgroundwidth' => [
                'label' => new lang_string('backgroundwidth', 'format_levels'),
                'element_type' => 'text',
                'element_attributes' => [],
                'default' => get_config('format_levels', 'backgroundwidth') ?: '',
                'type' => PARAM_INT
            ],
            'backgroundheight' => [
                'label' => new lang_string('backgroundheight', 'format_levels'),
                'element_type' => 'text',
                'element_attributes' => [],
                'default' => get_config('format_levels', 'backgroundheight') ?: '',
                'type' => PARAM_INT
            ],
            'imgwidth' => [
                'label' => new lang_string('thumbimgwidth', 'format_levels'),
                'element_type' => 'text',
                'element_attributes' => [],
                'default' => get_config('format_levels', 'imgwidth') ?: '',
                'type' => PARAM_INT
            ],
            'imgheight' => [
                'label' => new lang_string('thumbimgheight', 'format_levels'),
                'element_type' => 'text',
                'element_attributes' => [],
                'default' => get_config('format_levels', 'imgheight') ?: '',
                'type' => PARAM_INT
            ],
        ];
        return $sectionoptions;
    }

    /**
     * Updates format options for a section
     *
     * Section id is expected in $data->id (or $data['id'])
     * If $data does not contain property with the option name, the option will not be updated
     *
     * @param stdClass|array $data return value from {@see moodleform::get_data()} or array with data
     * @return bool whether there were any changes to the options values
     */
    public function update_section_format_options($data) {
        global $COURSE;
        $data = (array) $data;
        if ($bgfileoptions = format_levels_course_bgimage_options()) {
            $itemid = $data['id']; // Use section id as itemid.
            $sectionimages = self::section_levels_background_images();
            foreach ($sectionimages as $filearea) {
                $data = (array) file_postupdate_standard_filemanager(
                    (object) $data, $filearea, $bgfileoptions, $this->get_course_context(),
                    'format_levels', $filearea, $itemid
                );
            }
        }

        unset($data['headsectionappearence']);
        return $this->update_format_options($data, $data['id']);
    }

    /**
     * Indicates whether the course format supports the creation of a news forum.
     *
     * @return bool
     */
    public function supports_news() {
        return true;
    }

    /**
     * Add student view class to the body element for style purpose.
     *
     * @param moodle_page $page
     * @return void
     */
    public function page_set_course(moodle_page $page) {
        global $PAGE;
        if ($PAGE->bodyid == 'page-course-view' && format_levels\options::is_student_view()) {
            $PAGE->add_body_class('student-view');
            if ($section = optional_param('section', false, PARAM_INT)) {
                $formatoptions = new format_levels\options($page->course);
                $PAGE->add_body_class($formatoptions->get_levelmap_sizeclass($section));
                $PAGE->add_body_class('section-levels');
            }
        }
    }

    /**
     * Returns whether this course format allows the activity to
     * have "triple visibility state" - visible always, hidden on course page but available, hidden.
     *
     * @param stdClass|cm_info $cm course module (may be null if we are displaying a form for adding a module)
     * @param stdClass|section_info $section section where this module is located or will be added to
     * @return bool
     */
    public function allow_stealth_module_visibility($cm, $section) {
        // Allow the third visibility state inside visible sections or in section 0.
        return !$section->section || $section->visible;
    }

    /**
     * Return the plugin configs for external functions.
     *
     * @return array the list of configuration settings
     * @since Moodle 3.5
     */
    public function get_config_for_external() {
        // Return everything (nothing to hide).
        return $this->get_format_options();
    }

    /**
     * Extend the course navigation to setup the carousel.
     *
     * And check the student view and setup the student mode for the course.
     *
     * @param stdclass $navigation
     * @param navigation_node $node
     * @return void
     */
    public function extend_course_navigation($navigation, navigation_node $node) {
        global $PAGE, $OUTPUT;

        if ($this->get_course()->format == 'levels' && format_levels\options::is_student_view()) {
            $secondnav = new \core\navigation\views\secondary($PAGE);
            // Setup the secondary nav without initialise
            // Will hide the navigation $secondnav->initialise();.
            $PAGE->requires->css('/course/format/levels/style/slick.css');
            $PAGE->requires->css('/course/format/levels/style/slick-theme.css');
            $PAGE->set_secondarynav($secondnav);

            if ($section = optional_param('section', null, PARAM_INT)) {
                $sectionoptions = $this->get_section($section);
                $PAGE->set_heading(get_section_name($sectionoptions->course, $sectionoptions));
            }
        }

        if ($this->get_course()->format == 'levels'
            && has_capability('format/levels:managelevels', $PAGE->context) && !$PAGE->user_is_editing()) {
            $button = $PAGE->button;
            $url = $PAGE->url;
            if (optional_param('setup', false, PARAM_BOOL)) {
                $param = 0;
                $status = get_string('disablesetup', 'format_levels');
            } else {
                $param = 1;
                $status = get_string('enablesetup', 'format_levels');
            }
            $url->param('setup', $param);
            $switchbutton = html_writer::link($url, $status, ['class' => 'btn btn-primary']);
            $PAGE->set_button($button . ' '. $switchbutton);
        }
    }

    /**
     * Get the course display value for the current course.
     *
     * Formats extending topics or weeks will use coursedisplay as this setting name
     * so they don't need to override the method. However, if the format uses a different
     * display logic it must override this method to ensure the core renderers know
     * if a COURSE_DISPLAY_MULTIPAGE or COURSE_DISPLAY_SINGLEPAGE is being used.
     *
     * @return int The current value (COURSE_DISPLAY_MULTIPAGE or COURSE_DISPLAY_SINGLEPAGE)
     */
    public function get_course_display(): int {
        global $PAGE;
        return ($PAGE->user_is_editing()) ?: COURSE_DISPLAY_MULTIPAGE;
    }

    /**
     * Get section option.
     *
     * @param int $sectionid Course section number to get option for.
     * @param string $name Option name.
     * @return string|null
     * @throws dml_exception
     */
    public function get_section_option(int $sectionid, string $name) {
        global $DB;
        return $DB->get_field('course_format_options', 'value', [
            'courseid' => $this->courseid,
            'format' => 'levels',
            'sectionid' => $sectionid,
            'name' => $name
        ]) ?: null;
    }

}

/**
 * Implements callback inplace_editable() allowing to edit values in-place.
 *
 * @param string $itemtype
 * @param int $itemid
 * @param mixed $newvalue
 * @return inplace_editable
 */
function format_levels_inplace_editable($itemtype, $itemid, $newvalue) {
    global $DB, $CFG;
    require_once($CFG->dirroot . '/course/lib.php');
    if ($itemtype === 'sectionname' || $itemtype === 'sectionnamenl') {
        $section = $DB->get_record_sql(
            'SELECT s.* FROM {course_sections} s JOIN {course} c ON s.course = c.id WHERE s.id = ? AND c.format = ?',
            [$itemid, 'levels'], MUST_EXIST);
        return course_get_format($section->course)->inplace_editable_update_section_name($section, $itemtype, $newvalue);
    }
}

/**
 * Find the plugin format_popup installed.
 *
 * @return bool
 */
function format_levels_popup_installed() {
    $pluginman = \core_plugin_manager::instance();
    $plugininfo = $pluginman->get_plugin_info('format_popups');
    return !empty($plugininfo) ? true : false;
}
/**
 * Course background image file manager options.
 *
 * @return array
 */
function format_levels_course_bgimage_options() {
    $maxbytes = format_levels_convert_tobytes(get_config('format_levels', 'courseimgsize'));
    return format_levels_bgimage_options($maxbytes);
}

/**
 * Section background image file manager options.
 *
 * @return array
 */
function format_levels_section_bgimage_options() {
    $maxbytes = format_levels_convert_tobytes(get_config('format_levels', 'sectionimgsize'));
    return format_levels_bgimage_options($maxbytes);
}

/**
 * Activity background image file manager options.
 *
 * @return array
 */
function format_levels_activity_bgimage_options() {
    $maxbytes = format_levels_convert_tobytes(get_config('format_levels', 'activityimgsize'));
    return format_levels_bgimage_options($maxbytes);
}

/**
 * Background image file manager options.
 *
 * @param int $maxbytes Maximum bytes of the filearea
 * @return array
 */
function format_levels_bgimage_options($maxbytes) {
    return [
        'subdirs' => 0,
        'maxfiles' => 1,
        'maxbytes' => $maxbytes,
        'areamaxbytes' => $maxbytes,
        'accepted_types' => ["image"],
    ];
}

/**
 * Init the dynamic Style rules before the body html renderer.
 *
 * @param int $sectionnumber Section number for ajax update.
 * @return string
 */
function format_levels_before_standard_top_of_body_html($sectionnumber=null) {
    global $PAGE;
    if (!format_levels\options::is_student_view()) {
        return null;
    }
    $styles = (new format_levels\options($PAGE->course))->get_dynamic_css_rules($sectionnumber);
    return html_writer::tag('style', $styles);
}

/**
 * Serves file from sectiondesignbackground_filearea
 *
 * @param mixed $course course or id of the course
 * @param mixed $cm course module or id of the course module
 * @param context $context Context used in the file.
 * @param string $filearea Filearea the file stored
 * @param array $args Arguments
 * @param bool $forcedownload Force download the file instead of display.
 * @param array $options additional options affecting the file serving
 * @return bool false if file not found, does not return if found - just send the file
 */
function format_levels_pluginfile($course, $cm, $context, $filearea, $args, $forcedownload, array $options = array()) {
    require_login();

    if (!in_array($context->contextlevel, [CONTEXT_COURSE, CONTEXT_MODULE, CONTEXT_SYSTEM]) ||
        ( $filearea != 'coursebgimage'
        && !in_array($filearea,  \format_levels::section_levels_background_images())
        && !in_array($filearea, \format_levels::module_levels_bgimages())
        && !in_array($filearea, global_fallbackimages())  )) {
        return false;
    }

    $areas = array_merge(['coursebgimage'],
        \format_levels::section_levels_background_images(),
        \format_levels::module_levels_bgimages(), global_fallbackimages()
    );

    if (!in_array($filearea, $areas)) {
        return false;
    }
    $fs = get_file_storage();
    $file = $fs->get_file($context->id, 'format_levels', $filearea, $args[0], '/', $args[1]);
    if (!$file) {
        return false;
    }
    send_stored_file($file, 0, 0, 0, $options);
}

/**
 * Inject the levels module elements into all moodle module settings forms.
 *
 * @param moodleform $formwrapper The moodle quickforms wrapper object.
 * @param MoodleQuickForm $mform The actual form object (required to modify the form).
 */
function format_levels_coursemodule_standard_elements($formwrapper, $mform) {
    global $CFG, $DB;
    $cm = $formwrapper->get_coursemodule();
    $course = $formwrapper->get_course();
    if ($course->format == 'levels') {

        if (isset($cm->id) && $cm->id) {
            $leveloptions = \format_levels\options::get_options($cm->id);
        }
        // Activity elements list to manage the visibility.
        $elements = ['available', 'nextavailable', 'unavailable', 'completed', 'activestate'];

        $mform->addElement('header', 'moduledesign', get_string('activitydesign', 'format_levels'));

        $bgfileoptions = array('accepted_types' => 'images', 'maxfiles' => 1);

        foreach ($elements as $element) {
            // Module background image repeat.
            $name = 'levels_'.$element.'';
            $title = get_string('image:'.$element, 'format_levels');
            $mform->addElement('filemanager', $name, $title, null, format_levels_activity_bgimage_options());
        }

        $mform->addElement('text', 'levels_imgwidth', get_string('modimgwidth', 'format_levels'));
        $mform->setType('levels_imgwidth', PARAM_INT);
        if (isset($leveloptions->imgwidth)) {
            $mform->setDefault('levels_imgwidth', $leveloptions->imgwidth);
        } else {
            $mform->setDefault('levels_imgwidth', get_config('format_levels', 'imgwidth'));
        }

        $mform->addElement('text', 'levels_imgheight', get_string('modimgheight', 'format_levels'));
        $mform->setType('levels_imgheight', PARAM_INT);
        if (isset($leveloptions->imgheight)) {
            $mform->setDefault('levels_imgheight', $leveloptions->imgheight);
        } else {
            $mform->setDefault('levels_imgheight', get_config('format_levels', 'imgheight'));
        }

        $section = $formwrapper->get_section();
        $cmid = (isset($cm->id) && $cm->id) ? $cm->id : null;
        list($defaultxpos, $defaultypos) = format_levels\options::get_default_module_position($cmid, $section, $course);

        $mform->addElement('text', 'levels_xposition', get_string('xposition', 'format_levels'));
        $mform->setType('levels_xposition', PARAM_INT);
        if (isset($leveloptions->xposition)) {
            $mform->setDefault('levels_xposition', $leveloptions->xposition);
        } else {
            $mform->setDefault('levels_xposition', $defaultxpos);
        }

        $mform->addElement('text', 'levels_yposition', get_string('yposition', 'format_levels'));
        $mform->setType('levels_yposition', PARAM_INT);

        if (isset($leveloptions->yposition)) {
            $mform->setDefault('levels_yposition', $leveloptions->yposition);
        } else {
            $mform->setDefault('levels_yposition', $defaultypos);
        }

        $positions = [
            \format_levels::OBJECTTOP => 'Top',
            \format_levels::OBJECTBOTTOM => 'Bottom',
            \format_levels::OBJECTLEFT => 'Left',
            \format_levels::OBJECTRIGHT => 'Right'
        ];
        $mform->addElement('select', 'levels_objectposition', get_string('objectposition', 'format_levels'), $positions);
        $mform->setType('levels_objectposition', PARAM_INT);
        $mform->addHelpButton('levels_objectposition', 'objectposition', 'format_levels');
        if (isset($leveloptions->objectposition)) {
            $mform->setDefault('levels_objectposition', $leveloptions->objectposition);
        } else {
            $mform->setDefault('levels_objectposition', get_config('format_levels', 'objectposition'));
        }
    }
}

/**
 * Hook the add/edit of the course module.
 *
 * @param stdClass $data Data from the form submission.
 * @param stdClass $course The course.
 */
function format_levels_coursemodule_edit_post_actions($data, $course) {
    global $DB;
    $cmid = $data->coursemodule;
    if ($course->format == 'levels') {
        $fields = [
            'levels_available', 'levels_nextavailable',
            'levels_unavailable', 'levels_completed',
            'levels_activestate', 'levels_xposition', 'levels_yposition',
            'levels_imgheight', 'levels_imgwidth', 'levels_objectposition'
        ];
        foreach ($fields as $field) {
            if (!isset($data->$field)) {
                continue;
            }
            $name = str_replace('levels_', '', $field);
            $name = str_replace('_filemanager', '', $name);
            if (isset($data->$field)) {
                if (is_array($data->$field)) {
                    $value = json_encode($data->$field);
                } else {
                    $value = $data->{$field};
                }
                \format_levels\options::insert_option($cmid, $course->id, $name, $value);
            }
        }

        $fields = ['available', 'nextavailable', 'unavailable', 'completed', 'activestate'];
        $modulecontext = context_module::instance($cmid);
        $bgfileoptions = array('accepted_types' => 'images', 'maxfiles' => 1);

        foreach ($fields as $field) {
            if (isset($data->{'levels_'.$field})) {
                file_save_draft_area_files(
                    $data->{'levels_'.$field}, $modulecontext->id, 'format_levels', $field, $data->coursemodule
                );
            }
        }
    }
    return $data;
}

/**
 * Set up the definitions after data set to the form elements.
 *
 * @param moodle_modform $formwrapper
 * @param moodle_form $mform
 * @return void
 */
function format_levels_coursemodule_definition_after_data($formwrapper, $mform) {
    $elements = ['available', 'nextavailable', 'unavailable', 'completed', 'activestate'];
    $cm = $formwrapper->get_coursemodule();
    foreach ($elements as $element) {
        $name = 'levels_'.$element;
        $draftitemid = file_get_submitted_draft_itemid($name);
        if (!empty($cm)) {
            $context = context_module::instance($cm->id);
            file_prepare_draft_area($draftitemid, $context->id, 'format_levels', $element, $cm->id,
                format_levels_activity_bgimage_options());
        }
        $mform->setDefault($name, $draftitemid);
    }
}

/**
 * List of fileareas in global fallback images.
 *
 * @return array
 */
function global_fallbackimages() {
    return [
        'courselevelbg',
        'sectionavailablelevelbg',
        'sectionunavailablelevelbg',
        'sectioncompletelevelbg',
        'sectionpreviewlevelbg',
        'modavailablelevelbg',
        'modnextavailablelevelbg',
        'modunavailablelevelbg',
        'modcompletelevelbg',
        'modactivebg',
    ];
}

/**
 * Convert the file size to Bytes.
 *
 * @param string $from Size of the image file.
 * @return int size of the image in bytes.
 */
function format_levels_convert_tobytes($from) {
    $units = ['B', 'KB', 'MB', 'GB'];
    $number = (int) substr($from, 0, -2);
    $unit = trim(strtoupper(substr($from, -2)));
    if (!$unit) {
        return;
    }
    $mag = (int) array_flip($units)[$unit] ?? 0;
    if (is_numeric(substr($unit, 0, 1))) {
        return preg_replace('/[^\d]/', '', $from);
    }

    return $number * (1024 ** $mag);
}

/**
 * Update the modules position.
 *
 * @param [type] $args
 * @return void
 */
function format_levels_output_fragment_update_mod_positions($args) {
    global $PAGE;
    if ($args['context']->contextlevel == CONTEXT_COURSE && !empty($args['modid'])) {
        list($course, $cm) = get_course_and_cm_from_cmid($args['modid']);
        if ($xposition = $args['x']) {
            format_levels\options::insert_option($cm->id, $course->id, 'xposition', (int) $xposition);
        }
        if ($yposition = $args['y']) {
            format_levels\options::insert_option($cm->id, $course->id, 'yposition', (int) $yposition);
        }
        return true;
    }
    return false;
}
