<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Event observer class definition.
 *
 * @package   local_reminder
 * @copyright lmsace dev team
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_reminder;

defined('MOODLE_INTERNAL') || die('No direct access !');

require_once($CFG->dirroot."/local/reminder/lib.php");

use stdclass;

class reminder_handler {

    /**
     * Reminder instance data record.
     *
     * @var array
     */
    public $instance;

    public function __construct() {
        global $DB;
        $this->notificationinterval = get_config('local_reminder', 'notificationinterval');
    }

    public function reminder() {
        // Define the which action to send notification.
        if (get_config('local_reminder', 'statusnotifinotlogin')) {
            $this->process_user_not_accessed();
        }
        if (get_config('local_reminder', 'statusnotifinotaccess')) {
            $this->process_user_not_accessedcourse();
        }
    }

    /**
     * process send notification to the email
     */
    public static function process_send_notification($user, $type, $course = null) {
        global $SITE;
        // Update the reminder events on tracker.
        if ($type != 'notifienrol') {
            local_reminder_update_event_tracker($user, $course, $type);
        }
        $reminderconfig = get_config("local_reminder");
        $strparams = ['firstname' => $user->firstname, 'lastname' => $user->lastname, 'sitename' => $SITE->fullname,
            'course' => ($course != null) ? $course->fullname : ''];
        $subject = get_string($type . "_subject", 'local_reminder', $strparams);
        $messagehtml = ($reminderconfig->{$type}) ? $reminderconfig->{$type} :
            get_string($type . "_body", 'local_reminder', $strparams);
        $messagehtml = local_reminder_update_emailvars($messagehtml, $user, $course);
        $messageplain = html_to_text($messagehtml); // Plain text.
        $messagesend = local_reminder_messagetouser($user, $subject, $messageplain, $messagehtml, $course);
    }

    public function process_user_not_accessed() {
        global $DB, $CFG;
        $interval = get_config('local_reminder', 'notifinotlogininterval');
        $timestart = time() - $interval;
        $sql = "SELECT *
        FROM {user} u
        WHERE u.suspended = 0 AND u.confirmed = 1 AND u.deleted = 0 AND u.id > 2
        AND ((u.lastaccess = 0 AND u.timecreated < :starttime) OR (u.lastaccess != 0 AND u.lastaccess < :timestart))";
        $notaccessedusers = $DB->get_records_sql($sql, ['timestart' => $timestart, 'starttime' => $timestart]);
        if ($notaccessedusers) {
            foreach ($notaccessedusers as $user) {
                // Check the reminder events tracker times exist or not.
                if (local_reminder_get_event_times($user, null, 'notifinotlogin') < $this->notificationinterval &&
                local_reminder_check_user_receivenotifcation($user, null, 'notifinotlogin', $interval)) {
                    mtrace("Call the process notification for userid ". $user->id."on the notifinotlogin");
                    self::process_send_notification($user, 'notifinotlogin');
                }
            }
        }
    }

    public function process_user_not_accessedcourse() {
        global $DB, $CFG;
        $interval = get_config('local_reminder', 'notifinotaccessinterval');
        $timestart = time() - $interval;
        $params = ['timestart' => $timestart, 'starttime' => $timestart];
        $params['now1'] = round(time(), -2);
        $params['now2'] = $params['now1'];
        $studentroles = array_keys(get_archetype_roles('student'));
        list($studentsql, $studentparams) = $DB->get_in_or_equal($studentroles, SQL_PARAMS_NAMED);
        $params += $studentparams;
        $sql = "SELECT DISTINCT ue.id, ue.userid, e.courseid
        FROM {user_enrolments} ue
        INNER JOIN {enrol} e ON e.id = ue.enrolid
        INNER JOIN {user} u ON u.id = ue.userid
        INNER JOIN {role_assignments} ra ON u.id = ra.userid
        INNER JOIN {context} cx ON ra.contextid = cx.id
        INNER JOIN {course} c ON cx.instanceid = c.id
        WHERE cx.contextlevel = 50 AND c.visible = 1 AND u.id > 2 AND u.suspended = 0 AND ra.roleid $studentsql
        AND u.deleted = 0 AND ue.status != 1 AND ue.timestart < :now1 AND
        (ue.timeend = 0 OR ue.timeend > :now2) AND ue.timecreated < :starttime ORDER BY e.courseid, ue.userid";
        $notcourseaccessedusers = $DB->get_records_sql($sql, $params);
        if ($notcourseaccessedusers) {
            foreach ($notcourseaccessedusers as $record) {
                $existsql = "SELECT * FROM {user_lastaccess} WHERE courseid = :courseid AND userid = :userid AND timeaccess > :timestart";
                if (!$DB->record_exists_sql($existsql, array('courseid' => $record->courseid, 'userid' => $record->userid,
                    'timestart' => $timestart))) {
                    $course = get_course($record->courseid);
                    $user = \core_user::get_user($record->userid, '*', MUST_EXIST);
                    $completion = new \completion_info($course);
                    if (!$completion->is_course_complete($user->id)) {
                        // Check the reminder events tracker times exist or not.
                        if (local_reminder_get_event_times($user, $course, 'notifinotaccess') < $this->notificationinterval &&
                            local_reminder_check_user_receivenotifcation($user, $course, 'notifinotaccess', $interval)) {
                            mtrace("Call the process notification for userid ". $user->id. " courseid ". $course->id . "on the notifinotaccess");
                            self::process_send_notification($user, 'notifinotaccess', $course);
                        }
                    }
                }
            }
        }
    }
}
