<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * @package   local_reminder
 * @copyright lmsace dev team
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die;

 /**
 * Replace email template placeholders with dynamic datas.
 *
 * @param  mixed $templatetext Email Body content with placeholders
 * @param  mixed $subject Mail subject with placeholders.
 * @param  mixed $course Course object data.
 * @param  mixed $user User data object.
 * @param  mixed $mod Pulse module data object.
 * @param  mixed $sender Sender user data object. - sender is the first enrolled teacher in the course of module.
 * @return array Updated subject and message body content.
 */
function local_reminder_update_emailvars($templatetext, $user, $course) {
    global $DB, $CFG;
    require_once($CFG->dirroot.'/local/reminder/lib/vars.php');
    $sender = core_user::get_support_user(); // Support user.
    $amethods = ReminderEmailVars::vars(); // List of available placeholders.
    $vars = new ReminderEmailVars($user, $course, $sender);

    foreach ($amethods as $funcname) {
        $replacement = "{" . $funcname . "}";
        // Message text placeholder update.
        if (stripos($templatetext, $replacement) !== false) {
            $val = $vars->$funcname;
            // Placeholder found on the text, then replace with data.
            $templatetext = str_replace($replacement, $val, $templatetext);
        }
    }
    return $templatetext;
}


/**
 * Send message to user using message api.
 *
 * @param  mixed $userto
 * @param  mixed $subject
 * @param  mixed $messageplain
 * @param  mixed $messagehtml
 * @param  mixed $course
 * @return bool message status
 */
function local_reminder_messagetouser($userto, $subject, $messageplain, $messagehtml, $course) {
    global $DB;
    $eventdata = new \core\message\message();
    $eventdata->name = 'instantmessage';
    $eventdata->component = 'moodle';
    $eventdata->courseid = ($course != null) ? $course->id : 0;
    $eventdata->userfrom = core_user::get_support_user();
    $eventdata->userto = $userto;
    $eventdata->subject = $subject;
    $eventdata->fullmessage = $messageplain;
    $eventdata->fullmessageformat = FORMAT_HTML;
    $eventdata->fullmessagehtml = $messagehtml;
    $eventdata->smallmessage = $subject;

    if (message_send($eventdata)) {
        return true;
    } else {
        return false;
    }
}


/**
 * Stores the reminder event data on the trackertable.
 * @param object $user
 * @param object $course
 * @param string $eventname
 */
function local_reminder_update_event_tracker($user, $course, $eventname) {
    global $DB, $SITE;
    $course = ($course != null) ? $course : $SITE;
    // Check the event already exist or not.
    if ($existrecord = $DB->get_record('local_reminder_eventstrack', array('userid' => $user->id, 'courseid' => $course->id,
        'eventname' => $eventname))) {
        $existrecord->times += 1;
        $existrecord->timemodified = time();
        $DB->update_record('local_reminder_eventstrack', $existrecord);
    } else {
        $record = new stdClass();
        $record->userid = $user->id;
        $record->courseid = $course->id;
        $record->eventname = $eventname;
        $record->times = 1;
        $record->timecreated = time();
        $record->timemodified = time();
        $DB->insert_record("local_reminder_eventstrack", $record);
    }
}

/**
 * Get the reminder event data times on the trackertable.
 * @param object $user
 * @param object $course
 * @param string $eventname
 * @return int times.
 */
function local_reminder_get_event_times($user, $course, $eventname) {
    global $DB, $SITE;
    $course = ($course != null) ? $course : $SITE;
    $times = $DB->get_field('local_reminder_eventstrack', 'times', array('userid' => $user->id, 'courseid' => $course->id,
    'eventname' => $eventname));
    return ($times) ? $times : 0;
}


function local_reminder_check_user_receivenotifcation($user, $course, $eventname, $interval) {
    global $DB, $SITE;
    $course = ($course != null) ? $course : $SITE;
    // Check the event record exist or not. If not exist user doesn't have any mail.
    if (!$existrecord = $DB->get_record('local_reminder_eventstrack', array('userid' => $user->id,
        'courseid' => $course->id, 'eventname' => $eventname))) {
        return true;
    } else {
        // If user exist in the event. So check the when lasttime sent it.
        if (($existrecord->timemodified + $interval) < time()) {
            return true;
        }
    }
	return false;
}


function local_reminder_delete_event_times($userid, $courseid = null) {
    global $DB;
    $params = ['userid' => $userid];
    if ($courseid != null) {
        $params['courseid'] = $courseid;
    }
    $DB->delete_records('local_reminder_eventstrack', $params);
}