<?php

namespace message_webpush;

require_once $CFG->dirroot.'/message/output/webpush/lib/vendor/autoload.php';

use Minishlink\WebPush\WebPush;
use Minishlink\WebPush\Subscription;
use Minishlink\WebPush\VAPID;
use stdClass;

/**
 * Webpush server.
 */
class webpushserver {

	private $webpushlib;

	private $log;

	public const ACTION_GOTO = 1;

	public const ACTION_EXPAND = 2;

	public const TITLESUBJECT = 1;

	public const TITLECUSTOM = 2;

	public const MESSAGESUBJECT = 1;

	public const MESSAGESMALL = 2;

	public const MESSAGEFULL = 3;


	/**
	 * Constructor.
	 */
	public function __construct() {

		// Get the VAPID keys to authenticate.
		$auth = self::get_my_vapid();

		// Create webpush library instance with the vapid auth keys.
		$this->webpushlib = new WebPush($auth);
		$this->log = new logs();
	}

	/**
	 * Create user notification.
	 */
	public function create_user_notification($message): void {
		global $CFG, $OUTPUT;

		if ($subscriptions = manageuser::get()->get_user_subscriptions($message->userto->id)) {

			// Create data object.
			$notification = new stdClass();
			$notification->message = strip_tags($message->pushmessage);
			// Title method.
			$titlemethod = get_config('message_webpush', 'title');
			$notification->title = ($titlemethod == self::TITLECUSTOM) ? get_config('message_webpush', 'customtitle') : $message->subject;
			// Notification tag - use the event type as tag to group the event messages, helps to renotify.
			$notification->tag = $message->eventname;
			// Language is RTL.
			$notification->dir = right_to_left() ? 'rtl' : 'ltr';
			$notification->data = ['url' => $notification->contexturl, 'wwwroot' => $CFG->wwwroot];

			// Get the compact logo URL.
			if ($clogo = $OUTPUT->get_compact_logo_url()) {
				$clogo = $clogo->out(false);
			}
			if ($logo = $OUTPUT->get_logo_url(200)) {
				$logo = $logo->out(false);
			}
			$notification->icon = $clogo ?: $logo;

			// Favicon.
			if ($icon = $OUTPUT->favicon()) {
				$notification->badge = $icon->out(false);
			}

			// Add notification actions.
            $this->get_actions($notification, $message);
print_r($notification);
			// Encode the notification into string.
			$payload = json_encode($notification);

			// Subscriptions.
			foreach ($subscriptions as $subscription) {
				// Create a subscription instance.
				$subscription = Subscription::create(json_decode((string) $subscription->subscriptionjson, true));
				// Add the notification in the Queue.
				$this->webpushlib->queueNotification($subscription, $payload);
			}
		}
	}

	/**
	 * Send notification.
	 */
	public function send_notifications() {

		// Send the notifications added in the queue.
		foreach ($this->webpushlib->flush() as $report) {
			$endpoint = $report->getRequest()->getUri()->__toString();
			// Check sent results
			if ($report->isSuccess()) {
				// echo "[v] Message sent successfully for subscription {$endpoint}.";
				// $this->log->add_notification_report($endpoint, $this->log::USER_NOTIFIED);
			} else {
				// echo "[x] Message failed to sent for subscription {$endpoint}: {$report->getReason()}";
				if ($report->isSubscriptionExpired()) {
					// TODO: remove the expired subscriptions.
					manageuser::get()->remove_subscription($endpoint);
				}
			}
		}
	}

	protected function get_actions(&$notification, $message) {

		$config = get_config('message_webpush', 'actions');
		$enabled = explode(',', $config);

		if (in_array(self::ACTION_GOTO, $enabled)) {
			$notification->actions[] = [
				'action' => 'action-goto',
				'title' => get_string('actiongoto', 'message_webpush'),
				// 'icon' => '/demos/notification-examples/images/action-1-128x128.png'
			];
		}

		if (in_array(self::ACTION_EXPAND, $enabled)) {

			$notification->actions[] = [
				'action' => 'action-expand',
				'title' => get_string('actionopen', 'message_webpush'),
				// 'icon' => '/demos/notification-examples/images/action-1-128x128.png'
			];
		}

		// print_r($notification);
	}

	/**
	 * Get my vapid keys.
	 *
	 * @return array
	 */
	public static function get_my_vapid(): array {
		global $CFG;

		$mailto = get_config('message_webpush', 'mailto');
		$publickey = get_config('message_webpush', 'publickey');
		$privatekey = get_config('message_webpush', 'privatekey');

		$auth = array(
			'VAPID' => array(
				'subject' => get_string('moodlewebpush', 'local_webpush'),
				'publicKey' => $publickey, // This key also used in service.js.
				'privateKey' => $privatekey,
			),
		);

		return $auth;
	}

	/**
	 * Generate vapid private and public keys.
	 *
	 * @return array
	 */
	public static function generate_vapid_keys() {
		// Create vapid keys.
		$keys = VAPID::createVapidKeys();

		return $keys;
	}

	/**
	 * Verify the libraries of the webpush server.
	 *
	 * @return bool
	 */
	public static function verify_libraries() {
		// List of libraries used in webpush libarary.
		$libraries = ['curl', 'mbstring', 'openssl'];
		$missing = [] ;
		foreach ($libraries as $library) {
			if (!extension_loaded($library)) {
				$missing[] = $library;
			}
		}

		return (count($missing) > 0) ? $missing : true;
	}
}
