<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Upgrade script for the contentdesigner.
 *
 * @package    mod_contentdesigner
 * @copyright  2022 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Uggrede plugin
 *
 * @param string $oldversion the version we are upgrading from.
 */
function xmldb_contentdesigner_upgrade($oldversion) {
    global $DB;

    $dbman = $DB->get_manager();

    if ($oldversion < 2024092308) {

        // Contentdesigner table.
        $table = new xmldb_table('contentdesigner');

        // Completion endreach condition.
        $completionendreach = new xmldb_field('completionendreach', XMLDB_TYPE_INTEGER, '4', null, null, null, '0', 'introformat');
        if (!$dbman->field_exists($table, $completionendreach)) {
            $dbman->add_field($table, $completionendreach);
        }

        // Navigation Method.
        $navigation = new xmldb_field('navigation', XMLDB_TYPE_INTEGER, '4', null, null, null, '0', 'completionendreach');
        if (!$dbman->field_exists($table, $navigation)) {
            $dbman->add_field($table, $navigation);
        }

        // Completion mandatory condition.
        $completionmandatory = new xmldb_field('completionmandatory', XMLDB_TYPE_INTEGER, '4', null, null, null, '0', 'navigation');
        if (!$dbman->field_exists($table, $completionmandatory)) {
            $dbman->add_field($table, $completionmandatory);
        }

        upgrade_mod_savepoint(true, 2024092308, 'contentdesigner');
    }

    if ($oldversion < 2024092312) {

        // Contentdesigner completion table.
        $table = new xmldb_table('contentdesigner_completion');

        // Changing the default of field completion on table contentdesigner completion to 0.
        $completion = new xmldb_field('completion', XMLDB_TYPE_INTEGER, '2', null, XMLDB_NOTNULL, null, '0', 'userid');
        // Launch change of default for field completion.
        if (!$dbman->field_exists($table, $completion)) {
            $dbman->change_field_default($table, $completion);
        }

        // Completion mandatory elements.
        $mandatorycompletion = new xmldb_field('mandatorycompletion', XMLDB_TYPE_INTEGER, '2', null,
            XMLDB_NOTNULL, null, '0', 'completion');
        if (!$dbman->field_exists($table, $mandatorycompletion)) {
            $dbman->add_field($table, $mandatorycompletion);
        }

        // Start time.
        $starttime = new xmldb_field('starttime', XMLDB_TYPE_INTEGER, '18', null,
            XMLDB_NOTNULL, null, '0', 'mandatorycompletion');
        if (!$dbman->field_exists($table, $starttime)) {
            $dbman->add_field($table, $starttime);
        }

        upgrade_mod_savepoint(true, 2024092312, 'contentdesigner');
    }

    if ($oldversion < 2024110804) {

        // Contentdesigner table.
        $table = new xmldb_table('contentdesigner');

        // Added completion endreach condition.
        $completionendreachfield = new xmldb_field('completionendreach', XMLDB_TYPE_INTEGER, '4', null, null, null,
            '0', 'introformat');
        if (!$dbman->field_exists($table, $completionendreachfield)) {
            $dbman->add_field($table, $completionendreachfield);
        }

        // Add navigation Method.
        $navigationfield = new xmldb_field('navigation', XMLDB_TYPE_INTEGER, '4', null, null, null, '0', 'completionendreach');
        if (!$dbman->field_exists($table, $navigationfield)) {
            $dbman->add_field($table, $navigationfield);
        }

        // Add completion mandatory condition.
        $completionmandatoryfield = new xmldb_field('completionmandatory', XMLDB_TYPE_INTEGER, '4', null, null, null,
            '0', 'navigation');
        if (!$dbman->field_exists($table, $completionmandatoryfield)) {
            $dbman->add_field($table, $completionmandatoryfield);
        }

        // Define fields to be added to contentdesigner_completion.
        $table = new xmldb_table('contentdesigner_completion');
        $table->add_field('id', XMLDB_TYPE_INTEGER, '18', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null, null);
        $table->add_field('contentdesignerid', XMLDB_TYPE_INTEGER, '18', null, null, null, null, 'id');
        $table->add_field('userid', XMLDB_TYPE_INTEGER, '18', null, null, null, null, 'contentdesignerid');
        $table->add_field('completion', XMLDB_TYPE_INTEGER, '2', null, XMLDB_NOTNULL, null, '0', 'userid');
        $table->add_field('mandatorycompletion', XMLDB_TYPE_INTEGER, '2', null, XMLDB_NOTNULL, null, '0', 'completion');
        $table->add_field('starttime', XMLDB_TYPE_INTEGER, '18', null, XMLDB_NOTNULL, null, '0', 'mandatorycompletion');
        $table->add_field('timecreated', XMLDB_TYPE_INTEGER, '18', null, null, null, null, 'starttime');
        $table->add_key('primary', XMLDB_KEY_PRIMARY, ['id']);

        // Conditionally launch create table.
        if (!$dbman->table_exists('contentdesigner_completion')) {
            $dbman->create_table($table);
        }

        upgrade_mod_savepoint(true, 2024110804, 'contentdesigner');
    }

    return true;
}
