<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Upgrade script for Moodle.
 *
 * @package    element_h5p
 * @copyright  2024 bdecent gmbh <https://bdecent.de>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Handles the upgrade process for the element_h5p plugin in Moodle.
 *
 * This function checks the current version of the plugin and performs
 * necessary database schema updates if the version is less than the specified
 * upgrade version. It creates the 'element_h5p_xapistates' table with the
 * required fields, keys, and indexes if it does not already exist.
 *
 * @param int $oldversion The version of the plugin before the upgrade.
 * @return bool True on successful upgrade.
 */
function xmldb_element_h5p_upgrade($oldversion) {
    global $DB;

    $dbman = $DB->get_manager();

    // Check if the version is less than the version where the table was added/modified.
    if ($oldversion < 2024110801) {

        // Define table element_h5p_xapistates to be created.
        $table = new xmldb_table('element_h5p_xapistates');

        // Adding fields to table element_h5p_xapistates.
        $table->add_field('id', XMLDB_TYPE_INTEGER, '18', null, XMLDB_NOTNULL, XMLDB_SEQUENCE, null);
        $table->add_field('component', XMLDB_TYPE_CHAR, '255', null, XMLDB_NOTNULL, null, null);
        $table->add_field('userid', XMLDB_TYPE_INTEGER, '18', null, null, null, null);
        $table->add_field('itemid', XMLDB_TYPE_INTEGER, '18', null, XMLDB_NOTNULL, null, null);
        $table->add_field('stateid', XMLDB_TYPE_CHAR, '255', null, null, null, null);
        $table->add_field('statedata', XMLDB_TYPE_TEXT, null, null, null, null, null);
        $table->add_field('registration', XMLDB_TYPE_CHAR, '255', null, null, null, null);
        $table->add_field('timecreated', XMLDB_TYPE_INTEGER, '18', null, XMLDB_NOTNULL, null, null);
        $table->add_field('timemodified', XMLDB_TYPE_INTEGER, '18', null, null, null, null);

        // Adding keys to table element_h5p_xapistates.
        $table->add_key('primary', XMLDB_KEY_PRIMARY, ['id']);

        // Adding indexes to table element_h5p_xapistates.
        $table->add_index('mdl_xapistat_comite_ix', XMLDB_INDEX_NOTUNIQUE, ['component', 'itemid']);
        $table->add_index('mdl_xapistat_use_ix', XMLDB_INDEX_NOTUNIQUE, ['userid']);
        $table->add_index('mdl_xapistat_tim_ix', XMLDB_INDEX_NOTUNIQUE, ['timemodified']);

        // Conditionally launch create table for element_h5p_xapistates.
        if (!$dbman->table_exists($table)) {
            $dbman->create_table($table);
        }

        // Pulse savepoint reached.
        upgrade_plugin_savepoint(true, 2024110801, 'element', 'h5p');
    }

    return true;
}
