<?php
// This file is part of the custom Moodle Bootstrap theme
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * course_renderer.php
 * @package    theme_alluri
 * @copyright  2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @author    LMSACE Dev Team
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
namespace theme_alluri\output\core;

use moodle_url;
use lang_string;
use html_writer;
use core_course_category;
use coursecat_helper;
use stdClass;
use core_course_list_element;
use context_course;

class course_renderer extends \core_course_renderer {

    /**
     * Call the frontpage slider js.
     * @param string $blockid
     * @return void
     */
    public function include_frontslide_js($blockid) {
        $this->page->requires->js_call_amd('theme_alluri/frontsliders', $blockid, []);
    }

    protected function coursecat_coursebox(coursecat_helper $chelper, $course, $additionalclasses = '') {
        global $CFG;
        if (!isset($this->strings->summary)) {
            $this->strings->summary = get_string('summary');
        }
        if ($chelper->get_show_courses() <= self::COURSECAT_SHOW_COURSES_COUNT) {
            return '';
        }
        if ($course instanceof stdClass) {
           // require_once($CFG->libdir. '/coursecatlib.php');
            $course = new core_course_list_element($course);
        }
        $content = '';
        $classes = trim('panel panel-default coursebox clearfix '. $additionalclasses);
        if ($chelper->get_show_courses() < self::COURSECAT_SHOW_COURSES_EXPANDED) {
            $classes .= ' collapsed';
        }

        // .coursebox
        $content .= html_writer::start_tag('div', array(
            'class' => $classes,
            'data-courseid' => $course->id,
            'data-type' => self::COURSECAT_TYPE_COURSE,
        ));

        $content .= html_writer::start_tag('div', array('class' => 'panel-heading info'));

        $fa_arrow = "fa-arrow-circle-right";

        if(right_to_left()) {
            $fa_arrow = "fa-arrow-circle-left";
        }
        else{
            $fa_arrow = "fa-arrow-circle-right";
        }

        // course name
        $coursename = $chelper->get_course_formatted_name($course);

        $courseurl = new moodle_url('/course/view.php', array('id' => $course->id));
        $courselink_class = $course->visible ? '' : 'dimmed';
        $coursenamelink = '<a href="'.$courseurl.'" class="'.$courselink_class.'"><i class="fa '.$fa_arrow.'"></i>'.$coursename.'</a>';

        $content .= html_writer::tag('span', $coursenamelink, array('class' => 'coursename'));
        // If we display course in collapsed form but the course has summary or course contacts, display the link to the info page.
        $content .= html_writer::start_tag('span', array('class' => 'moreinfo'));
        if ($chelper->get_show_courses() < self::COURSECAT_SHOW_COURSES_EXPANDED) {
            if ($course->has_summary() || $course->has_course_contacts() || $course->has_course_overviewfiles()) {
                $url = new moodle_url('/course/info.php', array('id' => $course->id));
                $image = html_writer::empty_tag('img', array('src' => $this->output->image_url('base/ic-info', 'theme'),
                    'alt' => $this->strings->summary));
                $content .= html_writer::link($url, $image, array('title' => $this->strings->summary));
                // Make sure JS file to expand course content is included.
                $this->coursecat_include_js();
            }
        }
        $content .= html_writer::end_tag('span'); // .moreinfo

        // print enrolmenticons
        if ($icons = enrol_get_course_info_icons($course)) {
            $content .= html_writer::start_tag('div', array('class' => 'enrolmenticons'));
            foreach ($icons as $pix_icon) {
                $content .= $this->render($pix_icon);
            }
            $content .= html_writer::end_tag('div'); // .enrolmenticons
        }

        $content .= html_writer::end_tag('div'); // .info

        $content .= html_writer::start_tag('div', array('class' => 'content panel-body'));
        $content .= $this->coursecat_coursebox_content($chelper, $course);

        if ($chelper->get_show_courses() >= self::COURSECAT_SHOW_COURSES_EXPANDED) {
            $icondirection = 'left';
            if ('ltr' === get_string('thisdirection', 'langconfig')) {
                $icondirection = 'right';
            }
            if (is_enrolled(context_course::instance($course->id))) {
                $arrow = html_writer::tag('span', '', array('class' => ' glyphicon glyphicon-arrow-'.$icondirection));
                $visit = html_writer::tag('span', get_string('course') . ' ' . $arrow);
                $visitlink = html_writer::link(new moodle_url('/course/view.php',
                    array('id' => $course->id)), $visit);
                $content .= html_writer::tag('div', $visitlink, array('class' => 'visitlink'));
            }
        }
        $content .= html_writer::end_tag('div'); // .content

        $content .= html_writer::end_tag('div'); // .coursebox
        return $content;
    }

    protected function coursecat_coursebox_content(coursecat_helper $chelper, $course) {
        global $CFG;
        if ($chelper->get_show_courses() < self::COURSECAT_SHOW_COURSES_EXPANDED) {
            return '';
        }
        if ($course instanceof stdClass) {
            //require_once($CFG->libdir. '/coursecatlib.php');
            $course = new core_course_list_element($course);
        }
        $content = '';

        // Display course overview files.
        $contentimages = $contentfiles = '';
        foreach ($course->get_course_overviewfiles() as $file) {
            $isimage = $file->is_valid_image();
            $url = file_encode_url("$CFG->wwwroot/pluginfile.php",
                    '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                    $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
            if ($isimage) {
                    $contentimages .= html_writer::start_tag('div', array('class' => 'imagebox'));

                    $images = html_writer::empty_tag('img', array('src' => $url, 'alt' => 'Course Image '. $course->get_formatted_name(),
                        'class' => 'courseimage'));
                    $contentimages .= html_writer::link(new moodle_url('/course/view.php', array('id' => $course->id)), $images);

                    $contentimages .= html_writer::end_tag('div');
            } else {
                $image = $this->output->pix_icon(file_file_icon($file, 24), $file->get_filename(), 'moodle');
                $filename = html_writer::tag('span', $image, array('class' => 'fp-icon')).
                        html_writer::tag('span', $file->get_filename(), array('class' => 'fp-filename'));
                $contentfiles .= html_writer::tag('span',
                        html_writer::link($url, $filename),
                        array('class' => 'coursefile fp-filename-icon'));
            }
        }
        $content .= $contentimages. $contentfiles;

        // Display course summary.
        if ($course->has_summary()) {
            $content .= $chelper->get_course_formatted_summary($course);
        }

        // Display course contacts. See course_in_list::get_course_contacts().
        if ($course->has_course_contacts()) {
            $content .= html_writer::start_tag('ul', array('class' => 'teachers'));
            foreach ($course->get_course_contacts() as $userid => $coursecontact) {
                $name = $coursecontact['rolename'].': '.
                        html_writer::link(new moodle_url('/user/view.php',
                                array('id' => $userid, 'course' => SITEID)),
                            $coursecontact['username']);
                $content .= html_writer::tag('li', $name);
            }
            $content .= html_writer::end_tag('ul'); // .teachers
        }

        // Display course category if necessary (for example in search results).
        if ($chelper->get_show_courses() == self::COURSECAT_SHOW_COURSES_EXPANDED_WITH_CAT) {
           // require_once($CFG->libdir. '/coursecatlib.php');
            if ($cat = core_course_category::get($course->category, IGNORE_MISSING)) {
                $content .= html_writer::start_tag('div', array('class' => 'coursecat'));
                $content .= get_string('category').': '.
                        html_writer::link(new moodle_url('/course/index.php', array('categoryid' => $cat->id)),
                                $cat->get_formatted_name(), array('class' => $cat->visible ? '' : 'dimmed'));
                $content .= html_writer::end_tag('div'); // .coursecat
            }
        }
        return $content;
    }

    public function course_search_form($value = '', $format = 'plain') {
        static $count = 0;
        $formid = 'coursesearch';
        if ((++$count) > 1) {
            $formid .= $count;
        }
        $inputid = 'coursesearchbox';
        $inputsize = 30;

        if ($format === 'navbar') {
            $formid = 'coursesearchnavbar';
            $inputid = 'navsearchbox';
        }
        /* Customize the search courses form */
        $strsearchcourses = get_string("searchcourses");
        $searchurl = new moodle_url('/course/search.php');

        $form = array('id' => $formid, 'action' => $searchurl, 'method' => 'get', 'class' => "form-inline text-right", 'role' => 'form');
        $output = html_writer::start_tag('form', $form);
        $output .= html_writer::start_div('input-group');
        $output .= html_writer::tag('label', $strsearchcourses, array('for' => $inputid, 'class' => 'sr-only'));
        $search = array('type' => 'text', 'id' => $inputid, 'size' => $inputsize, 'name' => 'search',
                        'class' => 'form-control', 'value' => s($value), 'placeholder' => $strsearchcourses);
        $output .= html_writer::empty_tag('input', $search);
        $output .= html_writer::start_span('input-group-btn');
        $output .= '<button class="btn btn-default" type="submit"><i class="fa fa-search"></i></button>';
        $output .= html_writer::end_span();
        $output .= html_writer::end_div(); // Close form-group.
        $output .= html_writer::end_tag('form');

        return $output;
    }


    public function featured_courses() {
        global $CFG,$OUTPUT,$DB;
        $featured_content = '';
        $row_content = "";
        /* Get Featured courses id from DB */
        $featured_ids = theme_alluri_get_setting('featured_courses');
        $rcourse_ids = (!empty($featured_ids)) ? explode(",",$featured_ids) : array();
        if (empty($rcourse_ids)) {
            return false;
        }

        $hcourse_ids = hidden_courses_id();
        if (!empty($hcourse_ids)) {
            foreach($rcourse_ids as $key => $val) {

                if (in_array($val,$hcourse_ids)) {
                    unset($rcourse_ids[$key]);
                    continue;
                }
            }
        }

        foreach($rcourse_ids as $key => $val) {
            $ccourse =  $DB->get_record('course', array('id' => $val));
            if (empty($ccourse)) {
                unset($rcourse_ids[$key]);
                continue;
            }
        }

        if(empty($rcourse_ids)) {
            return false;
        }

        // $fcourse_ids = array_chunk($rcourse_ids,1);
         $fcourse_ids = ($rcourse_ids);
        $total_fcourse = count($fcourse_ids);
        $fcoursetitle = theme_alluri_get_setting('fcoursestitle');
        $featured_courses_title = !empty($fcoursetitle) ? theme_alluri_lang($fcoursetitle) :  get_string('featured_courses', 'theme_alluri');

        $featured_header =  '<div id="Featured-Courses">
            <div class="container-fluid">
                <div class="title-row">
                    <h2 class="lbordered">'.$featured_courses_title.'<span class="hbar">&nbsp;</span></h2>
                    <div class="pagenav">
                        <button  class="slick-prev" type="button"></button>
                        <button  class="slick-next" type="button"></button>
                        <div class="clearfix"></div>
                    </div>
                    <div class="clearfix"></div>
                </div>

                <!--E.O.Title Row-->
                <div class="mpanel-wrap">
                <div class="featured_courses" data-crow="'.$total_fcourse.'">';

        $featured_footer = '</div>
        </div>
        </div></div>' ;

        if(!empty($fcourse_ids)) {
            foreach($fcourse_ids as $course_id) {

                // foreach($course_ids as $course_id) {
                    $course = get_course($course_id);
                    $noimg_url = $OUTPUT->image_url('no-image', 'theme');

                    $course_url = new moodle_url('/course/view.php',array('id' => $course_id ));

                    if ($course instanceof stdClass) {
                        //require_once($CFG->libdir. '/coursecatlib.php');
                        $course = new core_course_list_element($course);
                    }

                    $img_url = '';

                    $summary = strip_html_tags(format_string($course->summary));
                    $summary = course_trim_char($summary,75);

                    $context = context_course::instance($course->id);
                    $no_students = count_role_users(5,$context);


                    foreach ($course->get_course_overviewfiles() as $file) {
                        $isimage = $file->is_valid_image();
                        $img_url = file_encode_url("$CFG->wwwroot/pluginfile.php",
                        '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                        $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
                        if (!$isimage) {
                            $img_url = $noimg_url;
                        }
                    }

                    if (empty($img_url)) {
                        $img_url = $noimg_url;
                    }

                    $course_html = '<div>
                        <div class="panel mpanel dRect mpanel-shadowed">
                            <div class="mpanel-thumb">
                                <a href="'.$course_url.'" style="background:url('.$img_url.') no-repeat center center; background-size: cover;"></a>
                                <span>&nbsp;</span>
                            </div>
                            <div class="mpanel-info text-center">
                                <h3 class="nomargint"><a href="'.$course_url.'">'.$course->get_formatted_name().'</a></h3>
                                <p>'.$summary.'</p>
                                <div class="readmore">
                                    <a class="btn btn-tdefault btn-sm" href="'.$course_url.'">'.get_string('readmore', 'theme_alluri').'</a>
                                </div>
                            </div>
                        </div>
                    </div>' ;

                    $row_content .= $course_html;
                // }

                $featured_content .= $row_content;
            }
        }

        $featured_courses = $featured_header.$featured_content.$featured_footer;
        //echo $featured_courses;
        $this->include_frontslide_js('featurecourse');
        return $featured_courses;
    }

    public function popular_courses() {

        global $CFG, $OUTPUT;
        $chelper = new coursecat_helper();
        $chelper->set_show_courses(self::COURSECAT_SHOW_COURSES_EXPANDED)->set_courses_display_options(array(
            'recursive' => true,
            'limit' => $CFG->frontpagecourselimit,
            'viewmoreurl' => new moodle_url('/course/index.php'),
            'viewmoretext' => new lang_string('fulllistofcourses')
        ));

        $courses = core_course_category::get(0)->get_courses($chelper->get_courses_display_options());

        $my_courses = enrol_get_my_courses('summary, summaryformat');

        if(isloggedin()) {
            $rcourseids = array_keys($my_courses);
        } else {
            $rcourseids = array_keys($courses);
        }
            $imgurl = '';
            if (isloggedin()) {
              
                $coursetitle = theme_alluri_get_setting('ecoursestitle');
                $coursetitle = theme_alluri_lang($coursetitle);
                $courses_title = !empty($coursetitle) ? ($coursetitle) :  get_string('enrolled_courses', 'theme_alluri');
            } else {
               
                $coursetitle = theme_alluri_get_setting('acoursestitle');
                $coursetitle = theme_alluri_lang($coursetitle);
                $courses_title = !empty($coursetitle) ? ($coursetitle) :  get_string('available_courses', 'theme_alluri');
            }
           
           // $name = (isloggedin()) ? 'Enrolled courses' : 'Available Courses';

            $popular_header = html_writer::start_tag('div', array('id' => 'Popular-Courses'));
                $popular_header .= html_writer::start_tag('div', array('class' => 'container'));
                $popular_header .= html_writer::start_tag('div', array('class' => 'title-row'));
                    $popular_header .= html_writer::start_tag('h2', array('class' => 'lbordered'));
                    $popular_header .= $courses_title;
                    $popular_header .= html_writer::start_tag('span', array('class' => 'hbar'));
                    $popular_header .= '&nbsp';
                    $popular_header .= html_writer::end_tag('span');
                    $popular_header .= html_writer::end_tag('h2');
                $popular_header .= html_writer::start_tag('div', array('class' => 'pagenav'));
                    $popular_header .= html_writer::start_tag('button', array('class' => 'slick-prev', 'type' => 'button'));
                    $popular_header .= html_writer::end_tag('button');
                    $popular_header .= html_writer::start_tag('button', array('class' => 'slick-next', 'type' => 'button'));
                    $popular_header .= html_writer::end_tag('button');
                    $popular_header .= html_writer::start_tag('div', array('class' => 'clearfix'));
                    $popular_header .= html_writer::end_tag('div');
                $popular_header .= html_writer::end_tag('div');

                $popular_header .= html_writer::start_tag('div', array('class' => 'clearfix'));
                $popular_header .= html_writer::end_tag('div');
                $popular_header .= html_writer::end_tag('div');

                    $popular_header .= html_writer::start_tag('div', array('class' => 'mpanel-wrap'));
                        $popular_header .= html_writer::start_tag('div', array('class' => 'popular_courses', 'data-crow' => ''));
                        $popular_footer = html_writer::end_tag('div');
                    $popular_footer .= html_writer::end_tag('div');
                    /*   $popular_header .= html_writer::end_tag('div');*/
                    $content = $popular_header;

                $popular_footer .= html_writer::end_tag('div');
                $popular_footer .= html_writer::end_tag('div');
                //echo $popular_header;

        foreach ($rcourseids as $courseid) {

            $course = get_course($courseid);
            $courseurl = new moodle_url('/course/view.php', array('id' => $courseid ));
            $noimg_url = $OUTPUT->image_url('no-image', 'theme');
            if ($course instanceof stdClass) {
               // require_once($CFG->libdir. '/coursecatlib.php');
                $course = new core_course_list_element($course);
            }
            $imgurl='';

            foreach ($course->get_course_overviewfiles() as $file) {
                $isimage = $file->is_valid_image();
                $imgurl = file_encode_url("$CFG->wwwroot/pluginfile.php",
                    '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                    $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
                 if (!$isimage) {
                    $imgurl = $noimg_url;
                }
            }
            if (empty($imgurl)) {
                $imgurl = $noimg_url;
            }

                $popular_content = html_writer::start_tag('div', array('class' => 'col-lg-3 '));
                $popular_content .= html_writer::start_tag('div', array('class' => 'panel mpanel dSqua mpanel-bordered'));
                    $popular_content .= html_writer::start_tag('div', array('class' => 'mpanel-thumb'));
                    $popular_content .= html_writer::start_tag('a', array('href' => $courseurl, 'style' => 'background:url('.$imgurl.') no-repeat center center; background-size: cover;'));
                    $popular_content .= html_writer::end_tag('a');
                    $popular_content .= html_writer::end_tag('div');

                    $popular_content .= html_writer::start_tag('div', array('class' => 'mpanel-info'));

                        $popular_content .= html_writer::start_tag('h4', array('class' => 'nomargint'));
                        $popular_content .= html_writer::start_tag('a', array('href' => $courseurl));
                        if($course->get_formatted_name() != '') {
                        $popular_content .= $course->get_formatted_name();
                        }
                        $popular_content .= html_writer::end_tag('a');
                        $popular_content .= html_writer::end_tag('h4');

                        $popular_content .= html_writer::start_tag('p');
                    if($course->summary != '') {
                    $summary = strip_html_tags(format_string($course->summary));
                    $summary = course_trim_char($summary,75);
                        $popular_content .= $summary;
                    }
                        $popular_content .= '<br>';
                        $popular_content .= html_writer::start_tag('a', array('href' => $courseurl, 'class' => 'rmore-text'));
                        $popular_content .= get_string('readmore', 'theme_alluri');
                        $popular_content .= '&raquo';
                        $popular_content .= html_writer::end_tag('a');
                        $popular_content .= html_writer::end_tag('p');

                    $popular_content .= html_writer::end_tag('div');
                $popular_content .= html_writer::end_tag('div');
                $popular_content .= html_writer::end_tag('div');

            $content .= $popular_content;
        }

        $content .= $popular_footer;
        if(!empty($course)) {
            return $content;
        }
    }

    public function course_insights_home($f=0) {

       $countype = theme_alluri_get_setting('counttype');

            $val1 = 0;
            $val2 = 0;
            $val3 = 0;
            if ($countype == 'default') {

                $acourses = array();
                $astu = array();
                $atea = array();
            /* Get all courses */
                if ($ccc = get_courses('all', 'c.sortorder ASC', 'c.id,c.shortname,c.visible')) {

                    foreach ($ccc as $cc) {
                        if($cc->visible=="0" || $cc->id=="1") {
                            continue;
                        }
                        $val1++;
                        $context = context_course::instance($cc->id);
                        /* count no of teachers */
                        $no_teachers = count_role_users(3,$context);
                        $val2 = $val2+$no_teachers;
                        /* count no of students */
                        $no_students = count_role_users(5,$context);
                        $val3 = $val3+$no_students;
                        $acourses[] = array('cid' => $cc->id ,'students' => $no_students ,'teachers' => $no_teachers);
                        $astu[] = $no_students;
                        $atea[] = $no_teachers;
                    }
                }

                if ($f=="1") {
                    array_multisort($astu,SORT_DESC,$atea,SORT_DESC,$acourses);
                    return $acourses;
                }
                return compact('val1','val2','val3');

            } else { 

                $val1 = theme_alluri_get_setting('countBlockValue1');
                $val2 = theme_alluri_get_setting('countBlockValue2');
                $val3 = theme_alluri_get_setting('countBlockValue2');

                return compact('val1','val2','val3');

            }

    } 


    /**
     * Returns HTML to print list of courses user is enrolled to for the frontpage
     *
     * Also lists remote courses or remote hosts if MNET authorisation is used
     *
     * @return string
     */
    public function frontpage_my_courses() {
        global $USER, $CFG, $DB, $OUTPUT;

        if (!isloggedin() or isguestuser()) {
            return '';
        }

        $content = '';
        $imgurl = '';
        $courses  = enrol_get_my_courses('summary, summaryformat');

        if (empty($courses)) {
            return $content;
        }

        $rcourseids = array_keys($courses);
        $coursetitle = theme_alluri_get_setting('ecoursestitle');
        $coursetitle = theme_alluri_lang($coursetitle);
        $courses_title = !empty($coursetitle) ? ($coursetitle) :  get_string('enrolled_courses', 'theme_alluri');

            $popular_header = html_writer::start_tag('div', array('id' => 'My-Courses', 'class' => 'enrolled-course'));
            $popular_header .= html_writer::start_tag('div', array('class' => 'container-fluid'));

                $popular_header .= html_writer::start_tag('div', array('class' => 'title-row'));
                    $popular_header .= html_writer::start_tag('h2', array('class' => 'lbordered'));
                    $popular_header .= $courses_title;
                    $popular_header .= html_writer::start_tag('span', array('class' => 'hbar'));
                    $popular_header .= '&nbsp';
                    $popular_header .= html_writer::end_tag('span');
                    $popular_header .= html_writer::end_tag('h2');

                    $popular_header .= html_writer::start_tag('div', array('class' => 'pagenav'));
                        $popular_header .= html_writer::start_tag('button', array('class' => 'slick-prev', 'type' => 'button'));
                        $popular_header .= html_writer::end_tag('button');
                        $popular_header .= html_writer::start_tag('button', array('class' => 'slick-next', 'type' => 'button'));
                        $popular_header .= html_writer::end_tag('button');
                        $popular_header .= html_writer::start_tag('div', array('class' => 'clearfix'));
                        $popular_header .= html_writer::end_tag('div');
                    $popular_header .= html_writer::end_tag('div');

                    $popular_header .= html_writer::start_tag('div', array('class' => 'clearfix'));
                    $popular_header .= html_writer::end_tag('div');
                $popular_header .= html_writer::end_tag('div');

                $popular_header .= html_writer::start_tag('div', array('class' => 'mpanel-wrap'));
                    $popular_header .= html_writer::start_tag('div', array('class' => 'mycourses', 'data-crow' => ''));
                    $popular_footer = html_writer::end_tag('div');
                $popular_footer .= html_writer::end_tag('div');
                /*   $popular_header .= html_writer::end_tag('div');*/
                $content = $popular_header;



            $popular_footer .= html_writer::end_tag('div');
            $popular_footer .= html_writer::end_tag('div');
            //echo $popular_header;

            foreach ($rcourseids as $courseid) {

                $course = get_course($courseid);
                $courseurl = new moodle_url('/course/view.php', array('id' => $courseid ));
                $noimg_url = $OUTPUT->image_url('no-image', 'theme');
                if ($course instanceof stdClass) {
                   // require_once($CFG->libdir. '/coursecatlib.php');
                    $course = new core_course_list_element($course);
                }
                $imgurl='';

                foreach ($course->get_course_overviewfiles() as $file) {
                    $isimage = $file->is_valid_image();
                    $imgurl = file_encode_url("$CFG->wwwroot/pluginfile.php",
                        '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                        $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
                     if (!$isimage) {
                        $imgurl = $noimg_url;
                    }
                }
                if (empty($imgurl)) {
                    $imgurl = $noimg_url;
                }

                    $popular_content = html_writer::start_tag('div', array('class' => 'panel mpanel dSqua mpanel-bordered'));
                        $popular_content .= html_writer::start_tag('div', array('class' => 'mpanel-thumb'));
                        $popular_content .= html_writer::start_tag('a', array('href' => $courseurl, 'style' => 'background:url('.$imgurl.') no-repeat center center; background-size: cover;'));
                        $popular_content .= html_writer::end_tag('a');
                        $popular_content .= html_writer::end_tag('div');

                        $popular_content .= html_writer::start_tag('div', array('class' => 'mpanel-info'));

                            $popular_content .= html_writer::start_tag('h4', array('class' => 'nomargint'));
                            $popular_content .= html_writer::start_tag('a', array('href' => $courseurl));
                            if($course->get_formatted_name() != '') {
                            $popular_content .= $course->get_formatted_name();
                            }
                            $popular_content .= html_writer::end_tag('a');
                            $popular_content .= html_writer::end_tag('h4');

                            $popular_content .= html_writer::start_tag('p');
                        if($course->summary != '') {
                        $summary = strip_html_tags(format_string($course->summary));
                        $summary = course_trim_char($summary,75);
                            $popular_content .= $summary;
                        }
                            $popular_content .= '<br>';
                            $popular_content .= html_writer::start_tag('a', array('href' => $courseurl, 'class' => 'rmore-text'));
                            $popular_content .= get_string('readmore', 'theme_alluri');
                            $popular_content .= '&raquo';
                            $popular_content .= html_writer::end_tag('a');
                            $popular_content .= html_writer::end_tag('p');

                        $popular_content .= html_writer::end_tag('div');
                    $popular_content .= html_writer::end_tag('div');

                $content .= $popular_content;
            }
        $this->include_frontslide_js('mycourse');
        $content .= $popular_footer;
        return $content;
    }

    /**
     * Outputs contents for frontpage as configured in $CFG->frontpage or $CFG->frontpageloggedin
     *
     * @return string
     */
    public function frontpage() {
        global $CFG, $SITE;

        $output = '';
        $beforelayout = [ALLURIHOMESLIDER, ALLURIFRONTPAGEFEATURECOURSES, ALLURIFRONTPAGESITEFEATURES];
        $afterlayout = [ALLURIFRONTPAGESITEEVENTS, ALLURIFRONTPAGETESTIMONIALS, ALLURIFRONTPAGEMARKETINGSPOT];
        if (isloggedin() and !isguestuser() and isset($CFG->frontpageloggedin)) {
            $frontpagelayout = explode(",", $CFG->frontpageloggedin);
        } else {
            $frontpagelayout = explode(",", $CFG->frontpage);
        }
        $allurifrontpagelayout = array_merge($beforelayout, $frontpagelayout, $afterlayout);
        $alluriblocks = new \theme_alluri\alluri_blocks;
        foreach ($allurifrontpagelayout as $v) {
            switch ($v) {
                // Display the main part of the front page.
                case FRONTPAGENEWS:
                    if ($SITE->newsitems) {
                        // Print forums only when needed.
                        require_once($CFG->dirroot .'/mod/forum/lib.php');
                        if (($newsforum = forum_get_course_forum($SITE->id, 'news')) &&
                                ($forumcontents = $this->frontpage_news($newsforum))) {
                            $newsforumcm = get_fast_modinfo($SITE)->instances['forum'][$newsforum->id];
                            $output .= $this->frontpage_part('skipsitenews', 'site-news-forum',
                                $newsforumcm->get_formatted_name(), $forumcontents);
                        }
                    }
                    break;

                case FRONTPAGEENROLLEDCOURSELIST:
                    $mycourseshtml = $this->frontpage_my_courses();
                    $output .= $mycourseshtml;
                    break;

                case FRONTPAGEALLCOURSELIST:
                    $availablecourseshtml = $this->frontpage_available_courses();
                    $output .= $availablecourseshtml;
                    break;

                case FRONTPAGECATEGORYNAMES:
                    $output .= $this->frontpage_part('skipcategories', 'frontpage-category-names',
                        get_string('categories'), $this->frontpage_categories_list());
                    break;

                case FRONTPAGECATEGORYCOMBO:
                    $output .= $this->frontpage_part('skipcourses', 'frontpage-category-combo',
                        get_string('courses'), $this->frontpage_combo_list());
                    break;

                case FRONTPAGECOURSESEARCH:
                    $output .= $this->box($this->course_search_form(''), 'd-flex justify-content-center');
                    break;
                case ALLURIFRONTPAGEFEATURECOURSES:
                    $output .= $this->featured_courses();
                    break;
                case ALLURIFRONTPAGESITEFEATURES:
                    $output .= $alluriblocks->sitefeatures();
                    break;
                case ALLURIFRONTPAGESITEEVENTS:
                    $output .= $alluriblocks->siteevents();
                    break;
                case ALLURIFRONTPAGETESTIMONIALS:
                    $output .= $alluriblocks->testimonials();
                    break;
                case ALLURIFRONTPAGEMARKETINGSPOT:
                    $output .= $alluriblocks->marketingspot();
                    break;
                case ALLURIHOMESLIDER:
                    $output .= $alluriblocks->alluri_homebanner();
                    break;
            }
            //$output .= '<br />';
        }

        return $output;
    }

        /**
     * Returns HTML to print list of available courses for the frontpage
     *
     * @return string
     */
    public function frontpage_available_courses() {
        global $CFG, $OUTPUT;

        $chelper = new coursecat_helper();
        $chelper->set_show_courses(self::COURSECAT_SHOW_COURSES_EXPANDED)->
                set_courses_display_options(array(
                    'recursive' => true,
                    'limit' => $CFG->frontpagecourselimit,
                    'viewmoreurl' => new moodle_url('/course/index.php'),
                    'viewmoretext' => new lang_string('fulllistofcourses')));

        $chelper->set_attributes(array('class' => 'frontpage-course-list-all'));
        $courses = core_course_category::top()->get_courses($chelper->get_courses_display_options());
        $content = '';
        if (empty($courses)) {
            return $content;
        }

        $rcourseids = array_keys($courses);

        $coursetitle = theme_alluri_get_setting('acoursestitle');
        $coursetitle = theme_alluri_lang($coursetitle);
        $courses_title = !empty($coursetitle) ? ($coursetitle) :  get_string('available_courses', 'theme_alluri');

            $popular_header = html_writer::start_tag('div', array('id' => 'Available-Courses', 'class' => 'available-course'));
            $popular_header .= html_writer::start_tag('div', array('class' => 'container-fluid'));

                $popular_header .= html_writer::start_tag('div', array('class' => 'title-row'));
                    $popular_header .= html_writer::start_tag('h2', array('class' => 'lbordered'));
                    $popular_header .= $courses_title;
                    $popular_header .= html_writer::start_tag('span', array('class' => 'hbar'));
                    $popular_header .= '&nbsp';
                    $popular_header .= html_writer::end_tag('span');
                    $popular_header .= html_writer::end_tag('h2');

                    $popular_header .= html_writer::start_tag('div', array('class' => 'pagenav'));
                        $popular_header .= html_writer::start_tag('button', array('class' => 'slick-prev', 'type' => 'button'));
                        $popular_header .= html_writer::end_tag('button');
                        $popular_header .= html_writer::start_tag('button', array('class' => 'slick-next', 'type' => 'button'));
                        $popular_header .= html_writer::end_tag('button');
                        $popular_header .= html_writer::start_tag('div', array('class' => 'clearfix'));
                        $popular_header .= html_writer::end_tag('div');
                    $popular_header .= html_writer::end_tag('div');

                    $popular_header .= html_writer::start_tag('div', array('class' => 'clearfix'));
                    $popular_header .= html_writer::end_tag('div');
                $popular_header .= html_writer::end_tag('div');

                $popular_header .= html_writer::start_tag('div', array('class' => 'mpanel-wrap'));
                    $popular_header .= html_writer::start_tag('div', array('class' => 'availablecourses', 'data-crow' => ''));
                    $popular_footer = html_writer::end_tag('div');
                $popular_footer .= html_writer::end_tag('div');
                /*   $popular_header .= html_writer::end_tag('div');*/
                $content = $popular_header;



            $popular_footer .= html_writer::end_tag('div');
            $popular_footer .= html_writer::end_tag('div');
            //echo $popular_header;

            foreach ($rcourseids as $courseid) {

                $course = get_course($courseid);
                $courseurl = new moodle_url('/course/view.php', array('id' => $courseid ));
                $noimg_url = $OUTPUT->image_url('no-image', 'theme');
                if ($course instanceof stdClass) {
                   // require_once($CFG->libdir. '/coursecatlib.php');
                    $course = new core_course_list_element($course);
                }
                $imgurl='';

                foreach ($course->get_course_overviewfiles() as $file) {
                    $isimage = $file->is_valid_image();
                    $imgurl = file_encode_url("$CFG->wwwroot/pluginfile.php",
                        '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                        $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
                     if (!$isimage) {
                        $imgurl = $noimg_url;
                    }
                }
                if (empty($imgurl)) {
                    $imgurl = $noimg_url;
                }

                    $popular_content = html_writer::start_tag('div', array('class' => 'panel mpanel dSqua mpanel-bordered'));
                        $popular_content .= html_writer::start_tag('div', array('class' => 'mpanel-thumb'));
                        $popular_content .= html_writer::start_tag('a', array('href' => $courseurl, 'style' => 'background:url('.$imgurl.') no-repeat center center; background-size: cover;'));
                        $popular_content .= html_writer::end_tag('a');
                        $popular_content .= html_writer::end_tag('div');

                        $popular_content .= html_writer::start_tag('div', array('class' => 'mpanel-info'));

                            $popular_content .= html_writer::start_tag('h4', array('class' => 'nomargint'));
                            $popular_content .= html_writer::start_tag('a', array('href' => $courseurl));
                            if($course->get_formatted_name() != '') {
                            $popular_content .= $course->get_formatted_name();
                            }
                            $popular_content .= html_writer::end_tag('a');
                            $popular_content .= html_writer::end_tag('h4');

                            $popular_content .= html_writer::start_tag('p');
                        if($course->summary != '') {
                        $summary = strip_html_tags(format_string($course->summary));
                        $summary = course_trim_char($summary,75);
                            $popular_content .= $summary;
                        }
                            $popular_content .= '<br>';
                            $popular_content .= html_writer::start_tag('a', array('href' => $courseurl, 'class' => 'rmore-text'));
                            $popular_content .= get_string('readmore', 'theme_alluri');
                            $popular_content .= '&raquo';
                            $popular_content .= html_writer::end_tag('a');
                            $popular_content .= html_writer::end_tag('p');

                        $popular_content .= html_writer::end_tag('div');
                    $popular_content .= html_writer::end_tag('div');

                $content .= $popular_content;
            }
        $this->include_frontslide_js('availablecourse');
        $content .= $popular_footer;
        return $content;


    }

    /**
     * Returns HTML to display a tree of subcategories and courses in the given category
     *
     * @param coursecat_helper $chelper various display options
     * @param core_course_category $coursecat top category (this category's name and description will NOT be added to the tree)
     * @return string
     */
    protected function coursecat_tree(coursecat_helper $chelper, $coursecat) {
        // Reset the category expanded flag for this course category tree first.
        $this->categoryexpandedonload = false;
        $categorycontent = $this->coursecat_category_content($chelper, $coursecat, 0);
        if (empty($categorycontent)) {
            return '';
        }

        // Start content generation
        $content = '';
        $attributes = $chelper->get_and_erase_attributes('course_category_tree clearfix');
        $content .= html_writer::start_tag('div', $attributes);

        if ($coursecat->get_children_count()) {
            $classes = array(
                'collapseexpand', 'aabtn'
            );

            // Check if the category content contains subcategories with children's content loaded.
            if ($this->categoryexpandedonload) {
                $classes[] = 'collapse-all';
                $linkname = get_string('collapseall');
            } else {
                $linkname = get_string('expandall');
            }

            // Only show the collapse/expand if there are children to expand.
            $content .= html_writer::start_tag('div', array('class' => 'collapsible-actions'));
            $content .= html_writer::link('#', $linkname, array('class' => implode(' ', $classes)));
            $content .= html_writer::end_tag('div');
            $this->page->requires->strings_for_js(array('collapseall', 'expandall'), 'moodle');
        }

        $content .= html_writer::tag('div', $categorycontent, array('class' => 'content'));

        $content .= html_writer::end_tag('div'); // .course_category_tree

        return $content;
    }

        /**
     * Renders part of frontpage with a skip link (i.e. "My courses", "Site news", etc.)
     *
     * @param string $skipdivid
     * @param string $contentsdivid
     * @param string $header Header of the part
     * @param string $contents Contents of the part
     * @return string
     */
    protected function frontpage_part($skipdivid, $contentsdivid, $header, $contents) {
        if (strval($contents) === '') {
            return '';
        }
        $output = html_writer::link('#' . $skipdivid,
            get_string('skipa', 'access', \core_text::strtolower(strip_tags($header))),
            array('class' => 'skip-block skip aabtn'));

        // Wrap frontpage part in div container.
        $output .= html_writer::start_tag('div', array('id' => $contentsdivid));
        $output .= html_writer::start_tag('div', array('class' => "container-fluid"));
        $output .= $this->heading($header);

        $output .= $contents;
        $output .= html_writer::end_tag('div');
        // End frontpage part div container.
        $output .= html_writer::end_tag('div');

        $output .= html_writer::tag('span', '', array('class' => 'skip-block-to', 'id' => $skipdivid));
        return $output;
    }
}
