<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * List the available menus and manage the menu Create, Update, Delete actions, sort the order of menus.
 *
 * @package    theme_boost_union
 * @copyright  bdecent GmbH 2023
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

// Require config.
require(__DIR__.'/../../../config.php');

// Require admin library.
require_once($CFG->libdir.'/adminlib.php');
require_once($CFG->dirroot. '/theme/boost_union/smartmenus/menulib.php');

// Smart menu ID to edit.
$id = optional_param('id', null, PARAM_INT);
$action = optional_param('action', null, PARAM_ALPHAEXT);

// Page values.
$url = new moodle_url('/theme/boost_union/smartmenus/menus.php');
$context = \context_system::instance();

// Extend the features of admin settings.
admin_externalpage_setup('theme_boost_union_smartmenus');

// Setup page values.
$PAGE->set_url($url);
$PAGE->set_context($context);
$PAGE->set_heading(theme_boost_union_get_externaladminpage_heading());

// Setup the breadcrums for qick access.
$PAGE->navbar->add(get_string('themes', 'core'), new moodle_url('/admin/category.php', array('category' => 'themes')));
$PAGE->navbar->add(get_string('pluginname', 'theme_boost_union'), new moodle_url('/admin/category.php',
    array('category' => 'theme_boost_union'))
);
$PAGE->navbar->add(get_string('smartmenus', 'theme_boost_union'), new moodle_url('/theme/boost_union/smartmenus/menus.php'));

// Verfiy the user session.
if ($action !== null && confirm_sesskey()) {
    // Every action is based on menu, Menu id param should exist.
    $id = required_param('id', PARAM_INT);
    // Create menu instance. Actions are performed in smartmenu instance.
    $menu = theme_boost_union\smartmenu::instance($id);

    $transaction = $DB->start_delegated_transaction();
    // Perform the requested action.
    switch ($action) :
        // Triggered action is delete, then init the deletion of menu.
        case 'delete':
            // Delete the menu.
            if ($menu->delete_menu()) {
                // Notification to user for menu deleted success.
                \core\notification::success(get_string('smartmenu:menudeleted', 'theme_boost_union'));
            }
            break;
        // Move the menu order to down.
        case "movedown":
            // Move the menu downwards.
            $menu->move_downward();
            break;
        case "moveup":
            // Move the menu upwards.
            $menu->move_upward();
            break;
        case "copy":
            // Duplicate the menu and it items.
            $menu->duplicate();
            break;
        case "hidemenu":
            // Disable the menu visibility.
            $menu->update_visible(false);
            break;
        case "showmenu":
            // Enable the menu.
            $menu->update_visible(true);
            break;
    endswitch;

    // Allow to update the changes to database.
    $transaction->allow_commit();
    // End of any action redirect to overview page for clear the params from url.
    redirect($url);
}

// Page content display started.
echo $OUTPUT->header();

// Smart menu heading.
echo $OUTPUT->heading(get_string('smartmenus', 'theme_boost_union'));

// Smart menu description.
echo $OUTPUT->box(get_string('smartmenus_desc', 'theme_boost_union'));

// Add the create item and create menu buttons.
echo smartmenu_helper::theme_boost_union_smartmenu_buttons();

// Menus list table.
$table = new theme_boost_union\table\smartmenu($context->id);
$table->out(10, true);

// Footer.
echo $OUTPUT->footer();
