<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Additional custom blocks.
 *
 * @package    theme_educli
 * @copyright  2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @authors    LMSACE Dev Team
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace theme_educli;
use coursecat_helper;
use moodle_url;
use lang_string;


/**
 * Add a additional custom block for frontpage.
 */
class customblocks {

    public function courseinfo($chelper, $course) {
        global $OUTPUT, $CFG;
        $teacher = [];
        if (!empty($course)) {
            $data = array();
            require_once($CFG->dirroot.'/theme/educli/lib.php');
            $data = theme_educli_get_course_data($course);
            return $OUTPUT->render_from_template('theme_educli/courseoverview', $data);
        }
    }

    public function available_courses() {
        global $DB, $CFG, $OUTPUT;
        require_once($CFG->dirroot."/course/renderer.php");
        $noimgurl = $OUTPUT->image_url('no-image', 'theme');
        $chelper = new coursecat_helper();
        $chelper->set_show_courses(\core_course_renderer::COURSECAT_SHOW_COURSES_EXPANDED)->
                set_courses_display_options(array(
                    'recursive' => true,
                    'limit' => THEME_EDUCLI_DASHBOARD_AVAILABLE_COURSE,
                    'viewmoreurl' => new moodle_url('/course/index.php'),
                    'viewmoretext' => new lang_string('fulllistofcourses')));
        $chelper->set_attributes(array('class' => 'frontpage-course-list-all'));
        $courses = \core_course_category::top()->get_courses($chelper->get_courses_display_options());
        $data = self::get_course_data($courses, 'dashboardavailablecourses');
        return $OUTPUT->render_from_template('theme_educli/customblocks', $data);
    }

    public function my_courses() {
        global $OUTPUT;
        $courses  = enrol_get_my_courses('summary, summaryformat');
        $courses = array_slice($courses, 0, THEME_EDUCLI_DASHBOARD_MY_COURSE);
        $data = self::get_course_data($courses, 'dashboardmycourses');
        return $OUTPUT->render_from_template('theme_educli/customblocks', $data);
    }

    public static function get_course_data($courses, $block) {
        global $CFG, $USER, $OUTPUT;
        $courselist = [];
        $noimgurl = $OUTPUT->image_url('no-image', 'theme');
        foreach ($courses as $course) {
            if (!$course instanceof \core_course_list_element) {
                $course = new \core_course_list_element($course);
            }
            $list = [];
            $list['visible'] = $course->visible;
            $list['coursename'] = $course->get_formatted_name();
            $list['courseurl'] = new moodle_url('/course/view.php', array('id' => $course->id));
            $list['courseimgurl'] = $noimgurl;
            foreach ($course->get_course_overviewfiles() as $file) {
                $isimage = $file->is_valid_image();
                $list['courseimgurl'] = file_encode_url("$CFG->wwwroot/pluginfile.php",
                    '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                    $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
                if (!$isimage) {
                    $data['courseimgurl'] = $noimgurl;
                }
            }
            $list['visible'] = $course->visible;
            $courseprogress = \core_completion\progress::get_course_progress_percentage($course, $USER->id);
            $list['courseprogress'] = (int) $courseprogress;
            $courselist[] = $list;
        }
        $data['courses'] = $courselist;
        $data['viewcourse'] = new moodle_url("/course/index.php");
        $data['mycourses'] = new moodle_url("/my/courses.php");
        $data[$block] = true;
        return $data;
    }



    public function get_enrolled_courses() {
        global $DB;
    }

    /**
     * Get a config settings from home banner.
     * @return array $data config data for home banner settings.
     */
    public function homebanner() {
        $numberofquicklinks = theme_educli_get_setting('numberofquicklinks');
        $data = [
            'homebannertitle' => theme_educli_get_setting('homebannertitle'),
            'homebannerdesc'=> theme_educli_get_setting('homebannerdesc'),
            'homebannerbtn' => theme_educli_get_setting('homebannerbtn'),
            'homebannerbtnurl' => theme_educli_get_setting('homebannerbtnurl'),
        ];
        $quicklinkconfig = [
            'linktitle' => ['link{i}title'],
            'linkurl' => ['link{i}url'],
            'linkimg' => ['link{i}img', 'file'],
        ];
        $data['quicklinks'] = theme_educli_get_items_config($quicklinkconfig, $numberofquicklinks);
        return $data;
    }

    /**
     * Get a config settings from site features block.
     * @return array $data config data for site feature settings.
     */
    public function sitefeatures() {
        $numberofsitefeatures = theme_educli_get_setting('numberofsitefeature');
        $data = [
            'siteblocktitle' => theme_educli_get_setting('siteblocktitle'),
            'siteblockdesc' => theme_educli_get_setting('siteblockdesc')
        ];
        $sitefeatureconfig = [
            'sitefeaturetitle' => ['sitefeature{i}title'],
            'sitefeaturedesc' => ['sitefeature{i}desc'],
            'sitefeatureimg' => ['sitefeature{i}img', 'file'],
        ];
        $data['sitefeatures'] = theme_educli_get_items_config($sitefeatureconfig, $numberofsitefeatures);
        return $data;
    }

    /**
     * Get a config settings from our partners block.
     * @return array $data config data for our partners settings.
     */
    public function ourpartners() {
        $numberofpartners = get_config('theme_educli', 'numberofpartners');
        $data = [
            'ourpartnerstitle' => theme_educli_get_setting('ourpartnerstitle')
        ];
        $configlist  = array(
            'partnername' => ['ourpartners{i}name'],
            'partnerimg' => ['ourpartners{i}img', 'file']
        );
        $data['partners'] = theme_educli_get_items_config($configlist, $numberofpartners);
        return $data;
    }
}
