<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * lib function for the theme educli.
 *
 * @package    theme_educli
 * @copyright  2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @author    LMSACE Dev Team
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

define("THEME_EDUCLI_DASHBOARD_AVAILABLE_COURSE", 6);
define("THEME_EDUCLI_DASHBOARD_MY_COURSE", 2);

require_once($CFG->dirroot. "/theme/educli/classes/form/user_profile.php");

// Logo Image URL Fetch from theme settings.
if (!function_exists('theme_educli_get_logo_url')) {
    /**
     * Get a theme educli logo.
     * @return type|string
     */
    function theme_educli_get_logo_url() {
        static $theme;
        if (empty($theme)) {
            $theme = theme_config::load('educli');
        }
        $logo = $theme->setting_file_url('logo', 'logo');
        $logo = empty($logo) ? '' : $logo;
        return $logo;
    }
}


/**
 * Serves any files associated with the theme settings.
 *
 * @param stdClass $course
 * @param stdClass $cm
 * @param context $context
 * @param string $filearea
 * @param array $args
 * @param bool $forcedownload
 * @param array $options
 * @return bool
 */
function theme_educli_pluginfile($course, $cm, $context, $filearea, $args, $forcedownload, array $options = array()) {
    static $theme;

    if (empty($theme)) {
        $theme = theme_config::load('educli');
    }
    if ($context->contextlevel == CONTEXT_SYSTEM) {

        if ($filearea === 'logo') {
            return $theme->setting_file_serve('logo', $args, $forcedownload, $options);
        } else if ($filearea === 'homebannerbgimg') {
            return $theme->setting_file_serve('homebannerbgimg', $args, $forcedownload, $options);
        } else if ($filearea === 'loginpageimg') {
            return $theme->setting_file_serve('loginpageimg', $args, $forcedownload, $options);
        } else if ($filearea === 'headerbgimg') {
            return $theme->setting_file_serve('headerbgimg', $args, $forcedownload, $options);
        } else if (preg_match("/link*[1-9][0-9]img/", $filearea) !== false) {
            return $theme->setting_file_serve($filearea, $args, $forcedownload, $options);
        } else if (preg_match("/sitefeature*[1-9][0-9]img/", $filearea) !== false) {
            return $theme->setting_file_serve($filearea, $args, $forcedownload, $options);
        } else if (preg_match("/ourpartners*[1-5][0-5]img/", $filearea) !== false) {
            return $theme->setting_file_serve($filearea, $args, $forcedownload, $options);
        } else {
            send_file_not_found();
        }
    } else {
        send_file_not_found();
    }
}

/**
 * Get a theme settings.
 *
 * @param string $setting
 * @param bool $format
 * @return string
 */
function theme_educli_get_setting($setting, $format = true) {
    global $CFG, $PAGE;
    require_once($CFG->dirroot . '/lib/weblib.php');
    static $theme;
    if (empty($theme)) {
        $theme = theme_config::load('educli');
    }
    if (empty($theme->settings->$setting)) {
        return false;
    } else if (!$format) {
        return $theme->settings->$setting;
    } else if ($format === 'format_text') {
        return format_text($theme->settings->$setting, FORMAT_PLAIN);
    } else if ($format === 'format_html') {
        return format_text($theme->settings->$setting, FORMAT_HTML, array('trusted' => true, 'noclean' => true));
    } else if ($format === 'file') {
        return $PAGE->theme->setting_file_url($setting, $setting);
    } else {
        return format_string($theme->settings->$setting);
    }
}

/**
 * Loads the CSS Styles and replace the background images.
 * If background image not available in the settings take the default images.
 *
 * @param string $css
 * @param object $theme
 * @return string
 */
function theme_educli_process_css($css, $theme) {
    $bgimgs = [
        'homebannerbgimg' => '',
        'loginpageimg' => '',
        'headerbgimg' => ''
    ];
    foreach ($bgimgs as $bgimg => $bgimgname) {
        $setting = $bgimg;
        $bgimage = $theme->setting_file_url($setting, $setting);
        if (empty($bgimage)) {
            $bgimage = '';
        }
        $css = theme_educli_load_bgimg_css($css, $bgimage, $setting);
    }
    return $css;
}


/**
 * Load the css for background images.
 * @param string $css
 * @param string $bgimage
 * @param string $setting
 * @return string
 */
function theme_educli_load_bgimg_css($css, $bgimage, $setting) {
    $tag = '[[' . $setting . ']]';
    if (!($bgimage)) {
        $replacement = '';
    } else {
        $replacement = $bgimage;
    }
    $css = str_replace($tag, $replacement, $css);
    return $css;
}

/**
 * Get a config items form settings.
 *
 * @param array $configlist The setting config list.
 * @param int $count.
 * @return array $items.
 */
function theme_educli_get_items_config($configlist=[], $count=null) {
    $items = [];
    if (!empty($count)) {
        for ($i = 1; $i <= $count; $i++) {
            $block = [];
            foreach ($configlist as $key => $config) {
                $setting = $config[0];
                $newconfig = preg_replace("/{i}/i", $i , $setting); // Convert the config with count variable to variable.
                $type = (isset($config[1])) ? $config[1] : '';
                $block[$key] = theme_educli_get_setting($newconfig, $type);
            }
            $items[] = $block;
        }
    }
    return isset($items) ? $items : [];
}

/**
 * Defines theme educli nodes for my profile navigation tree.
 *
 * @param \core_user\output\myprofile\tree $tree Tree object
 * @param stdClass $user user object
 * @param bool $iscurrentuser is the user viewing profile, current user ?
 * @param stdClass $course course object
 *
 * @return bool
 */
function theme_educli_myprofile_navigation(core_user\output\myprofile\tree $tree, $user, $iscurrentuser, $course) {
    global $USER, $CFG, $DB, $PAGE;
    // Get the myprofile category.
    if (!array_key_exists('myprofile', $tree->__get('categories'))) {
        $myprofilecat = new core_user\output\myprofile\category('myprofile', '', 'privacyandpolicies');
        $tree->add_category($myprofilecat);
    } else {
        // Get the existing category.
        $myprofilecat = $tree->__get('categories')['myprofile'];
    }
    if ($iscurrentuser) {
        $profile = html_writer::start_tag('div', array('class' => 'myprofile_navigation'));
        $form = user_profile::user_profile_form();
        $profile .= $form->render();
        $profile .= html_writer::end_tag('div');
        $node = new core_user\output\myprofile\node('myprofile','myprofile', $profile, null, null, '', null, 'user_profile');
        $tree->add_node($node);
    }
}


function theme_educli_get_course_data($course) {
    global $OUTPUT, $CFG, $USER, $PAGE;
    $data = [];
    if (!empty($course)) {
        $courseid = $course->id;
        $noimgurl = $OUTPUT->image_url('no-image', 'theme');
        $list['courseurl'] = new \moodle_url('/course/view.php',['id' => $course->id]);
		$list['categoryurl'] = new \moodle_url('/course/index.php',['categoryid' => $course->category]);
        $data['enrolurl'] = new moodle_url('/enrol/index.php', array('id' => $courseid));
        $modinfo = get_fast_modinfo($course);
        $totalactivities = count(course_modinfo::get_array_of_activities($course));
        if ($course instanceof stdClass) {
            $course = new core_course_list_element($course);
        }
        $imgurl = '';
        $languages = get_string_manager()->get_list_of_translations();
        $courselang = !empty($course->lang) ? $languages[$course->lang] : get_string('defaultcourselang', 'theme_educli');
        $data['lang'] = preg_replace('/\s*\([^\)]+\)/', '', $courselang);
        $data['visible'] = $course->visible;
        $data['id'] = $course->id;
        $data['name'] = $course->get_formatted_name();
        $data['shortname'] = $course->get_formatted_shortname();
        $data['startdate'] = $course->startdate;
        $data['enddate'] = $course->enddate;
        $summaryCon = trim(theme_educli_strip_html_tags($course->summary));
        $data['trimsummary'] = theme_educli_course_trim_char($summaryCon, 150);
        $data['summary'] = trim($course->summary);
        $context = context_course::instance($course->id);
        $data['iscurrentuserenrol'] = is_enrolled($context, $USER->id, '', true);
        $studentroles = array_keys(get_archetype_roles('student'));
        $data['totalstudents'] = count_role_users($studentroles, $context);
        $data['totalactivities'] = $totalactivities;
        $teacherroles = array_keys(get_archetype_roles('editingteacher'));
        $data['totalteachers'] = count_role_users($teacherroles , $context);
        $teachers = theme_educli_get_teachers($teacherroles, $context);
        $data['teacher_list'] = array();
        foreach ($teachers as $key => $val) {
            $user = core_user::get_user($val->id);
            $userpicture = new user_picture($user);
            $userpicture->size = 200; // Size f1.
            $usercontext = context_user::instance($user->id);
            $user->description = file_rewrite_pluginfile_urls($user->description, 'pluginfile.php', $usercontext->id, 'user',
                                                          'profile', null);
            $userdesc = format_text($user->description, $user->descriptionformat);
            list($teachercourses, $teacherstudents) = theme_educli_get_teacher_courses_info($user);
            $data['teacher_list'][] = array(
                'teacherimg' => $userpicture->get_url($PAGE),
                'username' => fullname($user),
                'userprofile' => new moodle_url('/user/profile.php', ['id' => $val->id]), 'userdesc' => $userdesc,
                'teachercourses' => $teachercourses,
                'teacherstudents' => $teacherstudents,
            );
        }
        $category = \core_course_category::get($course->category);
        $data['category'] = $category->get_formatted_name();
        $data['courseimgurl'] = $noimgurl;
        foreach ($course->get_course_overviewfiles() as $file) {
            $isimage = $file->is_valid_image();
            $data['courseimgurl'] = file_encode_url("$CFG->wwwroot/pluginfile.php",
                '/'. $file->get_contextid(). '/'. $file->get_component(). '/'.
                $file->get_filearea(). $file->get_filepath(). $file->get_filename(), !$isimage);
            if (!$isimage) {
                $data['courseimgurl'] = $noimgurl;
            }
        }
        if (empty($data['courseimgurl'])) {
            $data['courseimgurl'] = $noimgurl;
        }
        $data['grudatedpix'] = $OUTPUT->image_url('graduated-in', 'theme_educli');
        $data['teacherpix'] = $OUTPUT->image_url('teacher-in', 'theme_educli');
        $data['graduateovepix'] = $OUTPUT->image_url('graduate-ove', 'theme_educli');
        $data['knowledgeovepix'] = $OUTPUT->image_url('knowledge-ove', 'theme_educli');
        $data['calendarovepix'] = $OUTPUT->image_url('calendar-ove', 'theme_educli');
        $data['selfimprovementovepix'] = $OUTPUT->image_url('self-improvement-ove', 'theme_educli');
        $data['languageovepix'] = $OUTPUT->image_url('language-ove', 'theme_educli');
        $data['startdate'] = ($course->startdate) ? userdate($course->startdate, '%d/%m/%Y, %I:%M %p', '', false) : '';
        $data += theme_educli_get_section_data($course);
    }
    return $data;
}

/**
 * Remove the html special tags from course content.
 * This function used in course home page.
 *
 * @param string $text
 * @return string
 */
function theme_educli_strip_html_tags( $text ) {
    $text = preg_replace(
        array(
            // Remove invisible content.
            '@<head[^>]*?>.*?</head>@siu',
            '@<style[^>]*?>.*?</style>@siu',
            '@<script[^>]*?.*?</script>@siu',
            '@<object[^>]*?.*?</object>@siu',
            '@<embed[^>]*?.*?</embed>@siu',
            '@<applet[^>]*?.*?</applet>@siu',
            '@<noframes[^>]*?.*?</noframes>@siu',
            '@<noscript[^>]*?.*?</noscript>@siu',
            '@<noembed[^>]*?.*?</noembed>@siu',
            // Add line breaks before and after blocks.
            '@</?((address)|(blockquote)|(center)|(del))@iu',
            '@</?((div)|(h[1-9])|(ins)|(isindex)|(p)|(pre))@iu',
            '@</?((dir)|(dl)|(dt)|(dd)|(li)|(menu)|(ol)|(ul))@iu',
            '@</?((table)|(th)|(td)|(caption))@iu',
            '@</?((form)|(button)|(fieldset)|(legend)|(input))@iu',
            '@</?((label)|(select)|(optgroup)|(option)|(textarea))@iu',
            '@</?((frameset)|(frame)|(iframe))@iu',
        ),
        array(
            ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
            "\n\$0", "\n\$0", "\n\$0", "\n\$0", "\n\$0", "\n\$0",
            "\n\$0", "\n\$0",
        ),
        $text
    );
    return strip_tags( $text );
}


/**
 * Cut the Course content.
 *
 * @param $str
 * @param $n
 * @param $end_char
 * @return string
 */
function theme_educli_course_trim_char($str, $n = 500, $endchar = '&#8230;') {
    if (strlen($str) < $n) {
        return $str;
    }
    $str = preg_replace("/\s+/", ' ', str_replace(array("\r\n", "\r", "\n"), ' ', $str));
    if (strlen($str) <= $n) {
        return $str;
    }
    $out = "";
    $small = substr($str, 0, $n);
    $out = $small.$endchar;
    return $out;
}

function theme_educli_get_teacher_courses_info($teacher) {
    global $DB;
    $params = ['userid' => $teacher->id];
    $studentroles = array_keys(get_archetype_roles('student'));
    list($studentsql, $studentparams) = $DB->get_in_or_equal($studentroles, SQL_PARAMS_NAMED);
    $sql = "SELECT count(ra.id)
        FROM {user} u
        JOIN {role_assignments} ra ON ra.userid = u.id
        JOIN {context} ctx ON ctx.id = ra.contextid AND ctx.contextlevel = 50
        JOIN {course} c ON c.id = ctx.instanceid
        WHERE ra.roleid $studentsql AND u.id = :userid";
    $params += $studentparams;
    $teachercourses = $DB->count_records_sql($sql, $params);


    $teacherroles = array_keys(get_archetype_roles('editingteacher'));
    list($teachersql, $teacherparams) = $DB->get_in_or_equal($teacherroles, SQL_PARAMS_NAMED);
    $params += $teacherparams;
    $sql = "SELECT count(userid) AS students
        FROM {role_assignments} ra
        WHERE ra.roleid $studentsql
        AND ra.contextid IN (
            SELECT contextid
            FROM {role_assignments}
            WHERE roleid $teachersql AND userid = :userid
        )
    ";
    $students = $DB->count_records_sql($sql, $params);
    return [$teachercourses, $students];
}


function theme_educli_get_section_data($course) {
    global $OUTPUT;
    $modinfo = get_fast_modinfo($course);
    $data = [];
    $sectiondata = [];
    $sectionsnum = 0;
    $sectionscomplete = 0;
    $sectionnum = 1;
    foreach ($modinfo->get_section_info_all() as $section => $thissection) {
        $sectionlist = [];
        if (!isset($modinfo->sections[$thissection->section])) {
            continue;
        }
        $sectionlist['sectionnum'] = str_pad($sectionnum, 2, '0', STR_PAD_LEFT);;
        $sectionlist['sectionurl'] = new moodle_url('/course/view.php', array('id' => $course->id, 'section' => $thissection->section));
        $sectionlist['sectionname'] = get_section_name($course, $thissection);
        $sectionlist['sectionprogress'] = isset($sectionprogress) ? round($sectionprogress) : '';
        $sectionlist['modules'] = count($modinfo->sections[$thissection->section]);
        $sectionlist['sectionrestrict'] = (!$thissection->uservisible && $thissection->availableinfo) ? true : false;
        list($issectioncompleted, $issectioninprogress, $issectionnotyet) = theme_educli_is_section_completed($thissection, $course, $modinfo);
        $sectionlist['issectioncompleted'] = $issectioncompleted;
        $sectionlist['issectioninprogress'] = $issectioninprogress;
        $sectionlist['issectionnotyet'] = $issectionnotyet;
        $sectionsnum++;
        $sectionnum++;
        ($issectioncompleted == true) ? $sectionscomplete++ : false;
        $sectiondata[] = $sectionlist;
    }
    $data['sectiondata'] = $sectiondata;
    $data['sectionscomplete'] = $sectionscomplete;
    $data['sectionsnum'] = $sectionsnum;
    $data['course_sectionprogress'] = ($sectionsnum) ? round($sectionscomplete / $sectionsnum * 100) : 0;
    return $data;
}

function theme_educli_is_section_completed($section, $course, $modinfo) {
    $completioninfo = new \completion_info($course);
    $cmcompleted = 0;
    $totalmods = 0;
    if (!empty($modinfo->sections[$section->section]) && $section->uservisible) {
        foreach ($modinfo->sections[$section->section] as $modnumber) {
            $mod = $modinfo->cms[$modnumber];
            if (!empty($mod)) {
                if ($mod->uservisible && $completioninfo->is_enabled($mod) != COMPLETION_TRACKING_NONE) {
                    $totalmods++;
                    //
                    $cmcompletionstate = $completioninfo->get_data($mod, true)->completionstate;
                    if ($cmcompletionstate == COMPLETION_COMPLETE || $cmcompletionstate == COMPLETION_COMPLETE_PASS ) {
                        $cmcompleted++;
                    }
                }
            }
        }
    }

    if ($totalmods) {
        $sectionprogress = round($cmcompleted / $totalmods * 100);
    } else {
        $sectionprogress = 0;
    }
    $sectionprogresscomp = ($sectionprogress == 100) ? true : false;

    $issectioncompleted = false;
    $issectioninprogress = false;
    $issectionnotyet = false;
    switch(true) {
        case $sectionprogress == 100:
            $issectioncompleted = true;
            break;
        case $sectionprogress > 0:
            $issectioninprogress = true;
            break;
        default:
            $issectionnotyet = true;
    }
    return [$issectioncompleted, $issectioninprogress, $issectionnotyet];
}

function theme_educli_get_teachers($roleids, $context)  {
    $users = array();
    foreach ($roleids as $roleid) {
        // Ignoring duplicated keys keeping the first user appearance.
        $users = $users + get_role_users($roleid, $context);
    }
    return $users;
}

/**
 * Get user enroll course progress info.
 */
function theme_educli_get_course_progress() {
    global $USER;
    $courses = enrol_get_my_courses();
    $actioncourse = 0;
    $progresscourse = 0;
    $coursesprogress = 0;
    if (!empty($courses)) {
        foreach ($courses as $course) {
            $progress = \core_completion\progress::get_course_progress_percentage($course);
            if (is_numeric($progress)) {
                if ($progress) {
                    $progresscourse ++;
                }
                $actioncourse ++;
            }
            $coursesprogress += $progress;
        }
    }
    $res = 0;
    if ($coursesprogress && $actioncourse && $progresscourse) {
        $totprogress = $coursesprogress * ($progresscourse * 100);
        $res = (int) $totprogress / ($actioncourse * 100);
    }
    return ($res) ? round($res) : 0;
}

/**
 * get course details.
 */
function theme_educli_get_course_details() {
    global $USER;
    $courses = enrol_get_my_courses();
    if (!empty($courses)) {
        foreach ($courses as $course) {
            $coursedata = [
                'coursename' => $course->fullname,
                'courseprogress' => theme_educli_get_course_progress()
            ];
        }
    }
    return $coursedata;
}


function theme_educli_output_fragment_refersh_assign_submission_table($args) {
    global $DB;
    list($course, $cm) = get_course_and_cm_from_cmid($args['cmid'], 'assign');
    $assign = new \assign($args['context'], $cm, $course);
    $gradingtable = new \assign_grading_table($assign, $args['perpage'], $args['filter'], 0, false);
    $searchquery = $args['searchquery'];
    if ($searchquery) {
        $gradingtable->sql->where .= " AND CONCAT(u.firstname, ' ', u.lastname) LIKE :liketext  ESCAPE '\\\'";
        $gradingtable->sql->params += ['liketext' => '%'.$searchquery.'%'];
    }
    $content = '';
    ob_start();
    $gradingtable->out($gradingtable->get_rows_per_page(), true);
    $content = ob_get_contents();
    ob_end_clean();
    return $content;
}