<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Renderers to align Moodle's HTML with that expected by Bootstrap
 *
 * @package theme_enlight
 * @copyright 2015 onwards LMSACE Dev Team (http://www.lmsace.com)
 * @author LMSACE Dev Team
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace theme_enlight\output;

use moodle_url;
use lang_string;
use html_writer;
use stdClass;
use core_course_category;



/**
 * This class has function for renderer primary menu and top course menus
 */
class core_renderer extends \theme_boost\output\core_renderer {

    /**
     * This function have the code to create the primary menu from the settings.
     * @return type
     */
    public function primarymenu() {
        global $CFG, $PAGE, $OUTPUT;
        require_once($CFG->dirroot . '/lib/outputrenderers.php');
        $custommenuitems = isset($PAGE->theme->settings->primarymenu) ? $PAGE->theme->settings->primarymenu : "";  
        $custommenu = new \custom_menu($custommenuitems, current_language());
        return $this->custom_menu_render($custommenu);
    }

    /*
     * This renders the bootstrap top menu.
     *
     * This renderer is needed to enable the Bootstrap style navigation.
     */
    protected function custom_menu_render(\custom_menu $menu) {

        global $CFG, $PAGE;
        // echo "ll";
        if (isset($PAGE->theme->settings->cmenuPosition)) {
            $cmenu_position = $PAGE->theme->settings->cmenuPosition;
        } else {
            $cmenu_position = "";
        }
        $cmenushow = theme_enlight_get_setting('cmenushow');
        $langs = get_string_manager()->get_list_of_translations();
        $haslangmenu = $this->lang_menu() != '';
        if (!$menu->has_children() ) {
            if ($cmenushow && $PAGE->pagelayout != 'login') {
                return $this->course_menu();
            }
            return '';
        }
        $content = [];
        $count = count($menu->get_children());
        foreach ($menu->get_children() as $key => $item) {
            $context = $item->export_for_template($this);
            $content[] = $this->render_from_template('core/custom_menu_item', $context);
        }
        if ($cmenushow && $PAGE->pagelayout != 'login') {
            $coursemenu = $this->course_menu();
            $cmenu_position = $cmenu_position - 1;
            if ($cmenu_position) {
                if ($cmenu_position >= count($content) ){
                    array_push($content, $coursemenu);
                } else {
                    $start = array_slice($content, 0, $cmenu_position);
                    $end = array_slice($content, $cmenu_position, count($content) - 1);
                    $content = $start;
                    array_push($content, $coursemenu);
                    // print_r($end);
                    $content = array_merge($content, array_values($end));
                }
            } else {
                array_unshift($content, $coursemenu);
            }
        }



        if ($this->is_headercentersplit() ) {
            return $this->split_header($content);
        } else {
            return implode('', $content);
        }
    }

    public function is_headercentersplit() {
        $headerstyle = theme_enlight_get_setting('headerStyle');
        if ( (int) $headerstyle == 5) {
            return true;
        }
        return false;
    }

    public function split_header($menuslist=array()) {
        if (!empty($menuslist)) {
            $count = count($menuslist);
            $menus = array_chunk($menuslist, ceil( $count / 2 ));
            $splitone = (isset($menus[0])) ? $menus[0] : $menus; 
            $splittwo = (isset($menus[1])) ? $menus[1] : []; 
            return ['leftside' => implode('', $splitone), 'rightside' => implode('', $splittwo) ];
        } 
        return '';
    }

   
    /**
     * This function contains the code for create and add the course menu into primary menu
     * items.
     * @return type
     */
    public function course_menu() {
        global $PAGE;
        $tcmenu = $this->top_course_menu();
        $cmenushow = theme_enlight_get_setting('cmenushow');
        $ccontent = '';
        if ($cmenushow) {
            $ccontent = '<li class="dropdown d-block d-lg-none course-link"><a class="nav-item nav-link" href="javascript:void(0);" class="dropdown-toggle" data-toggle="dropdown">';

            $ccontent .= get_string('courses').'<i class="fa fa-chevron-down"></i><span class="caretup"></span></a>';
            $ccontent .= $tcmenu['topmmenu'];
            $ccontent .= '</li><li class="d-none d-lg-block course-link" id="cr_link">
                  <a class="nav-item nav-link" href="'.new moodle_url('/course/index.php').'" >'.get_string('courses').'
                  <i class="fa fa-chevron-down"></i><span class="caretup"></span></a>'.$tcmenu['topcmenu'].'</li>';

        } else {
            $ccontent = '';
        }

        return $ccontent;

    }


    /* top course menu custom */
    /**
     * Course list for course menu on header
     * @return type
     */
    public function top_course_menu() {
        global $CFG , $DB;
        $list = core_course_category::make_categories_list();
        $mclist = array();

        $sql = "SELECT a.category , a.cnt from ( SELECT category , count(category) as cnt FROM {course}";
        $sql .= " WHERE category != '0' and visible = ? group by category ) as a order by a.cnt desc ";

        $params = array('1');
        $result = $DB->get_records_sql($sql, $params, 0, 0);
        shuffle($result);
        if ($result) {
            foreach ($result as $rowcat) {
                if ($result = $DB->record_exists('course_categories', array('id' => $rowcat->category))) {
                    $mclist[] = $rowcat->category;
                }
            }
        }
        $mclist1 = array_slice($mclist, 0, 4, true);
        $rcourseids = array();
        foreach ($mclist1 as $catid) {
            $categorylist = $DB->get_record('course_categories', array('id' => $catid));
            if ($categorylist) {
                if (\core_course_category::can_view_category($categorylist)) {
                    $coursecat = core_course_category::get($catid);
                    $cname = $coursecat->get_formatted_name();
                    $menuheader = '<div class="cols"><h6>'.$cname.'</h6><ul>'."\n";
                    $menufooter = '</ul></div>'."\n";
                    $href = $CFG->wwwroot.'/course/index.php?categoryid='.$catid;
                    $mmenuheader = '<li class="dropdown-submenu"><a href="'.$href.'" class="">'.$cname.'</a><ul class="dropdown-menu">';
                    $mmenufooter = '</ul></li>';
                    $menuitems = '';
                    $options = array();
                    $options['recursive'] = true;
                    $options['offset'] = 0;
                    $options['limit'] = 6;
                    $options['sort'] = array('sortorder' => 'ASC');
                    if ($ccc = $coursecat->get_courses($options)) {
                        foreach ($ccc as $cc) {
                            if ($cc->visible == "0" || $cc->id == "1") {
                                continue;
                            }
                            $courseurl = new moodle_url("/course/view.php", array("id" => $cc->id));
                            $menuitems .= '<li><a href="'.$courseurl.'">'.$cc->get_formatted_name().'</a></li>'."\n";
                        }
                        if (!empty($menuitems)) {
                            $rcourseids[$catid] = array("desk" => $menuheader.$menuitems.$menufooter,
                                "mobile" => $mmenuheader.$menuitems.$mmenufooter
                            );
                        }
                    }
                }
            }
        }
        $mcourseids = array_slice($rcourseids, 0, 4);
        $strcourse = $mstrcourse = '';
        foreach ($mcourseids as $ctid => $marr) {
            $strcourse .= $marr["desk"]."\n";
            $mstrcourse .= $marr["mobile"]."\n";
        }

        $courseaurl = new moodle_url('/course/index.php');
        if (!empty($strcourse)) {
            $topcmenu = '<div class="custom-dropdown-menu" id="cr_menu" style="display:none;">';
            $topcmenu .= '<div class="cols-wrap">'.$strcourse.'<div class="clearfix"></div></div></div>';
        } else {
            $topcmenu = "";
        }
        $topmmenu = '<ul class="dropdown-menu">'.$mstrcourse.'
        <li><a href="'.$courseaurl.'">
        '.get_string('viewall', 'theme_enlight').'</a></li></ul>';
        return compact('topcmenu', 'topmmenu');
    }


    /*
     * This code renders the custom menu items for the
     * bootstrap dropdown menu.
     */
    protected function render_custom_menu_item(\custom_menu_item $menunode, $level = 0 ) {
        static $submenucount = 0;
        $content = '';
        if ($menunode->has_children()) {
            if ($level == 1) {
                $class = 'dropdown';
            } else {
                $class = 'dropdown-submenu';
            }
            if ($menunode === $this->language) {
                $class .= ' langmenu';
            }
            $content = html_writer::start_tag('li', array('class' => $class));
            // If the child has menus render it as a sub menu.
            $submenucount++;
            if ($menunode->get_url() !== null) {
                $url = $menunode->get_url();
            } else {
                $url = '#cm_submenu_'.$submenucount;
            }
            $content .= html_writer::start_tag('a', array('href' => $url, 'class' => 'dropdown-toggle', 'data-toggle' => 'dropdown', 'title' => $menunode->get_title()));
            $content .= $menunode->get_text();
            if ($level == 1) {
                $content .= '<b class="caret"></b>';
            }
            $content .= '</a>';
            $content .= '<ul class="dropdown-menu">';
            foreach ($menunode->get_children() as $menunode) {
                $content .= $this->render_custom_menu_item($menunode, 0);
            }
            $content .= '</ul>';
        } else {
            // The node doesn't have children so produce a final menuitem.
            // Also, if the node's text matches '####', add a class so we can treat it as a divider.
            if (preg_match("/^#+$/", $menunode->get_text())) {
                // This is a divider.
                $content = '<li class="divider">&nbsp;</li>';
            } else {
                $content = '<li>';
                if ($menunode->get_url() !== null) {
                    $url = $menunode->get_url();
                } else {
                    $url = '#';
                }
                $content .= html_writer::link($url, $menunode->get_text(), array('title' => $menunode->get_title()));
                $content .= '</li>';
            }
        }
        return $content;
    }

    public function user_menu($user = null, $withlinks = null) {
        global $USER, $CFG;
        require_once($CFG->dirroot . '/user/lib.php');

        if (is_null($user)) {
            $user = $USER;
        }

        // Note: this behaviour is intended to match that of core_renderer::login_info,
        // but should not be considered to be good practice; layout options are
        // intended to be theme-specific. Please don't copy this snippet anywhere else.
        if (is_null($withlinks)) {
            $withlinks = empty($this->page->layout_options['nologinlinks']);
        }

        // Add a class for when $withlinks is false.
        $usermenuclasses = 'usermenu';
        if (!$withlinks) {
            $usermenuclasses .= ' withoutlinks';
        }

        $returnstr = "";

        // If during initial install, return the empty return string.
        if (during_initial_install()) {
            return $returnstr;
        }

        $loginpage = $this->is_login_page();
        $loginurl = get_login_url();
        // If not logged in, show the typical not-logged-in string.
        if (!isloggedin()) {
            $returnstr = ""; // get_string('loggedinnot', 'moodle');
            if (isset($CFG->registerauth) && $CFG->registerauth == 'email') {
                $returnstr .= " <a href='".new moodle_url('/login/signup.php')."'>". get_string('startsignup')."</a>";
            }
            if (!$loginpage) {
                if (isset($CFG->registerauth) && $CFG->registerauth == 'email') {
                    $returnstr .= " | ";
                }
                $returnstr .= " <a href=\"$loginurl\">" . get_string('login') . '</a>';
            }
            return html_writer::div(
                html_writer::span(
                    $returnstr,
                    'login'
                ),
                $usermenuclasses
            );

        }

        // If logged in as a guest user, show a string to that effect.
        if (isguestuser()) {
            $returnstr = get_string('loggedinasguest');
            if (!$loginpage && $withlinks) {
                $returnstr .= " (<a href=\"$loginurl\">".get_string('login').'</a>)';
            }

            return html_writer::div(
                html_writer::span(
                    $returnstr,
                    'login'
                ),
                $usermenuclasses
            );
        }

        // Get some navigation opts.
        $opts = user_get_user_navigation_info($user, $this->page);

        $avatarclasses = "avatars";
        $avatarcontents = html_writer::span($opts->metadata['useravatar'], 'avatar current');
        $usertextcontents = $opts->metadata['userfullname'];

        // Other user.
        if (!empty($opts->metadata['asotheruser'])) {
            $avatarcontents .= html_writer::span(
                $opts->metadata['realuseravatar'],
                'avatar realuser'
            );
            $usertextcontents = $opts->metadata['realuserfullname'];
            $usertextcontents .= html_writer::tag(
                'span',
                get_string(
                    'loggedinas',
                    'moodle',
                    html_writer::span(
                        $opts->metadata['userfullname'],
                        'value'
                    )
                ),
                array('class' => 'meta viewingas')
            );
        }

        // Role.
        if (!empty($opts->metadata['asotherrole'])) {
            $role = \core_text::strtolower(preg_replace('#[ ]+#', '-', trim($opts->metadata['rolename'])));
            $usertextcontents .= html_writer::span(
                $opts->metadata['rolename'],
                'meta role role-' . $role
            );
        }

        // User login failures.
        if (!empty($opts->metadata['userloginfail'])) {
            $usertextcontents .= html_writer::span(
                $opts->metadata['userloginfail'],
                'meta loginfailures'
            );
        }

        // MNet.
        if (!empty($opts->metadata['asmnetuser'])) {
            $mnet = strtolower(preg_replace('#[ ]+#', '-', trim($opts->metadata['mnetidprovidername'])));
            $usertextcontents .= html_writer::span(
                $opts->metadata['mnetidprovidername'],
                'meta mnet mnet-' . $mnet
            );
        }

        $returnstr .= html_writer::span(
            html_writer::span($usertextcontents, 'usertext mr-2') .
            html_writer::span($avatarcontents, $avatarclasses),
            'userbutton'
        );

        // Create a divider (well, a filler).
        $divider = new \action_menu_filler();
        $divider->primary = false;

        $am = new \action_menu();
        $am->set_menu_trigger(
            $returnstr
        );
        $am->set_alignment(\action_menu::TR, \action_menu::BR);
        $am->set_nowrap_on_items();
        if ($withlinks) {
            $navitemcount = count($opts->navitems);
            $idx = 0;
            foreach ($opts->navitems as $key => $value) {

                switch ($value->itemtype) {
                    case 'divider':
                        // If the nav item is a divider, add one and skip link processing.
                        $am->add($divider);
                        break;

                    case 'invalid':
                        // Silently skip invalid entries (should we post a notification?).
                        break;

                    case 'link':
                        // Process this as a link item.
                        $pix = null;
                        if (isset($value->pix) && !empty($value->pix)) {
                            $pix = new \pix_icon($value->pix, $value->title, null, array('class' => 'iconsmall'));
                        } else if (isset($value->imgsrc) && !empty($value->imgsrc)) {
                            $value->title = html_writer::img(
                                $value->imgsrc,
                                $value->title,
                                array('class' => 'iconsmall')
                            ) . $value->title;
                        }

                        $al = new \action_menu_link_secondary(
                            $value->url,
                            $pix,
                            $value->title,
                            array('class' => 'icon')
                        );
                        if (!empty($value->titleidentifier)) {
                            $al->attributes['data-title'] = $value->titleidentifier;
                        }
                        $am->add($al);
                        break;
                }

                $idx++;

                // Add dividers after the first item and before the last item.
                if ($idx == 1 || $idx == $navitemcount - 1) {
                    $am->add($divider);
                }
            }
        }

        return html_writer::div(
            $this->render($am),
            $usermenuclasses
        );
    }
    public function render_login(\core_auth\output\login $form) {
        global $SITE, $PAGE, $CFG;

        $context = $form->export_for_template($this);

        // Override because rendering is not supported in template yet.
        $context->cookieshelpiconformatted = $this->help_icon('cookiesenabled');
        $context->errorformatted = $this->error_text($context->error);
        $url = $this->get_logo_url();
        if ($url) {
            $url = $url->out(false);
        }
        $context->logourl = ($url) ? $url : theme_enlight_get_logo_url() ;
        $context->sitename = format_string($SITE->fullname, true,
            ['context' => \context_course::instance(SITEID), "escape" => false]);

        $loginpageType = $PAGE->theme->settings->loginpageType;
        $context->loginheader = get_string('loginheader', 'theme_enlight');
        $context->siteurl = new \moodle_url('/?redirect=0');
        // echo $loginpageType;
        if ($loginpageType == "1") {
            $context->slideimages = theme_enlight_get_login_slideimages();
            $context->show_slideimages = (!empty($context->slideimages)) ? true : false;
            return $this->render_from_template('theme_enlight/loginform', $context);
        } else {
            return $this->render_from_template('core/loginform', $context);
        }

    }

    public function header_logo() {
        global $CFG, $SITE;

        $logourl = theme_enlight_get_logo_url();
        $classes = 'navbar-brand';
        $classes .= (!empty($logourl)) ? ' has-logo ' : ' d-none d-sm-block ';
        $output = html_writer::start_tag('a', ['href' => $CFG->wwwroot, 'class' => $classes ]);
        if (!empty($logourl)) {
            $output .= html_writer::start_tag('span', ['class' => 'logo']);
            $output .= html_writer::empty_tag('img', ['src' => $logourl, 'alt' => $SITE->fullname ]);
           $output .=  html_writer::end_tag('span');
        } else {
            $output .= html_writer::tag('span', $SITE->fullname, ['class' => 'site-name d-none d-sm-block']);
        }
        $output .= html_writer::end_tag('a');

        return $output;
    }

            /**
     * Wrapper for header elements.
     *
     * @return string HTML to display the main header.
     */
    public function full_header() {
        global $PAGE, $USER, $DB;
        if ($this->page->include_region_main_settings_in_header_actions() &&
                !$this->page->blocks->is_block_present('settings')) {

            // Only include the region main settings if the page has requested it and it doesn't already have
            // the settings block on it. The region main settings are included in the settings block and
            // duplicating the content causes behat failures.
            
            $this->page->add_header_action(html_writer::div(
                $this->region_main_settings_menu(),
                'd-print-none',
                ['id' => 'region-main-settings-menu']
            ));
        }
        
        //add ediweser course format plugin 
        $course = $PAGE->course;
        $courseformat = course_get_format($course);
        $imgurl = '';
        $remuiformat = false;

        if($PAGE->pagelayout == 'course') {
            if ($courseformat->get_format() == 'remuiformat'){
                $remuiformat = true;
                // add image url 
                $settings = $courseformat->get_settings();
                $config = \format_remuiformat\course_format_data_common_trait::getinstance();;
                $imgurl = $config->display_file($settings['remuicourseimage_filemanager']);
              /*  echo $imgurl;
                exit();*/
                //add blocks
                $enrollusers = enrol_get_course_users($course->id);
                $sql = "SELECT * FROM {course_completions} WHERE course=:courseid AND timecompleted IS NOT NULL ";
                $completedusers = $DB->get_records_sql($sql,['courseid' => $course->id]);
                $activites = \course_modinfo::get_array_of_activities($course);
                $context = \context_course::instance($course->id);
                $teachers = get_role_users(3 , $context);
            }
        }
        
        $header = new stdClass();
        $header->settingsmenu = $this->context_header_settings_menu();
        $header->contextheader = $this->context_header();
        $header->hasnavbar = empty($this->page->layout_options['nonavbar']);
        $header->navbar = $this->navbar();
        $header->pageheadingbutton = $this->page_heading_button();
        $header->courseheader = $this->course_header();
        $header->headeractions = $this->page->get_header_actions();
        // image url added 
        $header->remuiformatimgurl = $imgurl;
        $header->enrollusers = isset($enrollusers) ? count($enrollusers) : '';
        $header->completedusers = isset($completedusers) ? count($completedusers) : '';
        $header->activites =  isset($activites) ? count($activites) : '';
        $header->teachers = isset($teachers) ? count($teachers) : '';
        $header->remuiformat = $remuiformat;
       /* print_object($header);
        exit();*/
        return $this->render_from_template('theme_enlight/core/full_header', $header);
    }

} // Here the course renderer fucntion closed.

?>