<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * A drawer based layout for the ludemy theme.
 *
 * @package   theme_ludemy
 * @copyright  2023 LMSACE Dev Team <lmsace.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace theme_ludemy;

use core\navigation\output\primary;
use context_course;


defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot."/theme/ludemy/lib.php");


class build_template {

    /**
     * Get custom regions data.
     */
    public function customregions() {
        global $OUTPUT, $PAGE;
        $regions = explode(',', get_config('theme_ludemy', 'customregions'));
        $template = [];
        foreach ($regions as $name) {

            if (!has_capability('theme/ludemy:viewregion'.$name, $PAGE->context)) {
                $template[$name] = ['hasblocks' => false];
                continue;
            }

            $regionhtml = $OUTPUT->blocks($name);
            $blockbutton = (has_capability('theme/ludemy:editregion'.$name, $PAGE->context)) ?
                             $OUTPUT->addblockbutton($name) : '';
            $blockbutton = $OUTPUT->addblockbutton($name);
            $template[$name] = [
                'hasblocks' => (strpos($regionhtml, 'data-block=') !== false || !empty($blockbutton)),
                'regionhtml' => $regionhtml,
                'addblockbutton' => $blockbutton
            ];
        }
        return $template;
    }

    /**
     * Load the base config data for the all layouts.
     */
    public function base_config() {
        global $CFG, $OUTPUT, $PAGE, $SITE;
        require_once($CFG->libdir . '/behat/lib.php');
        require_once($CFG->dirroot . '/course/lib.php');

        // Add block button in editing mode.
        $addblockbutton = $OUTPUT->addblockbutton();

        if (isloggedin()) {
            $courseindexopen = (get_user_preferences('drawer-open-index', true) == true);
            $blockdraweropen = (get_user_preferences('drawer-open-block') == true);
        } else {
            $courseindexopen = false;
            $blockdraweropen = false;
        }

        if (defined('BEHAT_SITE_RUNNING')) {
            $blockdraweropen = true;
        }

        $extraclasses = ['uses-drawers'];
        if ($courseindexopen) {
            $extraclasses[] = 'drawer-open-index';
        }

        $blockshtml = $OUTPUT->blocks('side-pre');
        $hasblocks = (strpos($blockshtml, 'data-block=') !== false || !empty($addblockbutton));
        if (!$hasblocks) {
            $blockdraweropen = false;
        }
        $courseindex = core_course_drawer();
        if (!$courseindex) {
            $courseindexopen = false;
        }

        $bodyattributes = $OUTPUT->body_attributes($extraclasses);
        $forceblockdraweropen = $OUTPUT->firstview_fakeblocks();

        $secondarynavigation = false;
        $overflow = '';
        if ($PAGE->has_secondary_navigation()) {
            $tablistnav = $PAGE->has_tablist_secondary_navigation();
            $moremenu = new \core\navigation\output\more_menu($PAGE->secondarynav, 'nav-tabs', true, $tablistnav);
            $secondarynavigation = $moremenu->export_for_template($OUTPUT);
            $overflowdata = $PAGE->secondarynav->get_overflow_menu_data();
            if (!is_null($overflowdata)) {
                $overflow = $overflowdata->export_for_template($OUTPUT);
            }
        }

        $primary = new primary($PAGE);
        $renderer = $PAGE->get_renderer('core');
        $primarymenu = $primary->export_for_template($renderer);
        $buildregionmainsettings = !$PAGE->include_region_main_settings_in_header_actions() && !$PAGE->has_secondary_navigation();
        // If the settings menu will be included in the header then don't add it here.
        $regionmainsettingsmenu = $buildregionmainsettings ? $OUTPUT->region_main_settings_menu() : false;

        $header = $PAGE->activityheader;
        $headercontent = $header->export_for_template($renderer);

        $templatecontext = [
            'sitename' => format_string($SITE->shortname, true, ['context' => context_course::instance(SITEID), "escape" => false]),
            'output' => $OUTPUT,
            'sidepreblocks' => $blockshtml,
            'hasblocks' => $hasblocks,
            'bodyattributes' => $bodyattributes,
            'courseindexopen' => $courseindexopen,
            'blockdraweropen' => $blockdraweropen,
            'courseindex' => $courseindex,
            'primarymoremenu' => $primarymenu['moremenu'],
            'secondarymoremenu' => $secondarynavigation ?: false,
            'mobileprimarynav' => $primarymenu['mobileprimarynav'],
            'usermenu' => $primarymenu['user'],
            'langmenu' => $primarymenu['lang'],
            'forceblockdraweropen' => $forceblockdraweropen,
            'regionmainsettingsmenu' => $regionmainsettingsmenu,
            'hasregionmainsettingsmenu' => !empty($regionmainsettingsmenu),
            'overflow' => $overflow,
            'headercontent' => $headercontent,
            'addblockbutton' => $addblockbutton,
            'customregions' => $this->customregions(),
        ];

        return $templatecontext;
    }


    /**
     * List of available theme layout config.
     *
     * @param string $layout
     * @return void
     */
    public function layout_config(string $layout = '', string $baseconfig = '',) {

        // Can customize the base config template data.
        if (!empty($baseconfig) && method_exists($this, $baseconfig.'_config')) {
            $config = $this->{$baseconfig.'_config'}();
        } else {
            $config = $this->base_config();
        }

        // echo $layout;exit;
        if (!empty($layout) && method_exists($this, $layout.'_config')) {

            $layoutconfig = $this->{$layout.'_config'}();
            $config = array_merge((array) $config, $layoutconfig);
        }
        return $config;
    }
}